@@ -66,6 +66,10 @@
 
 #define LOGLEVEL_TAX LOG_DEBUG
 
+/* When setting rates, we accept negative balance if we have a lot of
+ * gold reserves. This is how long time gold reserves should last */
+#define AI_GOLD_RESERVE_MIN_TURNS 35
+
 /**************************************************************************
  handle spaceship related stuff
 **************************************************************************/
@@ -102,12 +106,14 @@ static void ai_manage_taxes(struct player *pplayer)
   int can_celebrate = 0, total_cities = 0;
   int trade = 0; /* total amount of trade generated */
   int expenses = 0; /* total amount of gold upkeep */
+  int min_reserve = ai_gold_reserve(pplayer);
+  bool refill_coffers = pplayer->economic.gold < min_reserve;
 
   if (!game.info.changable_tax) {
     return; /* This ruleset does not support changing tax rates. */
   }
 
-  if (government_of_player(pplayer) == game.government_when_anarchy) {
+  if (government_of_player(pplayer) == game.government_during_revolution) {
     return; /* This government does not support changing tax rates. */
   }
 
@@ -117,6 +123,14 @@ static void ai_manage_taxes(struct player *pplayer)
     expenses += pcity->usage[O_GOLD];
   } city_list_iterate_end;
 
+  if (game.info.gold_upkeep_style > 0) {
+    /* Account for units with gold upkeep paid for by the nation.
+     * (game.info.gold_upkeep_style = 1,2) */
+    unit_list_iterate(pplayer->units, punit) {
+      expenses += punit->upkeep[O_GOLD];
+    } unit_list_iterate_end;
+  }
+
   /* Find minimum tax rate which gives us a positive balance. We assume
    * that we want science most and luxuries least here, and reverse or 
    * modify this assumption later. on */
@@ -127,7 +141,8 @@ static void ai_manage_taxes(struct player *pplayer)
   pplayer->economic.luxury = (100 - pplayer->economic.science
                              - pplayer->economic.tax); /* Spillover */
 
-  /* Now find the minimum tax with positive balance */
+  /* Now find the minimum tax with positive balance
+   * Negative balance is acceptable if we have a lot of gold. */
   while(pplayer->economic.tax < maxrate
         && (pplayer->economic.science > 0
             || pplayer->economic.luxury > 0)) {
@@ -142,7 +157,11 @@ static void ai_manage_taxes(struct player *pplayer)
     rates[TAX] = 100 - rates[SCIENCE] - rates[LUXURY];
     distribute(trade, 3, rates, result);
 
-    if (expenses - result[TAX] > 0) {
+    if (expenses - result[TAX] > 0
+        && (refill_coffers
+            || expenses - result[TAX] >
+               pplayer->economic.gold / AI_GOLD_RESERVE_MIN_TURNS)) {
+      /* Clearly negative balance. Unacceptable */
       pplayer->economic.tax += 10;
       if (pplayer->economic.luxury > 0) {
         pplayer->economic.luxury -= 10;
@@ -150,8 +169,9 @@ static void ai_manage_taxes(struct player *pplayer)
         pplayer->economic.science -= 10;
       }
     } else {
-      /* Ok, got positive balance */
-      if (pplayer->economic.gold < ai_gold_reserve(pplayer)) {
+      /* Ok, got positive balance
+       * Or just slightly negative, if we can afford that for a while */
+      if (refill_coffers) {
         /* Need to refill coffers, increase tax a bit */
         pplayer->economic.tax += 10;
         if (pplayer->economic.luxury > 0) {
@@ -200,10 +220,10 @@ static void ai_manage_taxes(struct player *pplayer)
           && pcity->surplus[O_FOOD] > 0
           && pcity->size >= game.info.celebratesize
 	  && city_can_grow_to(pcity, pcity->size + 1)) {
-        pcity->ai.celebrate = TRUE;
+        pcity->ai->celebrate = TRUE;
         can_celebrate++;
       } else {
-        pcity->ai.celebrate = FALSE;
+        pcity->ai->celebrate = FALSE;
       }
     } city_list_iterate_end;
     /* If more than half our cities can celebrate, go for it! */
@@ -211,12 +231,12 @@ static void ai_manage_taxes(struct player *pplayer)
     if (celebrate) {
       freelog(LOGLEVEL_TAX, "*** %s CELEBRATES! ***", player_name(pplayer));
       city_list_iterate(pplayer->cities, pcity) {
-        if (pcity->ai.celebrate == TRUE) {
+        if (pcity->ai->celebrate == TRUE) {
           freelog(LOGLEVEL_TAX, "setting %s to celebrate", city_name(pcity));
           cm_query_result(pcity, &cmp, &cmr);
           if (cmr.found_a_valid) {
             apply_cmresult_to_city(pcity, &cmr);
-            generic_city_refresh(pcity, TRUE, NULL);
+            city_refresh_from_main_map(pcity, TRUE);
             if (!city_happy(pcity)) {
               CITY_LOG(LOG_ERROR, pcity, "is NOT happy when it should be!");
             }
@@ -227,11 +247,10 @@ static void ai_manage_taxes(struct player *pplayer)
       pplayer->economic.luxury = luxrate;
       pplayer->economic.science = scirate;
       city_list_iterate(pplayer->cities, pcity) {
-        /* KLUDGE: Must refresh to restore the original values which 
-         * were clobbered in cm_query_result, after the tax rates 
-         * were changed.  This is because the cm_query_result() calls
-         * generic_city_refresh(). */
-        generic_city_refresh(pcity, TRUE, NULL);
+        /* KLUDGE: Must refresh to restore the original values which
+         * were clobbered in cm_query_result(), after the tax rates
+         * were changed. */
+        city_refresh_from_main_map(pcity, TRUE);
       } city_list_iterate_end;
     }
   }
@@ -292,7 +311,7 @@ void ai_best_government(struct player *pplayer)
       int val = 0;
       int dist;
 
-      if (gov == game.government_when_anarchy) {
+      if (gov == game.government_during_revolution) {
         continue; /* pointless */
       }
       if (gov->ai.better
@@ -327,19 +346,17 @@ void ai_best_government(struct player *pplayer)
       /* FIXME: handle reqs other than technologies. */
       dist = 0;
       requirement_vector_iterate(&gov->reqs, preq) {
-	if (preq->source.type == REQ_TECH) {
+	if (VUT_ADVANCE == preq->source.kind) {
 	  dist += MAX(1, num_unknown_techs_for_goal(pplayer,
-						    preq->source.value.tech));
+						    advance_number(preq->source.value.advance)));
 	}
       } requirement_vector_iterate_end;
       val = amortize(val, dist);
-      ai->government_want[gov->index] = val; /* Save want */
+      ai->government_want[government_index(gov)] = val; /* Save want */
     } government_iterate_end;
     /* Now reset our gov to it's real state. */
     pplayer->government = current_gov;
     city_list_iterate(pplayer->cities, acity) {
-      /* This isn't strictly necessary since it's done in aaw. */
-      generic_city_refresh(acity, TRUE, NULL);
       auto_arrange_workers(acity);
     } city_list_iterate_end;
     ai->govt_reeval = CLIP(5, city_list_size(pplayer->cities), 20);
@@ -348,20 +365,21 @@ void ai_best_government(struct player *pplayer)
 
   /* Figure out which government is the best for us this turn. */
   government_iterate(gov) {
-    if (ai->government_want[gov->index] > best_val 
+    int gi = government_index(gov);
+    if (ai->government_want[gi] > best_val 
         && can_change_to_government(pplayer, gov)) {
-      best_val = ai->government_want[gov->index];
+      best_val = ai->government_want[gi];
       ai->goal.revolution = gov;
     }
-    if (ai->government_want[gov->index] > ai->goal.govt.val) {
+    if (ai->government_want[gi] > ai->goal.govt.val) {
       ai->goal.govt.gov = gov;
-      ai->goal.govt.val = ai->government_want[gov->index];
+      ai->goal.govt.val = ai->government_want[gi];
 
       /* FIXME: handle reqs other than technologies. */
       ai->goal.govt.req = A_NONE;
       requirement_vector_iterate(&gov->reqs, preq) {
-	if (preq->source.type == REQ_TECH) {
-	  ai->goal.govt.req = preq->source.value.tech;
+	if (VUT_ADVANCE == preq->source.kind) {
+	  ai->goal.govt.req = advance_number(preq->source.value.advance);
 	  break;
 	}
       } requirement_vector_iterate_end;
@@ -389,7 +407,7 @@ static void ai_manage_government(struct player *pplayer)
 
   /* Crank up tech want */
   if (ai->goal.govt.req == A_UNSET
-      || get_invention(pplayer, ai->goal.govt.req) == TECH_KNOWN) {
+      || player_invention_state(pplayer, ai->goal.govt.req) == TECH_KNOWN) {
     return; /* already got it! */
   } else if (ai->goal.govt.val > 0) {
     /* We have few cities in the beginning, compensate for this to ensure
@@ -402,8 +420,8 @@ static void ai_manage_government(struct player *pplayer)
        * We want something better pretty soon! */
       want += 25 * game.info.turn;
     }
-    pplayer->ai.tech_want[ai->goal.govt.req] += want;
-    TECH_LOG(LOG_DEBUG, pplayer, ai->goal.govt.req,
+    pplayer->ai_data.tech_want[ai->goal.govt.req] += want;
+    TECH_LOG(LOG_DEBUG, pplayer, advance_by_number(ai->goal.govt.req), 
              "ai_manage_government() + %d for %s",
              want,
              government_rule_name(ai->goal.govt.gov));
@@ -453,15 +471,6 @@ void ai_do_last_activities(struct player *pplayer)
   ai_manage_tech(pplayer); 
   TIMING_LOG(AIT_TECH, TIMER_STOP);
   ai_manage_spaceship(pplayer);
-  ai_data_phase_done(pplayer);
 
   TIMING_LOG(AIT_ALL, TIMER_STOP);
 }
-
-/**************************************************************************
-  ...
-**************************************************************************/
-bool is_unit_choice_type(enum choice_type type)
-{
-   return type == CT_NONMIL || type == CT_ATTACKER || type == CT_DEFENDER;
-}
