@@ -58,32 +58,33 @@
 #include <winsock.h>
 #endif
 
-#ifdef GGZ_GTK
-#  include <ggz-embed.h>
-#endif
-
+/* utility */
 #include "capstr.h"
 #include "dataio.h"
 #include "fcintl.h"
-#include "game.h"
 #include "hash.h"
 #include "log.h"
 #include "mem.h"
 #include "netintf.h"
-#include "packets.h"
 #include "registry.h"
 #include "support.h"
+
+/* common */
+#include "game.h"
+#include "packets.h"
 #include "version.h"
 
+/* client */
 #include "agents.h"
 #include "attribute.h"
 #include "chatline_g.h"
-#include "civclient.h"
+#include "client_main.h"
 #include "climisc.h"
 #include "connectdlg_common.h"
 #include "connectdlg_g.h"
 #include "dialogs_g.h"		/* popdown_races_dialog() */
 #include "ggzclient.h"
+#include "ggz_g.h"
 #include "gui_main_g.h"		/* add_net_input(), remove_net_input() */
 #include "mapview_common.h"	/* unqueue_mapview_update */
 #include "menu_g.h"
@@ -96,8 +97,14 @@
 
 #include "clinet.h"
 
-struct connection aconnection;
-static union my_sockaddr server_addr;
+/* In autoconnect mode, try to connect to once a second */
+#define AUTOCONNECT_INTERVAL		500
+
+/* In autoconnect mode, try to connect 100 times */
+#define MAX_AUTOCONNECT_ATTEMPTS	100
+
+static union fc_sockaddr names[2];
+static int name_count;
 
 /*************************************************************************
   Close socket and cleanup.  This one doesn't print a message, so should
@@ -109,27 +116,16 @@ static void close_socket_nomessage(struct connection *pc)
     remove_ggz_input();
   }
   if (in_ggz) {
-#ifdef GGZ_GTK
-    ggz_embed_leave_table();
-#endif
+    gui_ggz_embed_leave_table();
   }
   connection_common_close(pc);
   remove_net_input();
   popdown_races_dialog(); 
   close_connection_dialog();
 
-  if (C_S_PREPARING == client_state()) {
-    if (!with_ggz) {
-      set_client_page(in_ggz ? PAGE_GGZ : PAGE_MAIN);
-    }
-  }
-
   reports_force_thaw();
   
-  set_client_state(C_S_PREPARING);
-  agents_disconnect();
-  update_menus();
-  client_remove_all_cli_conn();
+  set_client_state(C_S_DISCONNECTED);
 }
 
 /**************************************************************************
@@ -140,40 +136,23 @@ static void close_socket_callback(struct connection *pc)
   close_socket_nomessage(pc);
   /* If we lost connection to the internal server - kill him */
   client_kill_server(TRUE);
-  append_output_window(_("Lost connection to server!"));
-  freelog(LOG_NORMAL, "lost connection to server");
+  freelog(LOG_ERROR, "Lost connection to server!");
+  output_window_append(ftc_client, _("Lost connection to server!"));
   if (with_ggz) {
     client_exit();
   }
 }
 
-/**************************************************************************
-  Connect to a civserver instance -- or at least try to.  On success,
-  return 0; on failure, put an error message in ERRBUF and return -1.
-**************************************************************************/
-int connect_to_server(const char *username, const char *hostname, int port,
-		      char *errbuf, int errbufsize)
-{
-  if (get_server_address(hostname, port, errbuf, errbufsize) != 0) {
-    return -1;
-  }
-
-  if (try_to_connect(username, errbuf, errbufsize) != 0) {
-    return -1;
-  }
-  return 0;
-}
-
 /**************************************************************************
   Get ready to [try to] connect to a server:
    - translate HOSTNAME and PORT (with defaults of "localhost" and
      DEFAULT_SOCK_PORT respectively) to a raw IP address and port number, and
-     store them in the `server_addr' variable
+     store them in the `names' variable
    - return 0 on success
      or put an error message in ERRBUF and return -1 on failure
 **************************************************************************/
-int get_server_address(const char *hostname, int port, char *errbuf,
-		       int errbufsize)
+static int get_server_address(const char *hostname, int port,
+                              char *errbuf, int errbufsize)
 {
   if (port == 0)
     port = DEFAULT_SOCK_PORT;
@@ -182,17 +161,29 @@ int get_server_address(const char *hostname, int port, char *errbuf,
   if (!hostname)
     hostname = "localhost";
 
-  if (!net_lookup_service(hostname, port, &server_addr)) {
+  if (!net_lookup_service(hostname, port, &names[0], FALSE)) {
     (void) mystrlcpy(errbuf, _("Failed looking up host."), errbufsize);
     return -1;
   }
+  name_count = 1;
+#ifdef IPV6_SUPPORT
+  if (names[0].saddr.sa_family == AF_INET6) {
+    /* net_lookup_service() prefers IPv6 address.
+     * Check if there is also IPv4 address.
+     * TODO: This would be easier using getaddrinfo() */
+    if (net_lookup_service(hostname, port,
+                           &names[1], TRUE /* force IPv4 */)) {
+      name_count = 2;
+    }
+  }
+#endif
 
   return 0;
 }
 
 /**************************************************************************
   Try to connect to a server (get_server_address() must be called first!):
-   - try to create a TCP socket and connect it to `server_addr'
+   - try to create a TCP socket and connect it to `names'
    - if successful:
 	  - start monitoring the socket for packets from the server
 	  - send a "login request" packet to the server
@@ -201,26 +192,41 @@ int get_server_address(const char *hostname, int port, char *errbuf,
      message in ERRBUF and return the Unix error code (ie., errno, which
      will be non-zero).
 **************************************************************************/
-int try_to_connect(const char *username, char *errbuf, int errbufsize)
+static int try_to_connect(const char *username, char *errbuf, int errbufsize)
 {
+  int i;
+  int sock = -1;
+
   close_socket_set_callback(close_socket_callback);
 
   /* connection in progress? wait. */
-  if (aconnection.used) {
+  if (client.conn.used) {
     (void) mystrlcpy(errbuf, _("Connection in progress."), errbufsize);
     return -1;
   }
-  
-  if ((aconnection.sock = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
-    (void) mystrlcpy(errbuf, mystrerror(), errbufsize);
-    return -1;
+
+  /* Try all (IPv4, IPv6, ...) addresses until we have a connection. */
+  sock = -1;
+  for (i = 0; i < name_count; i++) {
+    if ((sock = socket(names[i].saddr.sa_family, SOCK_STREAM, 0)) == -1) {
+      /* Probably EAFNOSUPPORT or EPROTONOSUPPORT. */
+      continue;
+    }
+
+    if (fc_connect(sock, &names[i].saddr,
+                   sockaddr_size(&names[i])) == -1) {
+      fc_closesocket(sock);
+      sock = -1;
+      continue;
+    } else {
+      /* We have a connection! */
+      break;
+    }
   }
 
-  if (my_connect(aconnection.sock, &server_addr.sockaddr,
-      sizeof(server_addr)) == -1) {
-    (void) mystrlcpy(errbuf, mystrerror(), errbufsize);
-    my_closesocket(aconnection.sock);
-    aconnection.sock = -1;
+  client.conn.sock = sock;
+  if (client.conn.sock == -1) {
+    (void) mystrlcpy(errbuf, fc_strerror(fc_get_errno()), errbufsize);
 #ifdef HAVE_WINSOCK
     return -1;
 #else
@@ -228,11 +234,29 @@ int try_to_connect(const char *username, char *errbuf, int errbufsize)
 #endif
   }
 
-  make_connection(aconnection.sock, username);
+  make_connection(client.conn.sock, username);
 
   return 0;
 }
- 
+
+/**************************************************************************
+  Connect to a freeciv-server instance -- or at least try to.  On success,
+  return 0; on failure, put an error message in ERRBUF and return -1.
+**************************************************************************/
+int connect_to_server(const char *username, const char *hostname, int port,
+		      char *errbuf, int errbufsize)
+{
+  if (0 != get_server_address(hostname, port, errbuf, errbufsize)) {
+    return -1;
+  }
+
+  if (0 != try_to_connect(username, errbuf, errbufsize)) {
+    return -1;
+  }
+
+  return 0;
+}
+
 /**************************************************************************
   Called after a connection is completed (e.g., in try_to_connect).
 **************************************************************************/
@@ -240,17 +264,16 @@ void make_connection(int socket, const char *username)
 {
   struct packet_server_join_req req;
 
-  connection_common_init(&aconnection);
-  aconnection.sock = socket;
-  aconnection.is_server = FALSE;
-  aconnection.client.last_request_id_used = 0;
-  aconnection.client.last_processed_request_id_seen = 0;
-  aconnection.client.request_id_of_currently_handled_packet = 0;
-  aconnection.incoming_packet_notify = notify_about_incoming_packet;
-  aconnection.outgoing_packet_notify = notify_about_outgoing_packet;
+  connection_common_init(&client.conn);
+  client.conn.sock = socket;
+  client.conn.client.last_request_id_used = 0;
+  client.conn.client.last_processed_request_id_seen = 0;
+  client.conn.client.request_id_of_currently_handled_packet = 0;
+  client.conn.incoming_packet_notify = notify_about_incoming_packet;
+  client.conn.outgoing_packet_notify = notify_about_outgoing_packet;
 
   /* call gui-dependent stuff in gui_main.c */
-  add_net_input(aconnection.sock);
+  add_net_input(client.conn.sock);
 
   /* now send join_request package */
 
@@ -261,7 +284,7 @@ void make_connection(int socket, const char *username)
   sz_strlcpy(req.capability, our_capability);
   sz_strlcpy(req.username, username);
   
-  send_packet_server_join_req(&aconnection, &req);
+  send_packet_server_join_req(&client.conn, &req);
 }
 
 /**************************************************************************
@@ -269,7 +292,7 @@ void make_connection(int socket, const char *username)
 **************************************************************************/
 void disconnect_from_server(void)
 {
-  const bool force = !aconnection.used;
+  const bool force = !client.conn.used;
 
   attribute_flush();
   /* If it's internal server - kill him 
@@ -277,16 +300,16 @@ void disconnect_from_server(void)
   if (!force) {
     client_kill_server(FALSE);
   }
-  close_socket_nomessage(&aconnection);
+  close_socket_nomessage(&client.conn);
   if (force) {
     client_kill_server(TRUE);
   }
-  append_output_window(_("Disconnected from server."));
+  output_window_append(ftc_client, _("Disconnected from server."));
   if (with_ggz) {
     client_exit();
   }
   if (save_options_on_exit) {
-    save_options();
+    options_save();
   }
 }  
 
@@ -323,11 +346,11 @@ static int read_from_connection(struct connection *pc, bool block)
       MY_FD_ZERO(&writefs);
       FD_SET(socket_fd, &writefs);
       n =
-	  my_select(socket_fd + 1, &readfs, &writefs, &exceptfs,
+	  fc_select(socket_fd + 1, &readfs, &writefs, &exceptfs,
 		    block ? NULL : &tv);
     } else {
       n =
-	  my_select(socket_fd + 1, &readfs, NULL, &exceptfs,
+	  fc_select(socket_fd + 1, &readfs, NULL, &exceptfs,
 		    block ? NULL : &tv);
     }
 
@@ -345,8 +368,8 @@ static int read_from_connection(struct connection *pc, bool block)
 	continue;
       }
 
-      freelog(LOG_NORMAL, "error in select() return=%d errno=%d (%s)",
-	      n, errno, mystrerror());
+      freelog(LOG_ERROR, "select() return=%d errno=%d (%s)",
+	      n, errno, fc_strerror(fc_get_errno()));
       return -1;
     }
 
@@ -370,16 +393,16 @@ static int read_from_connection(struct connection *pc, bool block)
 **************************************************************************/
 void input_from_server(int fd)
 {
-  assert(fd == aconnection.sock);
+  assert(fd == client.conn.sock);
 
-  if (read_from_connection(&aconnection, FALSE) >= 0) {
+  if (read_from_connection(&client.conn, FALSE) >= 0) {
     enum packet_type type;
 
     reports_freeze();
     agents_freeze_hint();
     while (TRUE) {
       bool result;
-      void *packet = get_packet_from_connection(&aconnection,
+      void *packet = get_packet_from_connection(&client.conn,
 						&type, &result);
 
       if (result) {
@@ -394,7 +417,7 @@ void input_from_server(int fd)
     agents_thaw_hint();
     reports_thaw();
   } else {
-    close_socket_callback(&aconnection);
+    close_socket_callback(&client.conn);
   }
 }
 
@@ -408,19 +431,19 @@ void input_from_server_till_request_got_processed(int fd,
 						  int expected_request_id)
 {
   assert(expected_request_id);
-  assert(fd == aconnection.sock);
+  assert(fd == client.conn.sock);
 
   freelog(LOG_DEBUG,
 	  "input_from_server_till_request_got_processed("
 	  "expected_request_id=%d)", expected_request_id);
 
   while (TRUE) {
-    if (read_from_connection(&aconnection, TRUE) >= 0) {
+    if (read_from_connection(&client.conn, TRUE) >= 0) {
       enum packet_type type;
 
       while (TRUE) {
 	bool result;
-	void *packet = get_packet_from_connection(&aconnection,
+	void *packet = get_packet_from_connection(&client.conn,
 						  &type, &result);
 	if (!result) {
 	  assert(packet == NULL);
@@ -434,8 +457,8 @@ void input_from_server_till_request_got_processed(int fd,
 	if (type == PACKET_PROCESSING_FINISHED) {
 	  freelog(LOG_DEBUG, "ifstrgp: expect=%d, seen=%d",
 		  expected_request_id,
-		  aconnection.client.last_processed_request_id_seen);
-	  if (aconnection.client.last_processed_request_id_seen >=
+		  client.conn.client.last_processed_request_id_seen);
+	  if (client.conn.client.last_processed_request_id_seen >=
 	      expected_request_id) {
 	    freelog(LOG_DEBUG, "ifstrgp: got it; returning");
 	    return;
@@ -443,7 +466,7 @@ void input_from_server_till_request_got_processed(int fd,
 	}
       }
     } else {
-      close_socket_callback(&aconnection);
+      close_socket_callback(&client.conn);
       break;
     }
   }
@@ -485,10 +508,10 @@ double try_to_autoconnect(void)
   /* See PR#4042 for more info on issues with try_to_connect() and errno. */
   case ECONNREFUSED:		/* Server not available (yet) */
     if (!warning_shown) {
-      freelog(LOG_NORMAL, _("Connection to server refused. "
-			    "Please start the server."));
-      append_output_window(_("Connection to server refused. "
-			     "Please start the server."));
+      freelog(LOG_ERROR, "Connection to server refused. "
+                         "Please start the server.");
+      output_window_append(ftc_client, _("Connection to server refused. "
+                                         "Please start the server."));
       warning_shown = 1;
     }
     /* Try again in 0.5 seconds */
@@ -504,7 +527,7 @@ double try_to_autoconnect(void)
 }
 
 /**************************************************************************
-  Start trying to autoconnect to civserver.  Calls
+  Start trying to autoconnect to freeciv-server.  Calls
   get_server_address(), then arranges for try_to_autoconnect(), which
   calls try_to_connect(), to be called roughly every
   AUTOCONNECT_INTERVAL milliseconds, until success, fatal error or
@@ -514,19 +537,18 @@ void start_autoconnecting_to_server(void)
 {
   char buf[512];
 
-  my_snprintf(buf, sizeof(buf),
-	      _("Auto-connecting to server \"%s\" at port %d "
-		"as \"%s\" every %f second(s) for %d times"),
-	      server_host, server_port, user_name,
-	      0.001 * AUTOCONNECT_INTERVAL,
-	      MAX_AUTOCONNECT_ATTEMPTS);
-  append_output_window(buf);
+  output_window_printf(ftc_client,
+                       _("Auto-connecting to server \"%s\" at port %d "
+                         "as \"%s\" every %f second(s) for %d times"),
+                       server_host, server_port, user_name,
+                       0.001 * AUTOCONNECT_INTERVAL,
+                       MAX_AUTOCONNECT_ATTEMPTS);
 
   if (get_server_address(server_host, server_port, buf, sizeof(buf)) < 0) {
     freelog(LOG_FATAL,
-	    _("Error contacting server \"%s\" at port %d "
-	      "as \"%s\":\n %s\n"),
-	    server_host, server_port, user_name, buf);
+            _("Error contacting server \"%s\" at port %d "
+              "as \"%s\":\n %s\n"),
+            server_host, server_port, user_name, buf);
     exit(EXIT_FAILURE);
   }
   autoconnecting = TRUE;
