@@ -78,3 +78,21 @@ void color_free(struct color *color)
 {
   free(color);
 }
+
+/****************************************************************************
+  Fill the string with the color in "#rrggbb" mode.  Use it instead of
+  gdk_color() which have been included in gtk2.12 version only.
+****************************************************************************/
+size_t color_to_string(GdkColor *color, char *string, size_t length)
+{
+  RETURN_VAL_IF_FAIL(NULL != string, 0);
+  RETURN_VAL_IF_FAIL(0 < length, 0);
+
+  if (NULL == color) {
+    string[0] = '\0';
+    return 0;
+  } else {
+    return my_snprintf(string, length, "#%02x%02x%02x",
+                       color->red >> 8, color->green >> 8, color->blue >> 8);
+  }
+}
