@@ -20,22 +20,24 @@
 
 #include <gtk/gtk.h>
 
+/* utility */
+#include "log.h"
 #include "mem.h"
 #include "shared.h"
 #include "support.h"
 
+/* common */
 #include "diptreaty.h"
 #include "fcintl.h"
-#include "game.h"
 #include "government.h"
 #include "map.h"
 #include "packets.h"
 #include "player.h"
 
+/* client */
 #include "chatline.h"
-#include "civclient.h"
+#include "client_main.h"
 #include "climisc.h"
-#include "clinet.h"
 #include "options.h"
 
 #include "diplodlg.h"
@@ -176,19 +178,18 @@ static void popup_diplomacy_dialog(int other_player_id, int initiated_from)
     return;
   }
 
-  if (game.player_ptr->ai.control) {
+  if (client.conn.playing->ai_data.control) {
     return;			/* Don't show if we are AI controlled. */
   }
 
   if (!pdialog) {
-    pdialog =
-	create_diplomacy_dialog(game.player_ptr,
-				player_by_number(other_player_id));
+    pdialog = create_diplomacy_dialog(client.conn.playing,
+				      player_by_number(other_player_id));
   }
 
   gui_dialog_present(pdialog->dialog);
   /* We initated the meeting - Make the tab active */
-  if (initiated_from == player_number(game.player_ptr)) {
+  if (player_by_number(initiated_from) == client.conn.playing) {
     gui_dialog_raise(pdialog->dialog);
     
     if (players_dialog_shell != NULL) {
@@ -197,6 +198,18 @@ static void popup_diplomacy_dialog(int other_player_id, int initiated_from)
   }
 }
 
+/****************************************************************************
+  Utility for g_list_sort(). See below.
+****************************************************************************/
+static gint sort_advance_names(gconstpointer a, gconstpointer b)
+{
+  struct advance *padvance1 = (struct advance *) a;
+  struct advance *padvance2 = (struct advance *) b;
+
+  return compare_strings(advance_name_translation(padvance1),
+                         advance_name_translation(padvance2));
+}
+
 /****************************************************************
 ...
 *****************************************************************/
@@ -239,34 +252,66 @@ static void popup_add_menu(GtkMenuShell *parent, gpointer data)
 
   /* Advances. */
   {
-    bool flag;
-    int i;
+    GtkWidget *advance_item;
+    GList *sorting_list = NULL;
 
-    menu = gtk_menu_new();
+    advance_item = gtk_menu_item_new_with_mnemonic(_("_Advances"));
+    gtk_menu_shell_append(GTK_MENU_SHELL(parent), advance_item);
 
-    for (i = 1, flag = FALSE; i < game.control.num_tech_types; i++) {
-      if (get_invention(pgiver, i) == TECH_KNOWN
-	  && (get_invention(pother, i) == TECH_UNKNOWN
-	      || get_invention(pother, i) == TECH_REACHABLE)
-          && tech_is_available(pother, i)) {
-	item
-	  = gtk_menu_item_new_with_label(advance_name_for_player(game.player_ptr, i));
+    advance_iterate(A_FIRST, padvance) {
+      Tech_type_id i = advance_number(padvance);
 
-	gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-	g_signal_connect(item, "activate",
-			 G_CALLBACK(diplomacy_dialog_tech_callback),
-			 GINT_TO_POINTER((player_number(pgiver) << 24) |
-					 (player_number(pother) << 16) |
-					 i));
-	flag = TRUE;
+      if (player_invention_state(pgiver, i) == TECH_KNOWN
+          && player_invention_reachable(pother, i)
+          && (player_invention_state(pother, i) == TECH_UNKNOWN
+              || player_invention_state(pother, i) == TECH_PREREQS_KNOWN)) {
+        sorting_list = g_list_prepend(sorting_list, padvance);
       }
+    } advance_iterate_end;
+
+    if (NULL == sorting_list) {
+      /* No advance. */
+      gtk_widget_set_sensitive(advance_item, FALSE);
+    } else {
+      GList *list_item;
+      struct advance *padvance;
+
+      sorting_list = g_list_sort(sorting_list, sort_advance_names);
+      menu = gtk_menu_new();
+
+      /* TRANS: All technologies menu item in the diplomatic dialog. */
+      item = gtk_menu_item_new_with_label(_("All advances"));
+      gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+      g_object_set_data(G_OBJECT(item), "player_from",
+                        GINT_TO_POINTER(player_number(pgiver)));
+      g_object_set_data(G_OBJECT(item), "player_to",
+                        GINT_TO_POINTER(player_number(pother)));
+      g_signal_connect(item, "activate",
+                       G_CALLBACK(diplomacy_dialog_tech_callback),
+                       GINT_TO_POINTER(A_LAST));
+      gtk_menu_shell_append(GTK_MENU_SHELL(menu),
+                            gtk_separator_menu_item_new());
+
+      for (list_item = sorting_list; NULL != list_item;
+           list_item = g_list_next(list_item)) {
+        padvance = (struct advance *) list_item->data;
+        item =
+            gtk_menu_item_new_with_label(advance_name_translation(padvance));
+        gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+        g_object_set_data(G_OBJECT(item), "player_from",
+                          GINT_TO_POINTER(player_number(pgiver)));
+        g_object_set_data(G_OBJECT(item), "player_to",
+                          GINT_TO_POINTER(player_number(pother)));
+        g_signal_connect(item, "activate",
+                         G_CALLBACK(diplomacy_dialog_tech_callback),
+                         GINT_TO_POINTER(advance_number(padvance)));
+      }
+
+      gtk_menu_item_set_submenu(GTK_MENU_ITEM(advance_item), menu);
+      g_list_free(sorting_list);
     }
 
-    item = gtk_menu_item_new_with_mnemonic(_("_Advances"));
-    gtk_widget_set_sensitive(item, flag);
-    gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), menu);
-    gtk_menu_shell_append(GTK_MENU_SHELL(parent), item);
-    gtk_widget_show_all(item);
+    gtk_widget_show_all(advance_item);
   }
 
 
@@ -340,7 +385,7 @@ static void popup_add_menu(GtkMenuShell *parent, gpointer data)
 		   G_CALLBACK(diplomacy_dialog_embassy_callback), pdialog);
 
   /* Don't take in account the embassy effects. */
-  if (BV_ISSET(pother->embassy, player_index(pgiver))) {
+  if (player_has_real_embassy(pother, pgiver)) {
     gtk_widget_set_sensitive(item, FALSE);
   }
   gtk_menu_shell_append(GTK_MENU_SHELL(parent), item);
@@ -387,7 +432,7 @@ static void row_callback(GtkTreeView *view, GtkTreePath *path,
   i = 0; 
   clause_list_iterate(pdialog->treaty.clauses, pclause) {
     if (i == index[0]) {
-      dsend_packet_diplomacy_remove_clause_req(&aconnection,
+      dsend_packet_diplomacy_remove_clause_req(&client.conn,
 					       player_number(pdialog->treaty.plr1),
 					       player_number(pclause->from),
 					       pclause->type,
@@ -404,7 +449,11 @@ static void row_callback(GtkTreeView *view, GtkTreePath *path,
 *****************************************************************/
 static void diplomacy_destroy(struct Diplomacy_dialog* pdialog)
 {
-  gui_dialog_destroy(pdialog->dialog);
+  if (NULL != pdialog->dialog) {
+    /* pdialog->dialog may be NULL if the tab have been destroyed
+     * by an other way. */
+    gui_dialog_destroy(pdialog->dialog);
+  }
   dialog_list_unlink(dialog_list, pdialog);
   free(pdialog);
 }
@@ -421,11 +470,11 @@ static void diplomacy_response(struct gui_dialog *dlg, int response,
   case GTK_RESPONSE_ACCEPT:
 
 
-    dsend_packet_diplomacy_accept_treaty_req(&aconnection,
+    dsend_packet_diplomacy_accept_treaty_req(&client.conn,
 					     player_number(pdialog->treaty.plr1));
     break;
   default:
-    dsend_packet_diplomacy_cancel_meeting_req(&aconnection,
+    dsend_packet_diplomacy_cancel_meeting_req(&client.conn,
 					      player_number(pdialog->treaty.plr1));
     diplomacy_destroy(pdialog);
     break; 
@@ -676,7 +725,7 @@ static void update_diplomacy_dialog(struct Diplomacy_dialog *pdialog)
 
   gtk_list_store_clear(store);
   clause_list_iterate(pdialog->treaty.clauses, pclause) {
-    char buf[64];
+    char buf[128];
 
     client_diplomacy_clause_string(buf, sizeof(buf), pclause);
 
@@ -698,23 +747,46 @@ static void update_diplomacy_dialog(struct Diplomacy_dialog *pdialog)
 			    get_thumb_pixbuf(pdialog->treaty.accept1));
 }
 
-/****************************************************************
-...
-*****************************************************************/
+/****************************************************************************
+  Callback for the diplomatic dialog: give tech.
+****************************************************************************/
 static void diplomacy_dialog_tech_callback(GtkWidget *w, gpointer data)
 {
-  size_t choice = GPOINTER_TO_UINT(data);
-  int giver = (choice >> 24) & 0xff, dest = (choice >> 16) & 0xff, other;
-  int tech = choice & 0xffff;
+  int giver, dest, other, tech;
 
-  if (giver == game.info.player_idx) {
+  giver = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(w), "player_from"));
+  dest = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(w), "player_to"));
+  tech = GPOINTER_TO_INT(data);
+  if (player_by_number(giver) == client_player()) {
     other = dest;
   } else {
     other = giver;
   }
 
-  dsend_packet_diplomacy_create_clause_req(&aconnection, other, giver,
-					   CLAUSE_ADVANCE, tech);
+  if (A_LAST == tech) {
+    /* All techs. */
+    struct player *pgiver = player_by_number(giver);
+    struct player *pdest = player_by_number(dest);
+
+    RETURN_IF_FAIL(NULL != pgiver);
+    RETURN_IF_FAIL(NULL != pdest);
+
+    advance_iterate(A_FIRST, padvance) {
+      Tech_type_id i = advance_number(padvance);
+
+      if (player_invention_state(pgiver, i) == TECH_KNOWN
+          && player_invention_reachable(pdest, i)
+          && (player_invention_state(pdest, i) == TECH_UNKNOWN
+              || player_invention_state(pdest, i) == TECH_PREREQS_KNOWN)) {
+        dsend_packet_diplomacy_create_clause_req(&client.conn, other, giver,
+                                                 CLAUSE_ADVANCE, i);
+      }
+    } advance_iterate_end;
+  } else {
+    /* Only one tech. */
+    dsend_packet_diplomacy_create_clause_req(&client.conn, other, giver,
+                                             CLAUSE_ADVANCE, tech);
+  }
 }
 
 /****************************************************************
@@ -727,13 +799,13 @@ static void diplomacy_dialog_city_callback(GtkWidget * w, gpointer data)
   int giver = (choice >> 24) & 0xff, dest = (choice >> 16) & 0xff, other;
   int city = choice & 0xffff;
 
-  if (giver == game.info.player_idx) {
+  if (player_by_number(giver) == client.conn.playing) {
     other = dest;
   } else {
     other = giver;
   }
 
-  dsend_packet_diplomacy_create_clause_req(&aconnection, other, giver,
+  dsend_packet_diplomacy_create_clause_req(&client.conn, other, giver,
 					   CLAUSE_CITY, city);
 }
 
@@ -747,7 +819,7 @@ static void diplomacy_dialog_map_callback(GtkWidget *w, gpointer data)
   
   pgiver = (struct player *)g_object_get_data(G_OBJECT(w), "plr");
 
-  dsend_packet_diplomacy_create_clause_req(&aconnection,
+  dsend_packet_diplomacy_create_clause_req(&client.conn,
 					   player_number(pdialog->treaty.plr1),
 					   player_number(pgiver), CLAUSE_MAP, 0);
 }
@@ -762,7 +834,7 @@ static void diplomacy_dialog_seamap_callback(GtkWidget *w, gpointer data)
   
   pgiver = (struct player *)g_object_get_data(G_OBJECT(w), "plr");
 
-  dsend_packet_diplomacy_create_clause_req(&aconnection,
+  dsend_packet_diplomacy_create_clause_req(&client.conn,
 					   player_number(pdialog->treaty.plr1),
 					   player_number(pgiver), CLAUSE_SEAMAP,
 					   0);
@@ -776,7 +848,7 @@ static void diplomacy_dialog_add_pact_clause(GtkWidget *w, gpointer data,
 {
   struct Diplomacy_dialog *pdialog = (struct Diplomacy_dialog *)data;
 
-  dsend_packet_diplomacy_create_clause_req(&aconnection,
+  dsend_packet_diplomacy_create_clause_req(&client.conn,
 					   player_number(pdialog->treaty.plr1),
 					   player_number(pdialog->treaty.plr0),
 					   type, 0);
@@ -815,7 +887,7 @@ static void diplomacy_dialog_vision_callback(GtkWidget *w, gpointer data)
   struct player *pgiver =
       (struct player *) g_object_get_data(G_OBJECT(w), "plr");
 
-  dsend_packet_diplomacy_create_clause_req(&aconnection,
+  dsend_packet_diplomacy_create_clause_req(&client.conn,
 					   player_number(pdialog->treaty.plr1),
 					   player_number(pgiver), CLAUSE_VISION,
 					   0);
@@ -830,7 +902,7 @@ static void diplomacy_dialog_embassy_callback(GtkWidget *w, gpointer data)
   struct player *pgiver =
       (struct player *) g_object_get_data(G_OBJECT(w), "plr");
 
-  dsend_packet_diplomacy_create_clause_req(&aconnection,
+  dsend_packet_diplomacy_create_clause_req(&client.conn,
 					   player_number(pdialog->treaty.plr1),
 					   player_number(pgiver), CLAUSE_EMBASSY,
 					   0);
@@ -866,7 +938,8 @@ void diplomacy_dialog_done()
 *****************************************************************/
 static struct Diplomacy_dialog *find_diplomacy_dialog(int other_player_id)
 {
-  struct player *plr0 = game.player_ptr, *plr1 = player_by_number(other_player_id);
+  struct player *plr0 = client.conn.playing;
+  struct player *plr1 = player_by_number(other_player_id);
 
   dialog_list_iterate(dialog_list, pdialog) {
     if ((pdialog->treaty.plr0 == plr0 && pdialog->treaty.plr1 == plr1) ||
@@ -889,12 +962,12 @@ static void diplo_dialog_returnkey(GtkWidget *w, gpointer data)
   int amount = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(w));
   
   if (amount >= 0 && amount <= pgiver->economic.gold) {
-    dsend_packet_diplomacy_create_clause_req(&aconnection,
+    dsend_packet_diplomacy_create_clause_req(&client.conn,
 					     player_number(pdialog->treaty.plr1),
 					     player_number(pgiver),
 					     CLAUSE_GOLD, amount);
   } else {
-    append_output_window(_("Invalid amount of gold specified."));
+    output_window_append(ftc_client, _("Invalid amount of gold specified."));
   }
 }
 
