@@ -34,7 +34,7 @@
 #include "packets.h"
 
 /* client */
-#include "civclient.h"
+#include "client_main.h"
 #include "clinet.h"
 #include "connectdlg_common.h"
 
@@ -129,10 +129,7 @@ static int load_selected_game_callback(struct widget *pWidget)
     char *filename = (char*)pWidget->data.ptr;
 
     if (is_server_running()) {
-      char message[MAX_LEN_MSG];
-
-      my_snprintf(message, sizeof(message), "/load %s", filename);
-      send_chat(message);
+      send_chat_printf("/load %s", filename);
       
       if (get_client_page() == PAGE_LOAD) {
         set_client_page(PAGE_START);
@@ -249,7 +246,6 @@ static void popup_load_game_dialog(void)
     free(pfile);
   } datafile_list_iterate_end;
 
-  datafile_list_unlink_all(files);
   datafile_list_free(files);
 
   files = datafilelist_infix(NULL, ".sav", FALSE);
@@ -287,7 +283,6 @@ static void popup_load_game_dialog(void)
     free(pfile);
   } datafile_list_iterate_end;
 
-  datafile_list_unlink_all(files);
   datafile_list_free(files);
 
   pLastLabel = pFilenameLabel;
@@ -375,7 +370,7 @@ static int inputline_return_callback(struct widget *pWidget)
     if (theinput && *theinput) {
       send_chat(theinput);
   
-      append_output_window(theinput);
+      output_window_append(ftc_any, theinput);
       FC_FREE(theinput);
     }
   }  
@@ -410,7 +405,9 @@ void popup_input_line(void)
   Appends the string to the chat output window.  The string should be
   inserted on its own line, although it will have no newline.
 **************************************************************************/
-void real_append_output_window(const char *astring, int conn_id)
+void real_output_window_append(const char *astring,
+                               const struct text_tag_list *tags,
+                               int conn_id)
 {
   /* Currently this is a wrapper to the message subsystem. */
   if (pConnDlg) {
@@ -426,7 +423,7 @@ void real_append_output_window(const char *astring, int conn_id)
     char message[MAX_LEN_MSG];
     my_snprintf(message , MAX_LEN_MSG, "%s" , astring);
     
-    add_notify_window(message, NULL, E_CHAT_MSG);
+    add_notify_window(message, tags, NULL, E_CHAT_MSG);
   }
 }
 
@@ -544,7 +541,7 @@ static int input_edit_conn_callback(struct widget *pWidget)
     
       if (*theinput != '\0') {
         send_chat(theinput);
-        /*real_append_output_window(theinput);*/
+        /*real_output_window_append(theinput);*/
       }
       
       FC_FREE(pWidget->string16->text);
@@ -571,7 +568,7 @@ static int start_game_callback(struct widget *pWidget)
 static int select_nation_callback(struct widget *pWidget)
 {
   if (Main.event.button.button == SDL_BUTTON_LEFT) {
-    popup_races_dialog(game.player_ptr);
+    popup_races_dialog(client.conn.playing);
   }  
   return -1;
 }
@@ -669,8 +666,8 @@ void update_conn_list_dialog(void)
 
 /* FIXME: implement the server settings dialog and then reactivate this part */
 #if 0
-      if (aconnection.access_level == ALLOW_CTRL
-         || aconnection.access_level == ALLOW_HACK) {
+      if (ALLOW_CTRL == client.conn.access_level
+         || ALLOW_HACK == client.conn.access_level) {
 	set_wstate(pConnDlg->pConfigure, FC_WS_NORMAL);
       } else {
 	set_wstate(pConnDlg->pConfigure, FC_WS_DISABLED);
@@ -714,7 +711,7 @@ static void popup_conn_list_dialog(void)
   SDL_Rect area;
   SDL_Surface *pSurf;
     
-  if (pConnDlg || !aconnection.established) {
+  if (pConnDlg || !client.conn.established) {
     return;
   }
   
@@ -746,8 +743,9 @@ static void popup_conn_list_dialog(void)
   area.w = pConnDlg->text_width + adj_size(20);
   area.h = pWindow->size.h - adj_size(44) - adj_size(40);
   SDL_FillRectAlpha(pWindow->theme, &area, &window_bg_color);
-  putframe(pWindow->theme, area.x - 1, area.y - 1, area.x + area.w,
-           area.y + area.h, map_rgba(pWindow->theme->format, *get_game_colorRGB(COLOR_THEME_CONNLISTDLG_FRAME)));
+  putframe(pWindow->theme,
+           area.x - 1, area.y - 1, area.x + area.w, area.y + area.h,
+           get_game_colorRGB(COLOR_THEME_CONNLISTDLG_FRAME));
   
   /* user list background */
   area.x = pWindow->size.w - adj_size(130);
@@ -755,8 +753,9 @@ static void popup_conn_list_dialog(void)
   area.w = adj_size(120);
   area.h = pWindow->size.h - adj_size(44) - adj_size(40);
   SDL_FillRectAlpha(pWindow->theme, &area, &window_bg_color);
-  putframe(pWindow->theme, area.x - 1, area.y - 1, area.x + area.w,
-           area.y + area.h, map_rgba(pWindow->theme->format, *get_game_colorRGB(COLOR_THEME_CONNLISTDLG_FRAME)));
+  putframe(pWindow->theme,
+           area.x - 1, area.y - 1, area.x + area.w, area.y + area.h,
+           get_game_colorRGB(COLOR_THEME_CONNLISTDLG_FRAME));
   
   draw_frame(pWindow->theme, 0, 0, pWindow->theme->w, pWindow->theme->h);
     
