@@ -18,35 +18,41 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-                         
+
 #include <windows.h>
 #include <windowsx.h>
 
-#include "city.h"
+/* utility */
 #include "fcintl.h"
-#include "game.h"
-#include "packets.h"
+#include "log.h"
+#include "mem.h"
 #include "shared.h"
 #include "support.h"
+
+/* common */
+#include "city.h"
+#include "packets.h"
 #include "unit.h"
-#include "mem.h"
- 
+
+/* client */
 #include "chatline.h"
 #include "citydlg.h"
 #include "cityrepdata.h"
+#include "client_main.h"
+#include "climisc.h"
 #include "gui_main.h"
 #include "gui_stuff.h"
 #include "mapview.h"
 #include "optiondlg.h"
 #include "options.h"
 #include "repodlgs.h"
-#include "climisc.h"
 #include "text.h"
-                           
+
 #include "cityrep.h"
+
 extern HINSTANCE freecivhinst;
 extern HFONT font_12courier;
-extern struct connection aconnection;   
+
 int max_changemenu_id;
 int max_supportmenu_id;
 int max_presentmenu_id;
@@ -55,9 +61,8 @@ int max_improvement_id;
 static int city_sort_id;
 static int city_sort_order;
 static HMENU menu_shown; 
-static HWND sort_buttons[NUM_CREPORT_COLS];
+static HWND *sort_buttons;
 HWND cityrep_list;
-typedef bool TestCityFunc(struct city *, int);     
 
 #define ID_CITYREP_SORTBASE 6400
 #define ID_CHANGE_POPUP_BASE 7000
@@ -126,7 +131,7 @@ static void create_cityrep_config_dlg(HWND hWnd)
   struct city_report_spec *spec;   
   box=fcwin_vbox_new(hWnd,FALSE);
   fcwin_box_add_static_default(box,_("Set columns shown"),-1,SS_CENTER);
-  for(i=1, spec=city_report_specs+i; i<NUM_CREPORT_COLS; i++, spec++) {     
+  for(i=0, spec=city_report_specs+i; i<NUM_CREPORT_COLS; i++, spec++) {     
     fcwin_box_add_checkbox(box, spec->explanation,
 			   ID_CITYREP_CONFIG_BASE+i,0,FALSE,FALSE,5);
   }
@@ -158,7 +163,7 @@ LONG APIENTRY ConfigCityRepProc(HWND hWnd,
 	{
 	  struct city_report_spec *spec;   
 	  int i;
-	  for(i=1, spec=city_report_specs+i; i<NUM_CREPORT_COLS; i++, spec++) 
+	  for(i=0, spec=city_report_specs+i; i<NUM_CREPORT_COLS; i++, spec++) 
 	    {   
 	      spec->show=IsDlgButtonChecked(hWnd,ID_CITYREP_CONFIG_BASE+i);  
 	    }
@@ -186,9 +191,9 @@ append_impr_or_unit_to_menu_sub(HMENU menu,
 				int *selitems,
 				int selcount, int *idcount)
 {
-  cid cids[U_LAST + B_LAST];
-  struct item items[U_LAST + B_LAST];
-  int item, cids_used, num_selected_cities = 0;
+  struct universal targets[MAX_NUM_PRODUCTION_TARGETS];
+  struct item items[MAX_NUM_PRODUCTION_TARGETS];
+  int item, targets_used, num_selected_cities = 0;
   struct city **selected_cities = NULL;
 
   if (change_prod) {
@@ -205,26 +210,26 @@ append_impr_or_unit_to_menu_sub(HMENU menu,
     }
   }
 
-  cids_used = collect_cids1(cids, selected_cities,
+  targets_used = collect_production_targets(targets, selected_cities,
 			    num_selected_cities, append_units,
 			    append_wonders, change_prod, test_func);
   if (selected_cities) {
     free(selected_cities);
   }
-  name_and_sort_items(cids, cids_used, items, change_prod, NULL);
+  name_and_sort_items(targets, targets_used, items, change_prod, NULL);
 
-  for (item = 0; item < cids_used; item++) {
+  for (item = 0; item < targets_used; item++) {
     MENUITEMINFO iteminfo;
 
     AppendMenu(menu, MF_STRING, (*idcount), items[item].descr);
-    iteminfo.dwItemData = items[item].cid;
+    iteminfo.dwItemData = cid_encode(items[item].item);
     iteminfo.fMask = MIIM_DATA;
     iteminfo.cbSize = sizeof(MENUITEMINFO);
     SetMenuItemInfo(menu, (*idcount), FALSE, &iteminfo);
     (*idcount)++;
   }
 
-  if (cids_used == 0) {
+  if (targets_used == 0) {
     AppendMenu(menu, MF_STRING, -1, nothing_appended_text);
   }
 }
@@ -245,7 +250,6 @@ static void append_impr_or_unit_to_menu(HMENU menu,
     /* Add all buildings */
     append_impr_or_unit_to_menu_sub(menu, _("No Buildings Available"),
 				    FALSE, FALSE, change_prod,
-				    (bool (*)(struct city *, int))
 				    test_func,
 				    selitems, selcount, idcount);
     /* Add a separator */
@@ -269,7 +273,6 @@ static void append_impr_or_unit_to_menu(HMENU menu,
     /* Add all wonders */
     append_impr_or_unit_to_menu_sub(menu, _("No Wonders Available"),
 				    FALSE, TRUE, change_prod,
-				    (bool (*)(struct city *, int))
 				    test_func,
 				    selitems, selcount, idcount);
   }
@@ -300,7 +303,8 @@ static void get_city_text(struct city *pcity, char *buf[], int n)
   for(i=0, spec=city_report_specs; i<NUM_CREPORT_COLS; i++, spec++) {
     buf[i][0]='\0';
     if(!spec->show) continue;
-    my_snprintf(buf[i], n, "%*s", NEG_VAL(spec->width)-2, (spec->func)(pcity)); 
+    my_snprintf(buf[i], n, "%*s", NEG_VAL(spec->width)-2,
+		(spec->func)(pcity, spec->data)); 
   }
 }
 
@@ -340,7 +344,7 @@ static void cityrep_popup(HWND hWnd)
     {
       pcity=(struct city *)ListBox_GetItemData(GetDlgItem(hWnd,ID_CITYREP_LIST),
 				cityids[i]);
-      popup_city_dialog(pcity,0);
+      popup_city_dialog(pcity);
     }
 }
 
@@ -381,7 +385,7 @@ static void cityrep_refresh(HWND hWnd)
   if ((selcount==LB_ERR)||(selcount==0))
     {
       packet.value = 0;
-      send_packet_generic_integer(&aconnection, PACKET_CITY_REFRESH, &packet);
+      send_packet_generic_integer(&client.conn, PACKET_CITY_REFRESH, &packet);
       return;
     }
   selcount=MIN(256,selcount);
@@ -393,7 +397,7 @@ static void cityrep_refresh(HWND hWnd)
 							  ID_CITYREP_LIST),
 					       cityids[i]);
       packet.value = pcity->id;
-      send_packet_generic_integer(&aconnection, PACKET_CITY_REFRESH,
+      send_packet_generic_integer(&client.conn, PACKET_CITY_REFRESH,
 				  &packet);      
     }
 }
@@ -419,7 +423,7 @@ static void cityrep_change(HWND hWnd)
   popup=CreatePopupMenu();
   max_changemenu_id=ID_CHANGE_POPUP_BASE;
   append_impr_or_unit_to_menu(popup,TRUE,TRUE,TRUE,
-			      city_can_build_impr_or_unit,
+			      can_city_build_now,
 			      &cityids[0],selcount,&max_changemenu_id);
   GetWindowRect(GetDlgItem(hWnd,ID_CITYREP_CHANGE),&rc);
   menu_shown=popup;
@@ -464,7 +468,7 @@ static void cityrep_select(HWND hWnd)
   AppendMenu(popup,MF_POPUP,(UINT)submenu,_("Available To Build"));
   max_availablemenu_id=ID_AVAILABLE_POPUP_BASE;
   append_impr_or_unit_to_menu(submenu, FALSE, TRUE, TRUE,
-			      city_can_build_impr_or_unit,&cityids,selcount,
+			      can_city_build_now,&cityids,selcount,
 			      &max_availablemenu_id);
   
   submenu=CreatePopupMenu(); 
@@ -487,9 +491,8 @@ static void cityrep_change_menu(HWND hWnd, cid cid)
 {  
   int cityids[256];
   int selcount, i, last_request_id = 0;
-  struct city *pcity; 
-  bool is_unit = cid_is_unit(cid);
-  int number = cid_id(cid);
+  struct city *pcity;
+  struct universal target = cid_production(cid);
   
   selcount=ListBox_GetSelCount(GetDlgItem(hWnd,ID_CITYREP_LIST));
   if (selcount==LB_ERR) return;
@@ -497,17 +500,17 @@ static void cityrep_change_menu(HWND hWnd, cid cid)
   selcount=ListBox_GetSelItems(GetDlgItem(hWnd,ID_CITYREP_LIST),
 			       selcount,&cityids[0]);
 
-  connection_do_buffer(&aconnection);
+  connection_do_buffer(&client.conn);
   for (i = 0; i < selcount; i++) {
     pcity = (struct city *) ListBox_GetItemData(GetDlgItem(hWnd,
 							   ID_CITYREP_LIST),
 						cityids[i]);
     last_request_id =
-      city_change_production(pcity, is_unit, number);
+      city_change_production(pcity, target);
     ListBox_SetSel(GetDlgItem(hWnd, ID_CITYREP_LIST), FALSE, cityids[i]);
   }
 
-  connection_do_unbuffer(&aconnection);
+  connection_do_unbuffer(&client.conn);
   reports_freeze_till(last_request_id);
 }
 
@@ -580,8 +583,8 @@ static void list_sameisland_select(HWND hLst)
 	{
 	  struct city *selectedcity;
 	  selectedcity=(struct city *)ListBox_GetItemData(hLst,cityids[j]);
-          if (map_get_continent(pcity->tile)
-              == map_get_continent(selectedcity->tile))
+          if (tile_continent(pcity->tile)
+              == tile_continent(selectedcity->tile))
 	    {    
 	      ListBox_SetSel(hLst,TRUE,i);
 	      break;
@@ -593,8 +596,9 @@ static void list_sameisland_select(HWND hLst)
 /**************************************************************************
 
 **************************************************************************/
-static void list_impr_or_unit_select(HWND hLst, int num,
-				     TestCityFunc *test_func)
+static void list_impr_or_unit_select(HWND hLst,
+				     struct universal target,
+				     TestCityFunc test_func)
 {
   int i,rows;
   list_all_select(hLst,FALSE);
@@ -602,7 +606,7 @@ static void list_impr_or_unit_select(HWND hLst, int num,
   for (i=0;i<rows;i++)
     {
       struct city *pcity=(struct city *)ListBox_GetItemData(hLst,i);
-      if (test_func(pcity,num))
+      if (test_func(pcity, target))
 	ListBox_SetSel(hLst,TRUE,i);
     }
 }
@@ -613,6 +617,7 @@ static void list_impr_or_unit_select(HWND hLst, int num,
 static void menu_proc(HWND hWnd,int cmd, DWORD num)
 {
   HWND hLst;
+  struct universal target = cid_decode(num);
   hLst=GetDlgItem(hWnd,ID_CITYREP_LIST);
   if ((cmd>=ID_CHANGE_POPUP_BASE)&&
       (cmd<max_changemenu_id))
@@ -623,25 +628,25 @@ static void menu_proc(HWND hWnd,int cmd, DWORD num)
   if ((cmd>=ID_SUPPORTED_POPUP_BASE)&&
       (cmd<max_supportmenu_id))
     {
-      list_impr_or_unit_select(hLst,num,city_unit_supported);
+      list_impr_or_unit_select(hLst, target, city_unit_supported);
       max_supportmenu_id=0;
     }
   if ((cmd>=ID_PRESENT_POPUP_BASE)&&
       (cmd<max_presentmenu_id))
     {
-      list_impr_or_unit_select(hLst,num,city_unit_present);
+      list_impr_or_unit_select(hLst, target, city_unit_present);
       max_presentmenu_id=0;
     }
   if ((cmd>=ID_AVAILABLE_POPUP_BASE)&&
       (cmd<max_availablemenu_id))
     {
-      list_impr_or_unit_select(hLst,num,city_can_build_impr_or_unit);
+      list_impr_or_unit_select(hLst, target, can_city_build_now);
       max_availablemenu_id=0;
     }
   if ((cmd>=ID_IMPROVEMENTS_POPUP_BASE)&&
       (cmd<max_improvement_id))
     {
-      list_impr_or_unit_select(hLst,num,city_building_present);
+      list_impr_or_unit_select(hLst, target, city_building_present);
       max_improvement_id=0;
     }
 }
@@ -676,13 +681,14 @@ static LONG CALLBACK cityrep_changeall_proc(HWND hWnd,
 	case ID_PRODCHANGE_CHANGE:
 	  {
 	    int id;
-	    cid from, to;
+	    int from, to;
 
 	    id=ListBox_GetCurSel(GetDlgItem(hWnd,ID_PRODCHANGE_FROM));
 	    if (id==LB_ERR)
 	      {
-		append_output_window(_("Game: Select a unit or improvement"
-				       " to change production from."));
+                output_window_append(ftc_client,
+                                     _("Select a unit or improvement"
+                                       " to change production from."));
 		break;        
 	      }
 	    from=ListBox_GetItemData(GetDlgItem(hWnd,ID_PRODCHANGE_FROM),
@@ -690,16 +696,17 @@ static LONG CALLBACK cityrep_changeall_proc(HWND hWnd,
 	    id=ListBox_GetCurSel(GetDlgItem(hWnd,ID_PRODCHANGE_TO));
 	    if (id==LB_ERR)
 	      {
-		append_output_window(_("Game: Select a unit or improvement"
-				       " to change production to."));
+                output_window_append(ftc_client,
+                                     _("Select a unit or improvement"
+                                       " to change production to."));
 		break;          
 	      }
 	    to=ListBox_GetItemData(GetDlgItem(hWnd,ID_PRODCHANGE_TO),id);
 	    if (from==to) {
-	      append_output_window(_("Game: That's the same thing!"));
+              output_window_append(ftc_client, _("That's the same thing!"));
 	      break;
 	    }
-	    client_change_all(from,to);
+	    client_change_all(cid_decode(from), cid_decode(to));
 	    DestroyWindow(hWnd);
 	    hChangeAll=NULL;
 	  }
@@ -721,10 +728,10 @@ static void cityrep_changeall(HWND hWnd)
   struct fcwin_box *vbox;
   struct fcwin_box *hbox;
   int selid;
-  cid cids[B_LAST + U_LAST];
-  int cids_used;
+  struct universal targets[MAX_NUM_PRODUCTION_TARGETS];
+  struct item items[MAX_NUM_PRODUCTION_TARGETS];
+  int targets_used;
   cid selected_cid;
-  struct item items[U_LAST + B_LAST];
   int id,i;
   HWND hDlg;
   HWND hLst;
@@ -765,24 +772,24 @@ static void cityrep_changeall(HWND hWnd)
 						 selid));
   }
 
-  cids_used = collect_cids2(cids);
-  name_and_sort_items(cids, cids_used, items, FALSE, NULL);
+  targets_used = collect_currently_building_targets(targets);
+  name_and_sort_items(targets, targets_used, items, FALSE, NULL);
 
   hLst = GetDlgItem(hDlg, ID_PRODCHANGE_FROM);
-  for (i = 0; i < cids_used; i++) {
+  for (i = 0; i < targets_used; i++) {
     id = ListBox_AddString(hLst, items[i].descr);
-    ListBox_SetItemData(hLst, id, items[i].cid);
-    if (items[i].cid == selected_cid)
+    ListBox_SetItemData(hLst, id, cid_encode(items[i].item));
+    if (cid_encode(items[i].item) == selected_cid)
       ListBox_SetCurSel(hLst, id);
   }
 
-  cids_used = collect_cids3(cids);
-  name_and_sort_items(cids, cids_used, items, TRUE, NULL);
+  targets_used = collect_buildable_targets(targets);
+  name_and_sort_items(targets, targets_used, items, TRUE, NULL);
 
   hLst = GetDlgItem(hDlg, ID_PRODCHANGE_TO);
-  for (i = 0; i < cids_used; i++) {
+  for (i = 0; i < targets_used; i++) {
     id = ListBox_AddString(hLst, items[i].descr);
-    ListBox_SetItemData(hLst, id, items[i].cid);
+    ListBox_SetItemData(hLst, id, cid_encode(items[i].item));
   }
 
   fcwin_set_box(hDlg,hbox);
@@ -866,8 +873,8 @@ static void list_del(void *data)
 **************************************************************************/
 static void city_report_create(HWND hWnd)
 {
-  static char *titles   [NUM_CREPORT_COLS];
-  static char  buf      [NUM_CREPORT_COLS][64];   
+  static char **titles;
+  static char (*buf)[64];   
   struct city_report_spec *spec;        
   struct fcwin_box *vbox;
   struct fcwin_box *hbox;
@@ -889,24 +896,35 @@ static void city_report_create(HWND hWnd)
 			TRUE,TRUE,5);
   fcwin_box_add_box(vbox,hbox,FALSE,FALSE,5);
   
- 
+  if (titles) {
+    free(titles);
+  }
+  if (buf) {
+    free(buf);
+  }
+  if (sort_buttons) {
+    free(sort_buttons);
+  }
+  
+  titles = fc_malloc(sizeof(*titles) * NUM_CREPORT_COLS);
+  buf = fc_malloc(sizeof(*buf) * NUM_CREPORT_COLS);
+  sort_buttons = fc_malloc(sizeof(*sort_buttons) * NUM_CREPORT_COLS);
+
   for (i=0;i<NUM_CREPORT_COLS;i++)
     titles[i]=buf[i];
   
   get_city_table_header(titles, sizeof(buf[0]));  
   for(i=0, spec=city_report_specs; i<NUM_CREPORT_COLS; i++, spec++) {         
-    if (spec->show)
-      {
-	sort_buttons[i]=CreateWindow("BUTTON",titles[i],
-				     WS_CHILD | WS_VISIBLE | BS_MULTILINE,
-				     0,0,0,0,
-				     hWnd,
-				     (HMENU)(ID_CITYREP_SORTBASE+i),
-				     freecivhinst,
-				     NULL);
-	SendMessage(sort_buttons[i],
-		    WM_SETFONT,(WPARAM) font_12courier,MAKELPARAM(TRUE,0));
- 				           }
+    if (spec->show) {
+      sort_buttons[i]=CreateWindow("BUTTON", titles[i], WS_CHILD | WS_VISIBLE
+				   | BS_MULTILINE, 0, 0, 0, 0, hWnd,
+				   (HMENU)(ID_CITYREP_SORTBASE+i),
+				   freecivhinst, NULL);
+      SendMessage(sort_buttons[i], WM_SETFONT, (WPARAM)font_12courier,
+		  MAKELPARAM(TRUE,0));
+    } else {
+      sort_buttons[i] = NULL;
+    }
   }
 
   cityrep_list=CreateWindow("LISTBOX",NULL,WS_CHILD | WS_VISIBLE | 
@@ -1060,21 +1078,24 @@ static LONG APIENTRY city_report_proc(HWND hWnd,
 
 **************************************************************************/
 void
-popup_city_report_dialog(bool make_modal)
+popup_city_report_dialog(bool raise)
 {
-  if (hCityRep) return;
-  hCityRep=
-    fcwin_create_layouted_window(city_report_proc,_("City Report"),
-				 WS_OVERLAPPED | WS_SYSMENU | WS_MINIMIZEBOX | 
-				 WS_MAXIMIZEBOX | WS_THICKFRAME,
-				 CW_USEDEFAULT,CW_USEDEFAULT,
-				 root_window,NULL,
-				 JUST_CLEANUP,
-				 NULL);
+  if (!hCityRep) {
+    hCityRep =
+      fcwin_create_layouted_window(city_report_proc,_("City Report"),
+				   WS_OVERLAPPED | WS_SYSMENU | WS_MINIMIZEBOX
+				    | WS_MAXIMIZEBOX | WS_THICKFRAME,
+				   CW_USEDEFAULT, CW_USEDEFAULT,
+				   root_window, NULL, JUST_CLEANUP, NULL);
   
-  hChangeAll=NULL;
-  city_report_dialog_update();
-  ShowWindow(hCityRep,SW_SHOWNORMAL);
+    hChangeAll = NULL;
+    city_report_dialog_update();
+  }
+
+  ShowWindow(hCityRep, SW_SHOWNORMAL);
+  if (raise) {
+    SetFocus(hCityRep);
+  }
 }
 
 /**************************************************************************
@@ -1097,7 +1118,7 @@ static void my_cityrep_add_city(HWND hLst,struct city *pcity_new,char *full_row,
   int i,n,listcount;  
   struct city_report_spec *spec; 
   char *rowold   [NUM_CREPORT_COLS];
-  char  buf   [NUM_CREPORT_COLS][64];
+  char  buf   [NUM_CREPORT_COLS][128];
   for (i=0, spec=city_report_specs;i<NUM_CREPORT_COLS;i++, spec++)
     {
       rowold[i] = buf[i];    
@@ -1127,7 +1148,7 @@ void
 city_report_dialog_update(void)
 { 
   char *row   [NUM_CREPORT_COLS];
-  char  buf   [NUM_CREPORT_COLS][64];
+  char  buf   [NUM_CREPORT_COLS][128];
   char full_row[1024];
   int   i;
   struct city_report_spec *spec; 
@@ -1142,7 +1163,7 @@ city_report_dialog_update(void)
     }
   /* FIXME restore old selection */
   ListBox_ResetContent(GetDlgItem(hCityRep,ID_CITYREP_LIST));
-  city_list_iterate(game.player_ptr->cities, pcity) {
+  city_list_iterate(client.conn.playing->cities, pcity) {
     get_city_text(pcity, row, sizeof(buf[0]));
     full_row[0]=0;
     for(i=0; i<NUM_CREPORT_COLS; i++)
@@ -1164,7 +1185,7 @@ void
 city_report_dialog_update_city(struct city *pcity)
 {
   char *row [NUM_CREPORT_COLS];
-  char  buf [NUM_CREPORT_COLS][64];
+  char  buf [NUM_CREPORT_COLS][128];
   char full_row[512];
   int   i,nCount;     
   struct city_report_spec *spec; 
