@@ -24,17 +24,21 @@
 #include <windowsx.h>
 #include <commctrl.h>
 
+/* utility */
 #include "fcintl.h"
-#include "game.h"
-#include "government.h"
-#include "packets.h"
+#include "log.h"
 #include "shared.h"
 #include "support.h"
+
+/* common */
+#include "government.h"
+#include "packets.h"
 #include "unitlist.h"
 
+/* client */
 #include "chatline_common.h"	/* send_chat() */ 
 #include "cityrep.h"
-#include "civclient.h"
+#include "client_main.h"
 #include "climisc.h"
 #include "dialogs.h"
 #include "gui_stuff.h"
@@ -57,8 +61,8 @@ static int *categories;
 
 extern HINSTANCE freecivhinst;
 extern HWND root_window;
-extern struct connection aconnection;
-int economy_improvement_type[B_LAST];
+
+struct impr_type *economy_improvement_type[B_LAST];
 struct unit_type *activeunits_type[U_LAST];
 
 #define ID_OPTIONS_BASE 1000
@@ -96,21 +100,20 @@ science_dialog_update(void)
   SetWindowText(GetDlgItem(science_dlg, ID_SCIENCE_TOP), text);
   ListBox_ResetContent(GetDlgItem(science_dlg, ID_SCIENCE_LIST));
 
-  tech_type_iterate(tech_id) {
-    if (get_invention(game.player_ptr, tech_id) == TECH_KNOWN
-	&& tech_id != A_NONE) {
+  advance_index_iterate(A_FIRST, tech_id) {
+    if (TECH_KNOWN == player_invention_state(client.conn.playing, tech_id)) {
       id = ListBox_AddString(GetDlgItem(science_dlg, ID_SCIENCE_LIST),
-			     advance_name_for_player(game.player_ptr, tech_id));
+			     advance_name_for_player(client.conn.playing, tech_id));
       ListBox_SetItemData(GetDlgItem(science_dlg,ID_SCIENCE_LIST), id,
 			  tech_id);
     }
-  } tech_type_iterate_end;
+  } advance_index_iterate_end;
 
   ComboBox_ResetContent(GetDlgItem(science_dlg, ID_SCIENCE_RESEARCH));
 
-  if (get_player_research(game.player_ptr)->researching == A_UNSET) {
+  if (A_UNSET == get_player_research(client.conn.playing)->researching) {
     id = ComboBox_AddString(GetDlgItem(science_dlg, ID_SCIENCE_RESEARCH),
-			    advance_name_for_player(game.player_ptr, A_NONE));
+			    advance_name_for_player(client.conn.playing, A_NONE));
     ComboBox_SetItemData(GetDlgItem(science_dlg, ID_SCIENCE_RESEARCH),
 			 id, A_NONE);
     ComboBox_SetCurSel(GetDlgItem(science_dlg, ID_SCIENCE_RESEARCH),
@@ -118,32 +121,33 @@ science_dialog_update(void)
     text[0] = '\0';
   } else {
     my_snprintf(text, sizeof(text), "%d/%d",
-		get_player_research(game.player_ptr)->bulbs_researched,
-		total_bulbs_required(game.player_ptr));
+		get_player_research(client.conn.playing)->bulbs_researched,
+		total_bulbs_required(client.conn.playing));
   }
 
   SetWindowText(GetDlgItem(science_dlg, ID_SCIENCE_PROG), text);
 
-  if (!is_future_tech(get_player_research(game.player_ptr)->researching)) {
-    tech_type_iterate(tech_id) {
-      if (get_invention(game.player_ptr, tech_id) != TECH_REACHABLE) {
+  if (!is_future_tech(get_player_research(client.conn.playing)->researching)) {
+    advance_index_iterate(A_FIRST, tech_id) {
+      if (TECH_PREREQS_KNOWN !=
+            player_invention_state(client.conn.playing, tech_id)) {
 	continue;
       }
 
       id = ComboBox_AddString(GetDlgItem(science_dlg, ID_SCIENCE_RESEARCH),
-			      advance_name_for_player(game.player_ptr, tech_id));
+			      advance_name_for_player(client.conn.playing, tech_id));
       ComboBox_SetItemData(GetDlgItem(science_dlg, ID_SCIENCE_RESEARCH),
 			   id, tech_id);
-      if (tech_id == get_player_research(game.player_ptr)->researching) {
+      if (tech_id == get_player_research(client.conn.playing)->researching) {
 	ComboBox_SetCurSel(GetDlgItem(science_dlg, ID_SCIENCE_RESEARCH),
 			   id);
       }
-    } tech_type_iterate_end;
+    } advance_index_iterate_end;
   } else {
-      tech_id = game.control.num_tech_types + 1
-		+ get_player_research(game.player_ptr)->future_tech;
+      tech_id = advance_count() + 1
+		+ get_player_research(client.conn.playing)->future_tech;
       id = ComboBox_AddString(GetDlgItem(science_dlg, ID_SCIENCE_RESEARCH),
-			      advance_name_for_player(game.player_ptr, tech_id));
+			      advance_name_for_player(client.conn.playing, tech_id));
       ComboBox_SetItemData(GetDlgItem(science_dlg, ID_SCIENCE_RESEARCH),
 			   id, tech_id);
       ComboBox_SetCurSel(GetDlgItem(science_dlg, ID_SCIENCE_RESEARCH),
@@ -151,41 +155,47 @@ science_dialog_update(void)
   }
   ComboBox_ResetContent(GetDlgItem(science_dlg,ID_SCIENCE_GOAL));
     hist=0;
-  tech_type_iterate(tech_id) {
-    if (tech_is_available(game.player_ptr, tech_id)
-        && get_invention(game.player_ptr, tech_id) != TECH_KNOWN
-        && advances[tech_id].req[0] != A_LAST
-	&& advances[tech_id].req[1] != A_LAST
-        && (num_unknown_techs_for_goal(game.player_ptr, tech_id) < 11
-	    || tech_id == get_player_research(game.player_ptr)->tech_goal)) {
+  advance_index_iterate(A_FIRST, tech_id) {
+    if (player_invention_reachable(client.conn.playing, tech_id)
+        && TECH_KNOWN != player_invention_state(client.conn.playing, tech_id)
+        && (11 > num_unknown_techs_for_goal(client.conn.playing, tech_id)
+	    || tech_id == get_player_research(client.conn.playing)->tech_goal)) {
       id = ComboBox_AddString(GetDlgItem(science_dlg,ID_SCIENCE_GOAL),
-			      advance_name_for_player(game.player_ptr, tech_id));
+			      advance_name_for_player(client.conn.playing, tech_id));
        ComboBox_SetItemData(GetDlgItem(science_dlg,ID_SCIENCE_GOAL),
 			 id, tech_id);
-      if (tech_id == get_player_research(game.player_ptr)->tech_goal)
+      if (tech_id == get_player_research(client.conn.playing)->tech_goal)
  	ComboBox_SetCurSel(GetDlgItem(science_dlg,ID_SCIENCE_GOAL),
  			   id);
        
      }
-  } tech_type_iterate_end;
+  } advance_index_iterate_end;
 
-  if (get_player_research(game.player_ptr)->tech_goal == A_UNSET) {
+  if (A_UNSET == get_player_research(client.conn.playing)->tech_goal) {
     id = ComboBox_AddString(GetDlgItem(science_dlg, ID_SCIENCE_GOAL),
-			    advance_name_for_player(game.player_ptr, A_NONE));
+			    advance_name_for_player(client.conn.playing, A_NONE));
     ComboBox_SetItemData(GetDlgItem(science_dlg, ID_SCIENCE_GOAL),
 			 id, A_NONE);
     ComboBox_SetCurSel(GetDlgItem(science_dlg, ID_SCIENCE_GOAL),
 		       id);
    }
 
-  steps = num_unknown_techs_for_goal(game.player_ptr,
-                            get_player_research(game.player_ptr)->tech_goal);
+  steps = num_unknown_techs_for_goal(client.conn.playing,
+                            get_player_research(client.conn.playing)->tech_goal);
   my_snprintf(text, sizeof(text),
 	      PL_("(%d step)", "(%d steps)", steps), steps);
   SetWindowText(GetDlgItem(science_dlg,ID_SCIENCE_STEPS),text);
   fcwin_redo_layout(science_dlg);
 }
 
+/****************************************************************************
+  Resize and redraw the requirement tree.
+****************************************************************************/
+void science_dialog_redraw(void)
+{
+  /* No requirement tree yet. */
+}
+
 /**************************************************************************
 
 **************************************************************************/
@@ -215,10 +225,11 @@ static LONG CALLBACK science_proc(HWND hWnd,
 					to);
 	      
 	      if (IsDlgButtonChecked(hWnd, ID_SCIENCE_HELP)) {
-		popup_help_dialog_typed(advance_name_translation(to), HELP_TECH);
+		popup_help_dialog_typed(advance_name_translation(advance_by_number(to)),
+					HELP_TECH);
 		science_dialog_update();
 	      } else {
-		dsend_packet_player_research(&aconnection, to);
+		dsend_packet_player_research(&client.conn, to);
 	      }
 	    }
 	  }
@@ -231,12 +242,12 @@ static LONG CALLBACK science_proc(HWND hWnd,
 
 	      to = ComboBox_GetItemData(GetDlgItem(hWnd, ID_SCIENCE_GOAL),
 					to);
-	      steps = num_unknown_techs_for_goal(game.player_ptr, to);
+	      steps = num_unknown_techs_for_goal(client.conn.playing, to);
 	      my_snprintf(text, sizeof(text), 
 	                  PL_("(%d step)", "(%d steps)", steps),
 			  steps);
 	      SetWindowText(GetDlgItem(hWnd,ID_SCIENCE_STEPS), text);
-	      dsend_packet_player_tech_goal(&aconnection, to);
+	      dsend_packet_player_tech_goal(&client.conn, to);
 	    }
 	  }
 	  break;
@@ -485,7 +496,7 @@ popup_economy_report_dialog(bool raise)
 *****************************************************************/
 static void upgrade_callback_yes(HWND w, void * data)
 {
-  dsend_packet_unit_type_upgrade(&aconnection, (size_t)data);
+  dsend_packet_unit_type_upgrade(&client.conn, (size_t)data);
   destroy_message_dialog(w);
 }
  
@@ -535,7 +546,7 @@ static LONG CALLBACK activeunits_proc(HWND hWnd,
     case WM_NOTIFY:
       if (sel>=0) {
 	CHECK_UNIT_TYPE(activeunits_type[sel]);
-	if (can_upgrade_unittype(game.player_ptr,
+	if (can_upgrade_unittype(client.conn.playing,
 				 activeunits_type[sel]) != NULL) {
 	  EnableWindow(GetDlgItem(activeunits_dlg,ID_MILITARY_UPGRADE),
 		       TRUE);
@@ -563,14 +574,14 @@ static LONG CALLBACK activeunits_proc(HWND hWnd,
 
 	      ut1 = activeunits_type[sel];
 	      CHECK_UNIT_TYPE(ut1);
-	      ut2=can_upgrade_unittype(game.player_ptr,activeunits_type[sel]);
+	      ut2 = can_upgrade_unittype(client.conn.playing, activeunits_type[sel]);
 	      my_snprintf(buf, sizeof(buf),
 			  _("Upgrade as many %s to %s as possible for %d gold each?\n"
 			    "Treasury contains %d gold."),
 			  utype_name_translation(ut1),
 			  utype_name_translation(ut2),
-			  unit_upgrade_price(game.player_ptr, ut1, ut2),
-			  game.player_ptr->economic.gold);    
+			  unit_upgrade_price(client.conn.playing, ut1, ut2),
+			  client.conn.playing->economic.gold);
 
 	      popup_message_dialog(NULL, 
 				   /*"upgradedialog"*/
@@ -597,11 +608,10 @@ activeunits_report_dialog_update(void)
 {
   struct repoinfo {
     int active_count;
-    int upkeep_shield;
-    int upkeep_food;
-    /* int upkeep_gold;   FIXME: add gold when gold is implemented --jjm */
+    int upkeep[O_LAST];
     int building_count;
-  };                         
+  };
+
   if(activeunits_dlg) {
     HWND lv;
     int    i, k, can;
@@ -621,46 +631,55 @@ activeunits_report_dialog_update(void)
     }
 
     memset(unitarray, '\0', sizeof(unitarray));
-    unit_list_iterate(game.player_ptr->units, punit) {
-      (unitarray[unit_type(punit)->index].active_count)++;
-      if (punit->homecity) {
-        unitarray[unit_type(punit)->index].upkeep_shield += punit->upkeep[O_SHIELD];
-        unitarray[unit_type(punit)->index].upkeep_food += punit->upkeep[O_FOOD];
-	/* TODO: gold upkeep */
-      }
-    }
 
-    unit_list_iterate_end;
-    city_list_iterate(game.player_ptr->cities,pcity) {
-      if (pcity->production.is_unit) {
-        (unitarray[pcity->production.value].building_count)++;
+   city_list_iterate(client.conn.playing->cities, pcity) {
+      unit_list_iterate(client.conn.playing->units, punit) {
+        Unit_type_id uti = utype_index(unit_type(punit));
+
+        (unitarray[uti].active_count)++;
+        if (punit->homecity) {
+          output_type_iterate(o) {
+            unitarray[uti].upkeep[o] += punit->upkeep[o];
+          } output_type_iterate_end;
+        }
+      } unit_list_iterate_end;
+   } city_list_iterate_end;
+
+    city_list_iterate(client.conn.playing->cities, pcity) {
+      if (VUT_UTYPE == pcity->production.kind) {
+        struct unit_type *punittype = pcity->production.value.utype;
+        (unitarray[utype_index(punittype)].building_count)++;
       }
     }
     city_list_iterate_end;
 
     k = 0;
     memset(&unittotals, '\0', sizeof(unittotals));
-    unit_type_iterate(i) {
-      if ((unitarray[i->index].active_count > 0)
-	  || (unitarray[i->index].building_count > 0)) {
-        can = (can_upgrade_unittype(game.player_ptr, i) != NULL);
-        my_snprintf(buf[0], sizeof(buf[0]), "%s", utype_name_translation(i));
+    unit_type_iterate(putype) {
+      int index = utype_index(putype);
+      if ((unitarray[index].active_count > 0)
+	  || (unitarray[index].building_count > 0)) {
+        can = (can_upgrade_unittype(client.conn.playing, putype) != NULL);
+        my_snprintf(buf[0], sizeof(buf[0]), "%s",
+                    utype_name_translation(putype));
         my_snprintf(buf[1], sizeof(buf[1]), "%c", can ? '*': '-');
         my_snprintf(buf[2], sizeof(buf[2]), "%3d",
-				   unitarray[i->index].building_count);
+                    unitarray[index].building_count);
         my_snprintf(buf[3], sizeof(buf[3]), "%3d",
-				   unitarray[i->index].active_count);
+                    unitarray[index].active_count);
         my_snprintf(buf[4], sizeof(buf[4]), "%3d",
-				   unitarray[i->index].upkeep_shield);
+                    unitarray[index].upkeep[O_SHIELD]);
         my_snprintf(buf[5], sizeof(buf[5]), "%3d",
-				   unitarray[i->index].upkeep_food);
-	fcwin_listview_add_row(lv,k,AU_COL,row);
-        activeunits_type[k]=(unitarray[i->index].active_count > 0) ? i : NULL;
+                    unitarray[index].upkeep[O_FOOD]);
+        /* TODO: add upkeep[O_GOLD] here */
+        fcwin_listview_add_row(lv,k,AU_COL,row);
+        activeunits_type[k]=(unitarray[index].active_count > 0) ? putype : NULL;
         k++;
-        unittotals.active_count += unitarray[i->index].active_count;
-        unittotals.upkeep_shield += unitarray[i->index].upkeep_shield;
-        unittotals.upkeep_food += unitarray[i->index].upkeep_food;
-        unittotals.building_count += unitarray[i->index].building_count;
+        unittotals.active_count += unitarray[index].active_count;
+        output_type_iterate(o) {
+          unittotals.upkeep[0] = unitarray[index].upkeep[o];
+        } output_type_iterate_end;
+        unittotals.building_count += unitarray[index].building_count;
       }
     } unit_type_iterate_end;
 
@@ -668,8 +687,9 @@ activeunits_report_dialog_update(void)
     buf[1][0]='\0';
     my_snprintf(buf[2],sizeof(buf[2]),"%d",unittotals.building_count);
     my_snprintf(buf[3],sizeof(buf[3]),"%d",unittotals.active_count);
-    my_snprintf(buf[4],sizeof(buf[4]),"%d",unittotals.upkeep_shield);
-    my_snprintf(buf[5],sizeof(buf[5]),"%d",unittotals.upkeep_food);  
+    my_snprintf(buf[4],sizeof(buf[4]),"%d",unittotals.upkeep[O_SHIELD]);
+    my_snprintf(buf[5],sizeof(buf[5]),"%d",unittotals.upkeep[O_FOOD]);
+    /* TODO: add upkeep[O_GOLD] here */
     fcwin_listview_add_row(lv,k,AU_COL,row);
     EnableWindow(GetDlgItem(activeunits_dlg,ID_MILITARY_UPGRADE),FALSE);
     ListView_SetColumnWidth(lv,0,LVSCW_AUTOSIZE);
@@ -816,10 +836,7 @@ static LONG CALLBACK OptionsWndProc(HWND hWnd,
 	    int val = Button_GetState(GetDlgItem(tab_wnds[tab],
 				      ID_OPTIONS_BASE + i)) == BST_CHECKED;
 	    if (val != o->val) {
-	      char buffer[MAX_LEN_MSG];
-	      my_snprintf(buffer, MAX_LEN_MSG, "/set %s %d",
-			  o->name, val);
-	      send_chat(buffer);
+	      send_chat_printf("/set %s %d", o->name, val);
 	    }
 	  } else if (SSET_INT == o->stype) {
 	    char buf[512];
@@ -828,20 +845,14 @@ static LONG CALLBACK OptionsWndProc(HWND hWnd,
 				    512);
 	    val = atoi(buf);
 	    if (val != o->val) {
-	      char buffer[MAX_LEN_MSG];
-	      my_snprintf(buffer, MAX_LEN_MSG, "/set %s %d",
-			  o->name, val);
-	      send_chat(buffer);
-	    }	    
+              send_chat_printf("/set %s %d", o->name, val);
+	    }
 	  } else {
 	    char strval[512];
 	    Edit_GetText(GetDlgItem(tab_wnds[tab], ID_OPTIONS_BASE + i),
 			 strval, 512);
 	    if (strcmp(strval, o->strval) != 0) {
-	      char buffer[MAX_LEN_MSG];
-	      my_snprintf(buffer, MAX_LEN_MSG, "/set %s %s",
-			  o->name, strval);
-	      send_chat(buffer);
+              send_chat_printf("/set %s %s", o->name, strval);
 	    }
 	  }
 	}
