@@ -32,31 +32,39 @@
 #include <X11/Xaw/Toggle.h>     
 #include <X11/IntrinsicP.h>
 
-
+/* utility */
 #include "fcintl.h"
+#include "log.h"
 #include "mem.h"
 #include "genlist.h"
 #include "shared.h"
 #include "support.h"
 
+/* common */
 #include "city.h"
+#include "connection.h"	/* can_conn_edit */
 #include "game.h"
 #include "map.h"
 #include "packets.h"
 #include "player.h"
+#include "specialist.h"
 #include "unitlist.h"
 
+/* client */
+#include "client_main.h"
+#include "climap.h"
+#include "climisc.h"
+#include "control.h"	/* request_xxx and set_unit_focus */
+#include "options.h"
+#include "text.h"
+#include "tilespec.h"
+
 #include "cma_fec.h"
 
-#include "pixcomm.h"
 #include "canvas.h"
-
 #include "cityrep.h"
-#include "citydlg.h"
-#include "civclient.h"
 #include "cma_fe.h"
 #include "colors.h"
-#include "control.h" /* request_xxx and set_unit_focus */
 #include "dialogs.h"
 #include "graphics.h"
 #include "gui_main.h"
@@ -65,17 +73,13 @@
 #include "inputdlg.h"
 #include "mapctrl.h"
 #include "mapview.h"
-#include "optiondlg.h"		/* for toggle_callback */
+#include "optiondlg.h"	/* for toggle_callback */
+#include "pixcomm.h"
 #include "repodlgs.h"
 #include "wldlg.h"
 
 #include "citydlg_common.h"
-#include "climap.h"
-#include "climisc.h"
-#include "clinet.h"
-#include "options.h"
-#include "text.h"
-#include "tilespec.h"
+#include "citydlg.h"
 
 #define MIN_NUM_CITIZENS	22
 #define MAX_NUM_CITIZENS	50
@@ -195,12 +199,25 @@ static void get_contents_of_pollution(struct city_dialog *pdialog,
 {
   struct city *pcity;
   int pollution=0;
+  int corruption=0;
+  int waste=0;
+  int illness=0;
+  char buf[512];
 
   if (pdialog) {
     pcity=pdialog->pcity;
+    corruption=pcity->waste[O_TRADE];
+    waste=pcity->waste[O_SHIELD];
     pollution=pcity->pollution;
+    if (!game.info.illness_on) {
+      my_snprintf(buf, sizeof(buf), " -.-");
+    } else {
+      illness = city_illness_calc(pcity, NULL, NULL, NULL, NULL);
+      /* illness is in tenth of percent */
+      my_snprintf(buf, sizeof(buf), "%4.1f",
+                  (float)illness / 10.0);
+    }
   }
-
   my_snprintf(retbuf, n, _("Pollution:    %3d"), pollution);
 }
 
@@ -213,14 +230,29 @@ static void get_contents_of_storage(struct city_dialog *pdialog,
   struct city *pcity;
   int foodstock=0;
   int foodbox=0;
+  int granaryturns=0;
+  char buf[512];
 
   if (pdialog) {
     pcity=pdialog->pcity;
     foodstock=pcity->food_stock;
     foodbox=city_granary_size(pcity->size);
+    granaryturns = city_turns_to_grow(pcity);
+    if (granaryturns == 0) {
+      my_snprintf(buf, sizeof(buf), _("blocked"));
+    } else if (granaryturns == FC_INFINITY) {
+      my_snprintf(buf, sizeof(buf), _("never"));
+    } else {
+      /* A negative value means we'll have famine in that many turns.
+         But that's handled down below. */
+      my_snprintf(buf, sizeof(buf),
+                  PL_("%d turn", "%d turns", abs(granaryturns)),
+                  abs(granaryturns));
+    }
   }
 
   /* We used to mark cities with a granary with a "*" here. */
+  /* FIXME: actually use the "Change in:" info in buf[] */
   my_snprintf(retbuf, n, _("Granary:  %3d/%-3d"),
 	      foodstock, foodbox);
 }
@@ -371,7 +403,9 @@ void refresh_city_dialog(struct city *pcity)
     XtSetSensitive(pdialog->cma_command, True);
     XtSetSensitive(pdialog->cityopt_command, True);
   }
-  if (!game.player_ptr || city_owner(pcity) == game.player_ptr) {
+
+  if (NULL == client.conn.playing
+      || city_owner(pcity) == client.conn.playing) {
     city_report_dialog_update_city(pcity);
     economy_report_dialog_update();
   } else {
@@ -398,8 +432,8 @@ void refresh_unit_city_dialogs(struct unit *punit)
   struct city *pcity_sup, *pcity_pre;
   struct city_dialog *pdialog;
 
-  pcity_sup=player_find_city_by_id(game.player_ptr, punit->homecity);
-  pcity_pre=tile_get_city(punit->tile);
+  pcity_sup = player_find_city_by_id(client.conn.playing, punit->homecity);
+  pcity_pre=tile_city(punit->tile);
   
   if(pcity_sup && (pdialog=get_city_dialog(pcity_sup)))
     city_dialog_update_supported_units(pdialog, 0);
@@ -1139,15 +1173,20 @@ static void present_units_ok_callback(Widget w, XtPointer client_data,
 static void present_units_activate_callback(Widget w, XtPointer client_data, 
 					    XtPointer call_data)
 {
-  struct unit *punit;
-  struct city *pcity;
-  struct city_dialog *pdialog;
+  struct unit *punit =
+    player_find_unit_by_id(client.conn.playing, (size_t)client_data);
+
+  if (NULL != punit) {
+    struct city *pcity = tile_city(punit->tile);
 
-  if((punit=player_find_unit_by_id(game.player_ptr, (size_t)client_data)))  {
     set_unit_focus(punit);
-    if((pcity=tile_get_city(punit->tile)))
-      if((pdialog=get_city_dialog(pcity)))
+    if (NULL != pcity) {
+      struct city_dialog *pdialog = get_city_dialog(pcity);
+
+      if (NULL != pdialog) {
 	city_dialog_update_present_units(pdialog, 0);
+      }
+    }
   }
 
   destroy_message_dialog(w);
@@ -1160,15 +1199,20 @@ static void present_units_activate_callback(Widget w, XtPointer client_data,
 static void supported_units_activate_callback(Widget w, XtPointer client_data, 
 					      XtPointer call_data)
 {
-  struct unit *punit;
-  struct city *pcity;
-  struct city_dialog *pdialog;
+  struct unit *punit =
+    player_find_unit_by_id(client.conn.playing, (size_t)client_data);
+
+  if (NULL != punit) {
+    struct city *pcity = tile_city(punit->tile);
 
-  if((punit=player_find_unit_by_id(game.player_ptr, (size_t)client_data)))  {
     set_unit_focus(punit);
-    if((pcity=tile_get_city(punit->tile)))
-      if((pdialog=get_city_dialog(pcity)))
+    if (NULL != pcity) {
+      struct city_dialog *pdialog = get_city_dialog(pcity);
+
+      if (NULL != pdialog) {
 	city_dialog_update_supported_units(pdialog, 0);
+      }
+    }
   }
 
   destroy_message_dialog(w);
@@ -1182,17 +1226,22 @@ static void present_units_activate_close_callback(Widget w,
 						  XtPointer client_data, 
 						  XtPointer call_data)
 {
-  struct unit *punit;
-  struct city *pcity;
-  struct city_dialog *pdialog;
+  struct unit *punit =
+    player_find_unit_by_id(client.conn.playing, (size_t)client_data);
 
   destroy_message_dialog(w);
 
-  if((punit=player_find_unit_by_id(game.player_ptr, (size_t)client_data)))  {
+  if (NULL != punit) {
+    struct city *pcity = tile_city(punit->tile);
+
     set_unit_focus(punit);
-    if((pcity=tile_get_city(punit->tile)))
-      if((pdialog=get_city_dialog(pcity)))
+    if (NULL != pcity) {
+      struct city_dialog *pdialog = get_city_dialog(pcity);
+
+      if (NULL != pdialog) {
 	close_city_dialog(pdialog);
+      }
+    }
   }
 }
 
@@ -1203,17 +1252,23 @@ static void supported_units_activate_close_callback(Widget w,
 						    XtPointer client_data, 
 						    XtPointer call_data)
 {
-  struct unit *punit;
-  struct city *pcity;
-  struct city_dialog *pdialog;
+  struct unit *punit =
+    player_find_unit_by_id(client.conn.playing, (size_t)client_data);
 
   destroy_message_dialog(w);
 
-  if((punit=player_find_unit_by_id(game.player_ptr, (size_t)client_data)))  {
+  if (NULL != punit) {
+    struct city *pcity =
+      player_find_city_by_id(client.conn.playing, punit->homecity);
+
     set_unit_focus(punit);
-    if((pcity=player_find_city_by_id(game.player_ptr, punit->homecity)))
-      if((pdialog=get_city_dialog(pcity)))
+    if (NULL != pcity) {
+      struct city_dialog *pdialog = get_city_dialog(pcity);
+
+      if (NULL != pdialog) {
 	close_city_dialog(pdialog);
+      }
+    }
   }
 }
 
@@ -1224,10 +1279,12 @@ static void supported_units_activate_close_callback(Widget w,
 static void present_units_sentry_callback(Widget w, XtPointer client_data, 
 					   XtPointer call_data)
 {
-  struct unit *punit;
+  struct unit *punit =
+    player_find_unit_by_id(client.conn.playing, (size_t)client_data);
 
-  if((punit=player_find_unit_by_id(game.player_ptr, (size_t)client_data)))
+  if (NULL != punit) {
     request_unit_sentry(punit);
+  }
 
   destroy_message_dialog(w);
 }
@@ -1239,10 +1296,12 @@ static void present_units_sentry_callback(Widget w, XtPointer client_data,
 static void present_units_fortify_callback(Widget w, XtPointer client_data, 
 					   XtPointer call_data)
 {
-  struct unit *punit;
+  struct unit *punit =
+    player_find_unit_by_id(client.conn.playing, (size_t)client_data);
 
-  if((punit=player_find_unit_by_id(game.player_ptr, (size_t)client_data)))
+  if (NULL != punit) {
     request_unit_fortify(punit);
+  }
 
   destroy_message_dialog(w);
 }
@@ -1254,10 +1313,12 @@ static void present_units_fortify_callback(Widget w, XtPointer client_data,
 static void present_units_disband_callback(Widget w, XtPointer client_data, 
 					   XtPointer call_data)
 {
-  struct unit *punit;
+  struct unit *punit =
+    player_find_unit_by_id(client.conn.playing, (size_t)client_data);
 
-  if((punit=player_find_unit_by_id(game.player_ptr, (size_t)client_data)))
+  if (NULL != punit) {
     request_unit_disband(punit);
+  }
 
   destroy_message_dialog(w);
 }
@@ -1269,10 +1330,12 @@ static void present_units_disband_callback(Widget w, XtPointer client_data,
 static void present_units_homecity_callback(Widget w, XtPointer client_data, 
 					    XtPointer call_data)
 {
-  struct unit *punit;
-  
-  if((punit=player_find_unit_by_id(game.player_ptr, (size_t)client_data)))
+  struct unit *punit =
+    player_find_unit_by_id(client.conn.playing, (size_t)client_data);
+
+  if (NULL != punit) {
     request_unit_change_homecity(punit);
+  }
 
   destroy_message_dialog(w);
 }
@@ -1294,15 +1357,19 @@ static void present_units_cancel_callback(Widget w, XtPointer client_data,
 void present_units_callback(Widget w, XtPointer client_data, 
 			    XtPointer call_data)
 {
-  struct unit *punit;
-  struct city *pcity;
-  struct city_dialog *pdialog;
   Widget wd;
+  struct city_dialog *pdialog;
+  struct city *pcity;
   XEvent *e = (XEvent*)call_data;
+  struct unit *punit =
+    player_find_unit_by_id(client.conn.playing, (size_t)client_data);
   
-  if((punit=player_find_unit_by_id(game.player_ptr, (size_t)client_data)) &&
-     (pcity=tile_get_city(punit->tile)) &&
-     (pdialog=get_city_dialog(pcity))) {
+  if ((NULL != punit
+       || (can_conn_edit(&client.conn)
+           && NULL == client.conn.playing
+           && (punit = game_find_unit_by_number((size_t)client_data))))
+      && (pcity = tile_city(punit->tile))
+      && (pdialog = get_city_dialog(pcity))) {
     
     if(e->type==ButtonRelease && e->xbutton.button==Button2)  {
       set_unit_focus(punit);
@@ -1341,8 +1408,8 @@ void present_units_callback(Widget w, XtPointer client_data,
     if (punit->homecity == pcity->id) {
       XtSetSensitive(XtNameToWidget(wd, "*button5"), FALSE);
     }
-    if (!game.player_ptr
-	|| (can_upgrade_unittype(game.player_ptr, unit_type(punit)) == NULL)) {
+    if (NULL == client.conn.playing
+	|| (NULL == can_upgrade_unittype(client.conn.playing, unit_type(punit)))) {
       XtSetSensitive(XtNameToWidget(wd, "*button6"), FALSE);
     }
   }
@@ -1410,7 +1477,7 @@ void trade_callback(Widget w, XtPointer client_data, XtPointer call_data)
 	      city_name(pdialog->pcity));
   bptr = end_of_strn(bptr, &nleft);
   
-  for (i = 0; i < NUM_TRADEROUTES; i++)
+  for (i = 0; i < NUM_TRADE_ROUTES; i++)
     if(pdialog->pcity->trade[i]) {
       struct city *pcity;
       x=1;
@@ -1473,9 +1540,7 @@ void city_dialog_update_building(struct city_dialog *pdialog)
   XtSetSensitive(pdialog->sell_command, !pcity->did_sell);
 
   xaw_set_label(pdialog->building_label,
-		pcity->production.is_unit ?
-		  utype_name_translation(utype_by_number(pcity->production.value)) :
-		  get_impr_name_ex(pcity, pcity->production.value));
+		city_production_name_translation(pcity));
 
   get_contents_of_progress(pdialog, buf, sizeof(buf));
   xaw_set_label(pdialog->progress_label, buf);
@@ -1550,7 +1615,8 @@ void city_dialog_update_citizens(struct city_dialog *pdialog)
   if (i >= pdialog->num_citizens_shown && i < num_citizens) {
     i = pdialog->num_citizens_shown - 1;
     /* FIXME: what about the mask? */
-    xaw_set_bitmap(pdialog->citizen_labels[i], get_arrow_sprite(tileset)->pixmap);
+    xaw_set_bitmap(pdialog->citizen_labels[i],
+		   get_arrow_sprite(tileset, ARROW_RIGHT)->pixmap);
     XtSetSensitive(pdialog->citizen_labels[i], FALSE);
     XtRemoveAllCallbacks(pdialog->citizen_labels[i], XtNcallback);
     return;
@@ -1569,15 +1635,18 @@ void city_dialog_update_citizens(struct city_dialog *pdialog)
 static void support_units_callback(Widget w, XtPointer client_data, 
 				   XtPointer call_data)
 {
-  struct unit *punit;
-  struct city *pcity;
-  struct city_dialog *pdialog;
-  XEvent *e = (XEvent*)call_data;
   Widget wd;
+  XEvent *e = (XEvent*)call_data;
+  struct unit *punit =
+    player_find_unit_by_id(client.conn.playing, (size_t)client_data);
 
-  if((punit=player_find_unit_by_id(game.player_ptr, (size_t)client_data)))
-    if((pcity=game_find_city_by_number(punit->homecity)))
-      if((pdialog=get_city_dialog(pcity)))  {
+  if (NULL != punit) {
+    struct city *pcity = game_find_city_by_number(punit->homecity);
+
+    if (NULL != pcity) {
+      struct city_dialog *pdialog = get_city_dialog(pcity);
+
+      if ( NULL != pdialog) {
 	if(e->type==ButtonRelease && e->xbutton.button==Button2)  {
 	  set_unit_focus(punit);
 	  close_city_dialog(pdialog);
@@ -1600,6 +1669,8 @@ static void support_units_callback(Widget w, XtPointer client_data,
           XtSetSensitive(XtNameToWidget(wd, "*button3"), FALSE);
         }
       }
+    }
+  }
 }
 
 
@@ -1646,8 +1717,10 @@ void city_dialog_update_supported_units(struct city_dialog *pdialog,
   struct unit_list *plist;
   int i, j, adj_base;
   Widget pixcomm;
+  int free_unhappy = get_city_bonus(pdialog->pcity, EFT_MAKE_CONTENT_MIL);
 
-  if (game.player_ptr && (city_owner(pdialog->pcity) != game.player_ptr)) {
+  if (NULL != client.conn.playing
+      && city_owner(pdialog->pcity) != client.conn.playing) {
     plist = pdialog->pcity->info_units_supported;
   } else {
     plist = pdialog->pcity->units_supported;
@@ -1663,6 +1736,7 @@ void city_dialog_update_supported_units(struct city_dialog *pdialog,
   j = 0; /* index into list */
   unit_list_iterate(plist, punit) {
     struct canvas store;
+    int happy_cost = city_unit_unhappiness(punit, &free_unhappy);
 
     if (j++ < pdialog->support_unit_base) {
       continue;
@@ -1680,7 +1754,8 @@ void city_dialog_update_supported_units(struct city_dialog *pdialog,
     XawPixcommClear(pixcomm); /* STG */
     put_unit(punit, &store, 0, 0);
     put_unit_pixmap_city_overlays(punit,
-				  XawPixcommPixmap(pixcomm));
+                                  XawPixcommPixmap(pixcomm),
+                                  punit->upkeep, happy_cost);
 
     xaw_expose_now(pixcomm);
 
@@ -1707,7 +1782,8 @@ void city_dialog_update_present_units(struct city_dialog *pdialog, int unitid)
   int i, j, adj_base;
   Widget pixcomm;
 
-  if (game.player_ptr && (city_owner(pdialog->pcity) != game.player_ptr)) {
+  if (NULL != client.conn.playing
+      && city_owner(pdialog->pcity) != client.conn.playing) {
     plist = pdialog->pcity->info_units_present;
   } else {
     plist = pdialog->pcity->tile->units;
@@ -1784,16 +1860,16 @@ void city_dialog_update_improvement_list(struct city_dialog *pdialog)
 {
   int n = 0, flag = 0;
 
-  built_impr_iterate(pdialog->pcity, i) {
+  city_built_iterate(pdialog->pcity, pimprove) {
     if (!pdialog->improvlist_names_ptrs[n] ||
 	strcmp(pdialog->improvlist_names_ptrs[n],
-	       get_impr_name_ex(pdialog->pcity, i)) != 0)
+	       city_improvement_name_translation(pdialog->pcity, pimprove)) != 0)
       flag = 1;
     sz_strlcpy(pdialog->improvlist_names[n],
-	       get_impr_name_ex(pdialog->pcity, i));
+	       city_improvement_name_translation(pdialog->pcity, pimprove));
     pdialog->improvlist_names_ptrs[n] = pdialog->improvlist_names[n];
     n++;
-  } built_impr_iterate_end;
+  } city_built_iterate_end;
   
   if(pdialog->improvlist_names_ptrs[n]!=0) {
     pdialog->improvlist_names_ptrs[n]=0;
@@ -1866,29 +1942,19 @@ static void buy_callback_no(Widget w, XtPointer client_data,
 *****************************************************************/
 void buy_callback(Widget w, XtPointer client_data, XtPointer call_data)
 {
-  struct city_dialog *pdialog;
-  int value;
-  const char *name;
   char buf[512];
+  struct city_dialog *pdialog = (struct city_dialog *)client_data;;
+  const char *name = city_production_name_translation(pdialog->pcity);
+  int value = city_production_buy_gold_cost(pdialog->pcity);
   
-  pdialog=(struct city_dialog *)client_data;
-
   if (!can_client_issue_orders()) {
     return;
   }
 
-  if(pdialog->pcity->production.is_unit) {
-    name=utype_name_translation(utype_by_number(pdialog->pcity->production.value));
-  }
-  else {
-    name=get_impr_name_ex(pdialog->pcity, pdialog->pcity->production.value);
-  }
-  value=city_buy_cost(pdialog->pcity);
- 
-  if (game.player_ptr->economic.gold>=value) {
+  if (value <= client.conn.playing->economic.gold) {
     my_snprintf(buf, sizeof(buf),
 		_("Buy %s for %d gold?\nTreasury contains %d gold."), 
-		name, value, game.player_ptr->economic.gold);
+		name, value, client.conn.playing->economic.gold);
     popup_message_dialog(pdialog->shell, "buydialog", buf,
 			 buy_callback_yes, pdialog, 0,
 			 buy_callback_no, 0, 0,
@@ -1897,7 +1963,7 @@ void buy_callback(Widget w, XtPointer client_data, XtPointer call_data)
   else {
     my_snprintf(buf, sizeof(buf),
 		_("%s costs %d gold.\nTreasury contains %d gold."), 
-		name, value, game.player_ptr->economic.gold);
+		name, value, client.conn.playing->economic.gold);
     popup_message_dialog(pdialog->shell, "buynodialog", buf,
 			 buy_callback_no, 0, 0,
 			 NULL);
@@ -1911,16 +1977,18 @@ void buy_callback(Widget w, XtPointer client_data, XtPointer call_data)
 *****************************************************************/
 void unitupgrade_callback_yes(Widget w, XtPointer client_data, XtPointer call_data)
 {
-  struct unit *punit;
+  struct unit *punit =
+    player_find_unit_by_id(client.conn.playing, (size_t)client_data);
 
   /* Is it right place for breaking? -ev */
   if (!can_client_issue_orders()) {
     return;
   }
 
-  if((punit=player_find_unit_by_id(game.player_ptr, (size_t)client_data))) {
+  if (NULL != punit) {
     request_unit_upgrade(punit);
   }
+
   destroy_message_dialog(w);
 }
 
@@ -1939,9 +2007,9 @@ void unitupgrade_callback_no(Widget w, XtPointer client_data, XtPointer call_dat
 *****************************************************************/
 void upgrade_callback(Widget w, XtPointer client_data, XtPointer call_data)
 {
-  struct unit *punit = player_find_unit_by_id(game.player_ptr,
-					      (size_t)client_data);
   char buf[512];
+  struct unit *punit = player_find_unit_by_id(client.conn.playing,
+					      (size_t)client_data);
 
   if (!punit) {
     return;
@@ -1976,9 +2044,10 @@ static void change_to_callback(Widget w, XtPointer client_data,
   ret=XawListShowCurrent(pdialog->change_list);
 
   if (ret->list_index != XAW_LIST_NONE) {
-    struct city_production target;
-    target.is_unit = ret->list_index >= pdialog->change_list_num_improvements;
-    target.value = pdialog->change_list_ids[ret->list_index];
+    struct universal target =
+      universal_by_number((ret->list_index >= pdialog->change_list_num_improvements)
+			     ? VUT_UTYPE : VUT_IMPROVEMENT,
+			     pdialog->change_list_ids[ret->list_index]);
 
     city_change_production(pdialog->pcity, target);
   }
@@ -2019,10 +2088,10 @@ static void change_help_callback(Widget w, XtPointer client_data,
 
     if (is_unit) {
       popup_help_dialog_typed(utype_name_translation(utype_by_number(idx)), HELP_UNIT);
-    } else if(is_great_wonder(idx)) {
-      popup_help_dialog_typed(improvement_name_translation(idx), HELP_WONDER);
+    } else if (is_great_wonder(improvement_by_number(idx))) {
+      popup_help_dialog_typed(improvement_name_translation(improvement_by_number(idx)), HELP_WONDER);
     } else {
-      popup_help_dialog_typed(improvement_name_translation(idx), HELP_IMPROVEMENT);
+      popup_help_dialog_typed(improvement_name_translation(improvement_by_number(idx)), HELP_IMPROVEMENT);
     }
   }
   else
@@ -2058,7 +2127,7 @@ void change_callback(Widget w, XtPointer client_data, XtPointer call_data)
   Position x, y;
   Dimension width, height;
   struct city_dialog *pdialog;
-  struct city_production production;
+  struct universal production;
   int n;
   
   pdialog=(struct city_dialog *)client_data;
@@ -2131,30 +2200,30 @@ void change_callback(Widget w, XtPointer client_data, XtPointer call_data)
   XtSetSensitive(pdialog->shell, FALSE);
 
   n = 0;
-  impr_type_iterate(i) {
-    if(can_build_improvement(pdialog->pcity, i)) {
-      production.is_unit = false;
-      production.value = i;
+  improvement_iterate(pimprove) {
+    if(can_city_build_improvement_now(pdialog->pcity, pimprove)) {
+      production.kind = VUT_IMPROVEMENT;
+      production.value.building = pimprove;
       get_city_dialog_production_full(pdialog->change_list_names[n],
                                       sizeof(pdialog->change_list_names[n]),
                                       production, pdialog->pcity);
       pdialog->change_list_names_ptrs[n]=pdialog->change_list_names[n];
-      pdialog->change_list_ids[n++]=i;
+      pdialog->change_list_ids[n++] = improvement_number(pimprove);
     }
-  } impr_type_iterate_end;
+  } improvement_iterate_end;
   
   pdialog->change_list_num_improvements=n;
 
 
-  unit_type_iterate(i) {
-    if(can_build_unit(pdialog->pcity, i)) {
-      production.is_unit = true;
-      production.value = i->index;
+  unit_type_iterate(punittype) {
+    if (can_city_build_unit_now(pdialog->pcity, punittype)) {
+      production.kind = VUT_UTYPE;
+      production.value.utype = punittype;
       get_city_dialog_production_full(pdialog->change_list_names[n],
                                       sizeof(pdialog->change_list_names[n]),
                                       production, pdialog->pcity);
       pdialog->change_list_names_ptrs[n]=pdialog->change_list_names[n];
-      pdialog->change_list_ids[n++] = i->index;
+      pdialog->change_list_ids[n++] = utype_number(punittype);
     }
   } unit_type_iterate_end;
   
@@ -2181,7 +2250,7 @@ void worklist_callback(Widget w, XtPointer client_data, XtPointer call_data)
     XtPopup(pdialog->worklist_shell, XtGrabNone);
   } else {
     pdialog->worklist_shell = 
-      popup_worklist(&pdialog->pcity->worklist, pdialog->pcity,
+      popup_worklist(pdialog->pcity, NULL,
 		     pdialog->shell, (void *) pdialog, commit_city_worklist,
 		     cancel_city_worklist);
   }
@@ -2193,57 +2262,8 @@ void worklist_callback(Widget w, XtPointer client_data, XtPointer call_data)
 void commit_city_worklist(struct worklist *pwl, void *data)
 {
   struct city_dialog *pdialog = data;
-  struct city_production production;
-  int k;
 
-  /* Update the worklist.  Remember, though -- the current build 
-     target really isn't in the worklist; don't send it to the 
-     server as part of the worklist.  Of course, we have to
-     search through the current worklist to find the first
-     _now_available_ build target (to cope with players who try
-     mean things like adding a Battleship to a city worklist when
-     the player doesn't even yet have the Map Making tech).  */
-
-  for (k = 0; k < MAX_LEN_WORKLIST; k++) {
-    int same_as_current_build;
-    if (!worklist_peek_ith(pwl, &production, k)) {
-      break;
-    }
-    same_as_current_build = 
-      (production.value == pdialog->pcity->production.value)
-      && (production.is_unit == pdialog->pcity->production.is_unit);
-
-    /* Very special case: If we are currently building a wonder we
-       allow the construction to continue, even if we the wonder is
-       finished elsewhere, ie unbuildable. */
-    if (k == 0 && !production.is_unit && is_great_wonder(production.value)
-	&& same_as_current_build) {
-      worklist_remove(pwl, k);
-      break;
-    }
-
-    /* If it can be built... */
-    if ((production.is_unit
-	 && can_build_unit(pdialog->pcity, utype_by_number(production.value)))
-	|| (!production.is_unit
-	    && can_build_improvement(pdialog->pcity, production.value))) {
-      /* ...but we're not yet building it, then switch. */
-      if (!same_as_current_build) {
-	/* Change the current target */
-	city_change_production(pdialog->pcity, production);
-      }
-
-      /* This item is now (and may have always been) the current
-	 build target.  Drop it out of the worklist. */
-      worklist_remove(pwl, k);
-      break;
-    }
-  }
-
-  /* Send the rest of the worklist on its way. */
-  city_set_worklist(pdialog->pcity, pwl);
-
-  pdialog->worklist_shell = NULL;
+  city_worklist_commit(pdialog->pcity, pwl);
 }
 
 void cancel_city_worklist(void *data) {
@@ -2289,18 +2309,18 @@ void sell_callback(Widget w, XtPointer client_data, XtPointer call_data)
 
   if(ret->list_index!=XAW_LIST_NONE) {
     int n = 0;
-    built_impr_iterate(pdialog->pcity, i) {
+    city_built_iterate(pdialog->pcity, pimprove) {
       if (n == ret->list_index) {
 	char buf[512];
 
-	if (!can_city_sell_building(pdialog->pcity, i)) {
+	if (!can_city_sell_building(pdialog->pcity, pimprove)) {
 	  return;
 	}
 
-	pdialog->sell_id = i;
+	pdialog->sell_id = improvement_number(pimprove);
 	my_snprintf(buf, sizeof(buf), _("Sell %s for %d gold?"),
-		    get_impr_name_ex(pdialog->pcity, i),
-		    impr_sell_gold(i));
+		    city_improvement_name_translation(pdialog->pcity, pimprove),
+		    impr_sell_gold(pimprove));
 
 	popup_message_dialog(pdialog->shell, "selldialog", buf,
 			     sell_callback_yes, pdialog, 0,
@@ -2309,7 +2329,7 @@ void sell_callback(Widget w, XtPointer client_data, XtPointer call_data)
 	return;
       }
       n++;
-    } built_impr_iterate_end;
+    } city_built_iterate_end;
   }
 }
 
@@ -2332,11 +2352,11 @@ void close_city_dialog(struct city_dialog *pdialog)
   unit_list_iterate(pdialog->pcity->info_units_supported, psunit) {
     free(psunit);
   } unit_list_iterate_end;
-  unit_list_unlink_all(pdialog->pcity->info_units_supported);
+  unit_list_clear(pdialog->pcity->info_units_supported);
   unit_list_iterate(pdialog->pcity->info_units_present, psunit) {
     free(psunit);
   } unit_list_iterate_end;
-  unit_list_unlink_all(pdialog->pcity->info_units_present);
+  unit_list_clear(pdialog->pcity->info_units_present);
 
 /*
   if(pdialog->is_modal)
@@ -2398,7 +2418,11 @@ void cityopt_cancel_command_callback(Widget w, XtPointer client_data,
 void cityopt_newcit_triggle_callback(Widget w, XtPointer client_data,
 					XtPointer call_data);
 
-char *newcitizen_labels[] = { N_("Workers"), N_("Scientists"), N_("Taxmen") };
+char *newcitizen_labels[] = {
+  N_("?city:Workers"),
+  N_("Scientists"),
+  N_("Taxmen")
+};
 
 static Widget cityopt_shell = 0;
 static Widget cityopt_triggle;
@@ -2575,7 +2599,7 @@ void cityopt_ok_command_callback(Widget w, XtPointer client_data,
     } else if (newcitizen_index == 2) {
       BV_SET(new_options, CITYO_NEW_TAXMAN);
     }
-    dsend_packet_city_options_req(&aconnection, cityopt_city_id,new_options);
+    dsend_packet_city_options_req(&client.conn, cityopt_city_id,new_options);
   }
   XtDestroyWidget(cityopt_shell);
   cityopt_shell = 0;
