@@ -29,9 +29,12 @@
 #include <X11/Xaw/Viewport.h>
 #include <X11/Xaw/Toggle.h>     
 
+/* utility */
+#include "log.h"
 #include "mem.h"
 #include "support.h"
 
+/* common */
 #include "game.h"
 #include "map.h"
 #include "packets.h"
@@ -39,11 +42,12 @@
 #include "unit.h"
 #include "unitlist.h"
 
-#include "civclient.h"
-#include "clinet.h"
-#include "control.h" /* get_unit_in_focus */
+/* client */
+#include "client_main.h"
+#include "control.h"
 #include "goto.h"
 
+/* gui-xaw */
 #include "gui_main.h"
 #include "gui_stuff.h"
 #include "mapctrl.h"
@@ -106,7 +110,7 @@ void popup_goto_dialog(void)
     return;
   }
 
-  no_player_cities = !(city_list_size(game.player_ptr->cities));
+  no_player_cities = !(city_list_size(client.conn.playing->cities));
 
   original_tile = get_center_tile_mapcanvas();
   
@@ -205,7 +209,7 @@ static struct city *get_selected_city(void)
 **************************************************************************/
 void update_goto_dialog(Widget goto_list)
 {
-  int i, j;
+  int j = 0;
   Boolean all_cities;
 
   if (!can_client_issue_orders()) {
@@ -217,18 +221,21 @@ void update_goto_dialog(Widget goto_list)
   cleanup_goto_list();
 
   if(all_cities) {
-    for(i=0, ncities_total=0; i<game.info.nplayers; i++) {
-      ncities_total += city_list_size(game.players[i].cities);
-    }
+    ncities_total = 0;
+    players_iterate(pplayer) {
+      ncities_total += city_list_size(pplayer->cities);
+    } players_iterate_end;
   } else {
-    ncities_total = city_list_size(game.player_ptr->cities);
+    ncities_total = city_list_size(client.conn.playing->cities);
   }
 
   city_name_ptrs=fc_malloc(ncities_total*sizeof(char*));
   
-  for(i=0, j=0; i<game.info.nplayers; i++) {
-    if (!all_cities && i != game.info.player_idx) continue;
-    city_list_iterate(game.players[i].cities, pcity) {
+  players_iterate(pplayer) {
+    if (!all_cities && pplayer != client.conn.playing) {
+      continue;
+    }
+    city_list_iterate(pplayer->cities, pcity) {
       char name[MAX_LEN_NAME+3];
       sz_strlcpy(name, city_name(pcity));
       /* FIXME: should use unit_can_airlift_to(). */
@@ -236,9 +243,8 @@ void update_goto_dialog(Widget goto_list)
 	sz_strlcat(name, "(A)");
       }
       city_name_ptrs[j++]=mystrdup(name);
-    }
-    city_list_iterate_end;
-  }
+    } city_list_iterate_end;
+  } players_iterate_end;
 
   if(ncities_total) {
     qsort(city_name_ptrs, ncities_total, sizeof(char *), compare_strings_ptrs);
