@@ -18,6 +18,7 @@
 #include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <math.h> /* sqrt */
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
@@ -29,9 +30,9 @@
 #include <X11/Xaw/AsciiText.h>
 #include <X11/Xaw/Tree.h>
 
+/* common & utility */
 #include "city.h"
 #include "fcintl.h"
-#include "game.h"
 #include "genlist.h"
 #include "government.h"
 #include "mem.h"
@@ -42,6 +43,8 @@
 #include "support.h"
 #include "version.h"
 
+/* client */
+#include "client_main.h"
 #include "climisc.h"
 #include "dialogs.h"
 #include "graphics.h"
@@ -82,15 +85,12 @@ static Widget help_unit_tile;
 
 static Widget help_terrain_movement_defense, help_terrain_movement_defense_data;
 static Widget help_terrain_food_shield_trade, help_terrain_food_shield_trade_data;
-static Widget help_terrain_special_1, help_terrain_special_1_data;
-static Widget help_terrain_special_2, help_terrain_special_2_data;
+static Widget help_terrain_resources;
 static Widget help_terrain_road_result_time, help_terrain_road_result_time_data;
 static Widget help_terrain_irrigation_result_time, help_terrain_irrigation_result_time_data;
 static Widget help_terrain_mining_result_time, help_terrain_mining_result_time_data;
 static Widget help_terrain_transform_result_time, help_terrain_transform_result_time_data;
 
-/* HACK: we use a static string for convenience. */
-static char long_buffer[64000];
 
 static void create_help_page(enum help_page_type type);
 
@@ -254,7 +254,7 @@ static void create_tech_tree(Widget tree, Widget parent, int tech, int levels)
   char *bg="";
   char label[MAX_LEN_NAME+3];
   
-  type = (tech==A_LAST) ? TECH_UNKNOWN : get_invention(game.player_ptr, tech);
+  type = (tech==A_LAST) ? TECH_UNKNOWN : player_invention_state(client.conn.playing, tech);
   switch(type) {
     case TECH_UNKNOWN:
       bg=TREE_NODE_UNKNOWN_TECH_BG;
@@ -262,13 +262,14 @@ static void create_tech_tree(Widget tree, Widget parent, int tech, int levels)
     case TECH_KNOWN:
       bg=TREE_NODE_KNOWN_TECH_BG;
       break;
-    case TECH_REACHABLE:
+    case TECH_PREREQS_KNOWN:
       bg=TREE_NODE_REACHABLE_TECH_BG;
       break;
   }
   
   if(tech==A_LAST ||
-     (advances[tech].req[0]==A_LAST && advances[tech].req[1]==A_LAST))  {
+     (advance_required(tech, AR_ONE)==A_LAST
+     && advance_required(tech, AR_TWO)==A_LAST))  {
     sz_strlcpy(label, _("Removed"));
     bg=TREE_NODE_REMOVED_TECH_BG;
     l=XtVaCreateManagedWidget("treenode", commandWidgetClass, 
@@ -281,8 +282,8 @@ static void create_tech_tree(Widget tree, Widget parent, int tech, int levels)
   }
   
   my_snprintf(label, sizeof(label),
-	      "%s:%d", advances[tech].name,
-	      num_unknown_techs_for_goal(game.player_ptr, tech));
+	      "%s:%d", advance_name_translation(advance_by_number(tech)),
+	      num_unknown_techs_for_goal(client.conn.playing, tech));
 
   if(parent) {
     l=XtVaCreateManagedWidget("treenode", 
@@ -308,10 +309,10 @@ static void create_tech_tree(Widget tree, Widget parent, int tech, int levels)
 
   
   if(--levels>0) {
-    if(advances[tech].req[0]!=A_NONE)
-      create_tech_tree(tree, l, advances[tech].req[0], levels);
-    if(advances[tech].req[1]!=A_NONE)
-      create_tech_tree(tree, l, advances[tech].req[1], levels);
+    if(advance_required(tech, AR_ONE)!=A_NONE)
+      create_tech_tree(tree, l, advance_required(tech, AR_ONE), levels);
+    if(advance_required(tech, AR_TWO)!=A_NONE)
+      create_tech_tree(tree, l, advance_required(tech, AR_TWO), levels);
   }
   
   
@@ -490,8 +491,8 @@ static void create_help_page(enum help_page_type type)
       XtVaCreateManagedWidget("helpunittile",
 			      labelWidgetClass,
 			      help_right_form,
-			      XtNwidth, UNIT_TILE_WIDTH,
-			      XtNheight, UNIT_TILE_HEIGHT,
+			      XtNwidth, tileset_full_tile_width(tileset),
+			      XtNheight, tileset_full_tile_height(tileset),
 			      NULL);  
     XtAddCallback(help_unit_tile,
                   XtNdestroyCallback,free_bitmap_destroy_callback,
@@ -645,34 +646,14 @@ static void create_help_page(enum help_page_type type)
        help_right_form,
        NULL);
 
-    help_terrain_special_1 =
+    help_terrain_resources =
       XtVaCreateManagedWidget
       (
-       "help_terrain_special_1",
+       "help_terrain_resources",
        labelWidgetClass,
-       help_right_form,
-       NULL);
-    help_terrain_special_1_data =
-      XtVaCreateManagedWidget
-      (
-       "help_terrain_special_1_data",
-       labelWidgetClass,
-       help_right_form,
-       NULL);
-
-    help_terrain_special_2 =
-      XtVaCreateManagedWidget
-      (
-       "help_terrain_special_2",
-       labelWidgetClass,
-       help_right_form,
-       NULL);
-    help_terrain_special_2_data =
-      XtVaCreateManagedWidget
-      (
-       "help_terrain_special_2_data",
-       labelWidgetClass,
-       help_right_form,
+       help_right_form, 
+       XtNwidth, (w2-15)/2,
+       XtNjustify,XtJustifyCenter,
        NULL);
 
     help_terrain_road_result_time =
@@ -741,34 +722,51 @@ static void create_help_page(enum help_page_type type)
 ...
 **************************************************************************/
 static void help_update_improvement(const struct help_item *pitem,
-				    char *title, int which)
+				    char *title)
 {
   char buf[64000];
+  struct impr_type *imp = find_improvement_by_translated_name(title);
   
   create_help_page(HELP_IMPROVEMENT);
-  
-  if (which<game.num_impr_types) {
-    struct impr_type *imp = &improvement_types[which];
-    sprintf(buf, "%d ", impr_build_shield_cost(which));
+
+  if (imp  &&  !is_great_wonder(imp)) {
+    char req_buf[512];
+    int i;
+
+    sprintf(buf, "%d ", impr_build_shield_cost(imp));
     xaw_set_label(help_improvement_cost_data, buf);
     sprintf(buf, "%d ", imp->upkeep);
     xaw_set_label(help_improvement_upkeep_data, buf);
-    if (imp->tech_req == A_LAST) {
+    /*if (imp->tech_req == A_LAST) {
       xaw_set_label(help_improvement_req_data, _("(Never)"));
     } else {
       xaw_set_label(help_improvement_req_data,
-		    advances[imp->tech_req].name);
-    }
-    create_tech_tree(help_tech_tree, 0, imp->tech_req, 3);
-  }
-  else {
+		    advance_name_translation(advance_by_number(imp->tech_req)));
+    }*/
+    
+    /* FIXME: this should show ranges and all the MAX_NUM_REQS reqs. 
+     * Currently it's limited to 1 req but this code is partially prepared
+     * to be extended.  Remember MAX_NUM_REQS is a compile-time
+     * definition. */
+    i = 0;
+    requirement_vector_iterate(&imp->reqs, preq) {
+      xaw_set_label(help_improvement_req_data,
+                    universal_name_translation(&preq->source,
+                                               req_buf, sizeof(req_buf)));
+      i++;
+      break;
+    } requirement_vector_iterate_end;
+/*    create_tech_tree(help_tech_tree, 0,
+                     imp->req[0].source.value.advance, 3);
+*/
+  } else {
     xaw_set_label(help_improvement_cost_data, "0 ");
     xaw_set_label(help_improvement_upkeep_data, "0 ");
     xaw_set_label(help_improvement_req_data, _("(Never)"));
     create_tech_tree(help_tech_tree, 0, A_LAST, 3);
   }
   set_title_topic(pitem);
-  helptext_building(buf, sizeof(buf), which, pitem->text);
+  helptext_building(buf, sizeof(buf), client.conn.playing, pitem->text, imp);
   XtVaSetValues(help_text, XtNstring, buf, NULL);
 }
   
@@ -776,39 +774,52 @@ static void help_update_improvement(const struct help_item *pitem,
 ...
 **************************************************************************/
 static void help_update_wonder(const struct help_item *pitem,
-			       char *title, int which)
+			       char *title)
 {
   char buf[64000];
-  
+  struct impr_type *imp = find_improvement_by_translated_name(title);
+
   create_help_page(HELP_WONDER);
 
-  if (which<game.num_impr_types) {
-    struct impr_type *imp = &improvement_types[which];
-    sprintf(buf, "%d ", impr_build_shield_cost(which));
+  if (imp  &&  is_great_wonder(imp)) {
+    char req_buf[512];
+    int i;
+    struct advance *vap;
+
+    sprintf(buf, "%d ", impr_build_shield_cost(imp));
     xaw_set_label(help_improvement_cost_data, buf);
-    if (imp->tech_req == A_LAST) {
-      xaw_set_label(help_improvement_req_data, _("(Never)"));
-    } else {
+     /* FIXME: this should show ranges and all the MAX_NUM_REQS reqs. 
+      * Currently it's limited to 1 req but this code is partially prepared
+      * to be extended.  Remember MAX_NUM_REQS is a compile-time
+      * definition. */
+    i = 0;
+    requirement_vector_iterate(&imp->reqs, preq) {
       xaw_set_label(help_improvement_req_data,
-		    advances[imp->tech_req].name);
-    }
-    if (tech_exists(imp->obsolete_by)) {
+                    universal_name_translation(&preq->source,
+                                               req_buf, sizeof(req_buf)));
+      i++;
+      break;
+    } requirement_vector_iterate_end;
+
+    vap = valid_advance(imp->obsolete_by);
+    if (vap) {
       xaw_set_label(help_wonder_obsolete_data,
-		    advances[imp->obsolete_by].name);
+		    advance_name_translation(vap));
     } else {
       xaw_set_label(help_wonder_obsolete_data, _("(Never)"));
     }
-    create_tech_tree(help_tech_tree, 0, imp->tech_req, 3);
-  }
-  else {
+/*    create_tech_tree(help_tech_tree, 0, 
+                     imp->req[0].source.value.advance, 3);
+*/
+  } else {
     /* can't find wonder */
     xaw_set_label(help_improvement_cost_data, "0 ");
     xaw_set_label(help_improvement_req_data, _("(Never)"));
     xaw_set_label(help_wonder_obsolete_data, _("None"));
-    create_tech_tree(help_tech_tree, 0, game.num_tech_types, 3); 
+    create_tech_tree(help_tech_tree, 0, advance_count(), 3); 
   }
   set_title_topic(pitem);
-  helptext_building(buf, sizeof(buf), which, pitem->text);
+  helptext_building(buf, sizeof(buf), client.conn.playing, pitem->text, imp);
   XtVaSetValues(help_text, XtNstring, buf, NULL);
 }
 
@@ -816,43 +827,46 @@ static void help_update_wonder(const struct help_item *pitem,
 ...
 **************************************************************************/
 static void help_update_unit_type(const struct help_item *pitem,
-				  char *title, int i)
+				  char *title)
 {
-  char *buf = &long_buffer[0];
+  char buf[64000];
+  struct unit_type *punittype = find_unit_type_by_translated_name(title);
   
   create_help_page(HELP_UNIT);
-  if (i<game.num_unit_types) {
-    struct unit_type *utype = get_unit_type(i);
-    sprintf(buf, "%d ", unit_build_shield_cost(i));
+  if (punittype) {
+/*    struct unit_type *punittype = utype_by_number(i);*/
+    sprintf(buf, "%d ", utype_build_shield_cost(punittype));
     xaw_set_label(help_unit_cost_data, buf);
-    sprintf(buf, "%d ", utype->attack_strength);
+    sprintf(buf, "%d ", punittype->attack_strength);
     xaw_set_label(help_unit_attack_data, buf);
-    sprintf(buf, "%d ", utype->defense_strength);
+    sprintf(buf, "%d ", punittype->defense_strength);
     xaw_set_label(help_unit_def_data, buf);
-    sprintf(buf, "%d ", utype->move_rate/3);
+    sprintf(buf, "%d ", punittype->move_rate/3);
     xaw_set_label(help_unit_move_data, buf);
-    sprintf(buf, "%d ", utype->firepower);
+    sprintf(buf, "%d ", punittype->firepower);
     xaw_set_label(help_unit_fp_data, buf);
-    sprintf(buf, "%d ", utype->hp);
+    sprintf(buf, "%d ", punittype->hp);
     xaw_set_label(help_unit_hp_data, buf);
-    sprintf(buf, "%d ", utype->vision_range);
+    sprintf(buf, "%d ", (int)sqrt((double)punittype->vision_radius_sq));
     xaw_set_label(help_unit_visrange_data, buf);
-    xaw_set_label(help_unit_upkeep_data, helptext_unit_upkeep_str(i));
-    if(utype->tech_requirement==A_LAST) {
+    xaw_set_label(help_unit_upkeep_data,
+		  helptext_unit_upkeep_str(punittype));
+    if (A_NEVER == punittype->require_advance) {
       xaw_set_label(help_improvement_req_data, _("(Never)"));
     } else {
       xaw_set_label(help_improvement_req_data,
-		    advances[utype->tech_requirement].name);
+		    advance_name_for_player(client.conn.playing,
+				  advance_number(punittype->require_advance)));
     }
-    create_tech_tree(help_tech_tree, 0, utype->tech_requirement, 3);
-    if(utype->obsoleted_by==-1) {
+    create_tech_tree(help_tech_tree, 0, advance_number(punittype->require_advance), 3);
+    if (U_NOT_OBSOLETED == punittype->obsoleted_by) {
       xaw_set_label(help_wonder_obsolete_data, _("None"));
     } else {
       xaw_set_label(help_wonder_obsolete_data,
-		    get_unit_type(utype->obsoleted_by)->name);
+		    utype_name_translation(punittype->obsoleted_by));
     }
     /* add text for transport_capacity, fuel, and flags: */
-    helptext_unit(buf, i, pitem->text);
+    helptext_unit(buf, sizeof(buf), client.conn.playing, pitem->text, punittype);
     XtVaSetValues(help_text, XtNstring, buf, NULL);
   }
   else {
@@ -864,66 +878,83 @@ static void help_update_unit_type(const struct help_item *pitem,
     xaw_set_label(help_unit_hp_data, "0 ");
     xaw_set_label(help_unit_visrange_data, "0 ");
     xaw_set_label(help_improvement_req_data, _("(Never)"));
-    create_tech_tree(help_tech_tree, 0, game.num_tech_types, 3);
+    create_tech_tree(help_tech_tree, 0, advance_count(), 3);
     xaw_set_label(help_wonder_obsolete_data, _("None"));
     XtVaSetValues(help_text, XtNstring, pitem->text, NULL);
   }
-  xaw_set_bitmap(help_unit_tile, create_overlay_unit(i));
+  xaw_set_bitmap(help_unit_tile, create_overlay_unit(punittype));
   set_title_topic(pitem);
 }
 
 /**************************************************************************
 ...
 **************************************************************************/
-static void help_update_tech(const struct help_item *pitem, char *title, int i)
+static void help_update_tech(const struct help_item *pitem, char *title)
 {
-  char *buf = &long_buffer[0];
-  int j;
+  char buf[4096];
+  int i;
+  struct advance *padvance = find_advance_by_translated_name(title);
 
   create_help_page(HELP_TECH);
   set_title_topic(pitem);
 
-  if (!is_future_tech(i)) {
+  if (padvance  &&  !is_future_tech(i = advance_number(padvance))) {
     create_tech_tree(help_tech_tree, 0, i, 3);
-    helptext_tech(buf, i, pitem->text);
+    helptext_advance(buf, sizeof(buf), client.conn.playing, pitem->text, i);
 
-    impr_type_iterate(j) {
-      if(i==improvement_types[j].tech_req) 
+    improvement_iterate(pimprove) {
+      /*if (i == j->tech_req) 
 	sprintf(buf+strlen(buf), _("Allows %s.\n"),
-		improvement_types[j].name);
-      if(i==improvement_types[j].obsolete_by)
-	sprintf(buf+strlen(buf), _("Obsoletes %s.\n"),
-		improvement_types[j].name);
-    } impr_type_iterate_end;
+		improvement_name_translation(j));
+      */
 
-    unit_type_iterate(j) {
-      if(i==get_unit_type(j)->tech_requirement) 
-	sprintf(buf+strlen(buf), _("Allows %s.\n"), 
-		get_unit_type(j)->name);
+       /* FIXME: need a more general mechanism for this, since this
+        * helptext needs to be shown in all possible req source types. */
+      requirement_vector_iterate(&pimprove->reqs, preq) {
+	if (VUT_IMPROVEMENT == preq->source.kind
+	    && preq->source.value.building == pimprove) {
+	  sprintf(buf+strlen(buf), _("Allows %s.\n"),
+		  improvement_name_translation(pimprove));
+        }
+      } requirement_vector_iterate_end;
+      if (padvance == pimprove->obsolete_by)
+	sprintf(buf+strlen(buf), _("Obsoletes %s.\n"),
+		improvement_name_translation(pimprove));
+    } improvement_iterate_end;
+
+    unit_type_iterate(punittype) {
+      if (padvance != punittype->require_advance) {
+	continue;
+      }
+      sprintf(buf + strlen(buf), _("Allows %s.\n"),
+	      utype_name_translation(punittype));
     } unit_type_iterate_end;
 
-    for (j = 0; j < game.num_tech_types; j++) {
-      if(i==advances[j].req[0]) {
-	if(advances[j].req[1]==A_NONE)
+    advance_iterate(A_NONE, ptest) {
+      if (padvance == advance_requires(ptest, AR_ONE)) {
+	if (advance_by_number(A_NONE) == advance_requires(ptest, AR_TWO))
 	  sprintf(buf+strlen(buf), _("Allows %s.\n"), 
-		  advances[j].name);
+		  advance_name_translation(ptest));
 	else
 	  sprintf(buf+strlen(buf), _("Allows %s (with %s).\n"), 
-		  advances[j].name, advances[advances[j].req[1]].name);
+		  advance_name_translation(ptest),
+		  advance_name_translation(advance_requires(ptest, AR_TWO)));
       }
-      if(i==advances[j].req[1]) {
+      if (padvance == advance_requires(ptest, AR_TWO)) {
 	sprintf(buf+strlen(buf), _("Allows %s (with %s).\n"), 
-		advances[j].name, advances[advances[j].req[0]].name);
+		advance_name_translation(ptest),
+		advance_name_translation(advance_requires(ptest, AR_ONE)));
       }
-    }
+    } advance_iterate_end;
     if (strlen(buf)) strcat(buf, "\n");
+/*
     if (advances[i].helptext) {
       sprintf(buf+strlen(buf), "%s\n", _(advances[i].helptext));
     }
-  }
-  else {
+*/
+  } else {
     create_help_page(HELP_TECH);
-    create_tech_tree(help_tech_tree, 0, game.num_tech_types, 3);
+    create_tech_tree(help_tech_tree, 0, advance_count(), 3);
     strcpy(buf, pitem->text);
   }
   wordwrap_string(buf, 68);
@@ -934,137 +965,112 @@ static void help_update_tech(const struct help_item *pitem, char *title, int i)
 ...
 **************************************************************************/
 static void help_update_terrain(const struct help_item *pitem,
-				char *title, int i)
+				char *title)
 {
-  char *buf = &long_buffer[0];
-  struct tile_type *ptype = get_tile_type(i);
+  char buf[4096];
+  struct terrain *pterrain = find_terrain_by_translated_name(title);
 
   create_help_page(HELP_TERRAIN);
   set_title_topic(pitem);
 
-  helptext_terrain(buf, i, pitem->text);
+  helptext_terrain(buf, sizeof(buf), client.conn.playing, pitem->text, pterrain);
   XtVaSetValues(help_text, XtNstring, buf, NULL);
 
-  if (i < T_COUNT)
-    {
-      sprintf (buf, "%d/%d.%d",
-	       ptype->movement_cost,
-	       (int)(ptype->defense_bonus/10), ptype->defense_bonus%10);
-      xaw_set_label (help_terrain_movement_defense_data, buf);
+  if (pterrain) {
+    sprintf(buf, "%d/%d.%d",
+	    pterrain->movement_cost,
+	    (int)((pterrain->defense_bonus + 100) / 100),
+	    (pterrain->defense_bonus + 100) % 100 / 10);
+    xaw_set_label(help_terrain_movement_defense_data, buf);
 
-      sprintf (buf, "%d/%d/%d",
-	       ptype->food,
-	       ptype->shield,
-	       ptype->trade);
-      xaw_set_label (help_terrain_food_shield_trade_data, buf);
+    sprintf(buf, "%d/%d/%d",
+	    pterrain->output[O_FOOD],
+	    pterrain->output[O_SHIELD],
+	    pterrain->output[O_TRADE]);
+    xaw_set_label(help_terrain_food_shield_trade_data, buf);
 
-      if (*(ptype->special_1_name))
-	{
-	  sprintf (buf, _("%s F/R/T:"),
-		   ptype->special_1_name);
-	  xaw_set_label (help_terrain_special_1, buf);
-	  sprintf (buf, "%d/%d/%d",
-		   ptype->food_special_1,
-		   ptype->shield_special_1,
-		   ptype->trade_special_1);
-	  xaw_set_label (help_terrain_special_1_data, buf);
-	} else {
-	  xaw_set_label (help_terrain_special_1, "");
-	  xaw_set_label (help_terrain_special_1_data, "");
-	}
+    buf[0] = '\0';
+    if (*(pterrain->resources)) {
+      struct resource **r;
 
-      if (*(ptype->special_2_name))
-	{
-	  sprintf (buf, _("%s F/R/T:"),
-		   ptype->special_2_name);
-	  xaw_set_label (help_terrain_special_2, buf);
-	  sprintf (buf, "%d/%d/%d",
-		   ptype->food_special_2,
-		   ptype->shield_special_2,
-		   ptype->trade_special_2);
-	  xaw_set_label (help_terrain_special_2_data, buf);
-	} else {
-	  xaw_set_label (help_terrain_special_2, "");
-	  xaw_set_label (help_terrain_special_2_data, "");
-	}
-
-      if (ptype->road_trade_incr > 0)
-	{
-	  sprintf (buf, _("+%d Trade / %d"),
-		   ptype->road_trade_incr,
-		   ptype->road_time);
-	}
-      else if (ptype->road_time > 0)
-	{
-	  sprintf (buf, _("no extra / %d"),
-		   ptype->road_time);
-	}
-      else
-	{
-	  strcpy (buf, _("n/a"));
-	}
-      xaw_set_label (help_terrain_road_result_time_data, buf);
-
-      strcpy (buf, _("n/a"));
-      if (ptype->irrigation_result == i)
-	{
-	  if (ptype->irrigation_food_incr > 0)
-	    {
-	      sprintf (buf, _("+%d Food / %d"),
-		       ptype->irrigation_food_incr,
-		       ptype->irrigation_time);
-	    }
-	}
-      else if (ptype->irrigation_result != T_NONE)
-	{
-	  sprintf (buf, "%s / %d",
-		   get_tile_type(ptype->irrigation_result)->terrain_name,
-		   ptype->irrigation_time);
-	}
-      xaw_set_label (help_terrain_irrigation_result_time_data, buf);
-
-      strcpy (buf, _("n/a"));
-      if (ptype->mining_result == i)
-	{
-	  if (ptype->mining_shield_incr > 0)
-	    {
-	      sprintf (buf, _("+%d Res. / %d"),
-		       ptype->mining_shield_incr,
-		       ptype->mining_time);
-	    }
-	}
-      else if (ptype->mining_result != T_NONE)
-	{
-	  sprintf (buf, "%s / %d",
-		   get_tile_type(ptype->mining_result)->terrain_name,
-		   ptype->mining_time);
-	}
-      xaw_set_label (help_terrain_mining_result_time_data, buf);
-
-      if (ptype->transform_result != T_NONE)
-	{
-	  sprintf (buf, "%s / %d",
-		   get_tile_type(ptype->transform_result)->terrain_name,
-		   ptype->transform_time);
-	} else {
-	  strcpy (buf, _("n/a"));
-	}
-      xaw_set_label (help_terrain_transform_result_time_data, buf);
+      for (r = pterrain->resources; *r; r++) {
+        /* TRANS: " Whales (2/1/2)," */
+	sprintf (buf + strlen (buf), " %s (%d/%d/%d),",
+                 resource_name_translation(*r),
+                 pterrain->output[O_FOOD]   + (*r)->output[O_FOOD],
+                 pterrain->output[O_SHIELD] + (*r)->output[O_SHIELD],
+                 pterrain->output[O_TRADE]  + (*r)->output[O_TRADE]);
+      }
+      buf[strlen (buf) - 1] = '.';
+    } else {
+      /* TRANS: "Resources: (none)". */
+      sprintf (buf + strlen (buf), _("(none)"));
     }
+    xaw_set_label(help_terrain_resources, buf);
+
+    if (pterrain->road_trade_incr > 0) {
+      sprintf(buf, _("+%d Trade / %d"),
+	      pterrain->road_trade_incr,
+	      pterrain->road_time);
+    } else if (pterrain->road_time > 0) {
+      sprintf(buf, _("no extra / %d"), pterrain->road_time);
+    } else {
+      strcpy(buf, _("n/a"));
+    }
+    xaw_set_label(help_terrain_road_result_time_data, buf);
+
+    strcpy(buf, _("n/a"));
+    if (pterrain->irrigation_result == pterrain) {
+      if (pterrain->irrigation_food_incr > 0) {
+	sprintf(buf, _("+%d Food / %d"),
+		pterrain->irrigation_food_incr,
+		pterrain->irrigation_time);
+      }
+    } else if (pterrain->irrigation_result != T_NONE) {
+      sprintf(buf, "%s / %d",
+	      terrain_name_translation(pterrain->irrigation_result),
+	      pterrain->irrigation_time);
+    }
+    xaw_set_label(help_terrain_irrigation_result_time_data, buf);
+
+    strcpy(buf, _("n/a"));
+    if (pterrain->mining_result == pterrain) {
+      if (pterrain->mining_shield_incr > 0) {
+	sprintf(buf, _("+%d Res. / %d"),
+		pterrain->mining_shield_incr,
+		pterrain->mining_time);
+      }
+    } else if (pterrain->mining_result != T_NONE) {
+      sprintf(buf, "%s / %d",
+	      terrain_name_translation(pterrain->mining_result),
+	      pterrain->mining_time);
+    }
+    xaw_set_label (help_terrain_mining_result_time_data, buf);
+
+    if (pterrain->transform_result != T_NONE) {
+      sprintf(buf, "%s / %d",
+	      terrain_name_translation(pterrain->transform_result),
+	      pterrain->transform_time);
+    } else {
+      strcpy(buf, _("n/a"));
+    }
+    xaw_set_label (help_terrain_transform_result_time_data, buf);
+  }
 }
 
 /**************************************************************************
   This is currently just a text page, with special text:
 **************************************************************************/
 static void help_update_government(const struct help_item *pitem,
-				   char *title, struct government *gov)
+				   char *title)
 {
-  char *buf = &long_buffer[0];
+  char buf[4096];
+  struct government *pgovernment = find_government_by_translated_name(title);
 
-  if (gov==NULL) {
+  if (!pgovernment) {
     strcat(buf, pitem->text);
   } else {
-    helptext_government(buf, gov-governments, pitem->text);
+    helptext_government(buf, sizeof(buf), client.conn.playing, pitem->text, pgovernment);
   }
   create_help_page(HELP_TEXT);
   set_title_topic(pitem);
@@ -1076,7 +1082,6 @@ static void help_update_government(const struct help_item *pitem,
 **************************************************************************/
 static void help_update_dialog(const struct help_item *pitem)
 {
-  int i;
   char *top;
 
   /* figure out what kind of item is required for pitem ingo */
@@ -1087,26 +1092,22 @@ static void help_update_dialog(const struct help_item *pitem)
 
   switch(pitem->type) {
   case HELP_IMPROVEMENT:
-    i = find_improvement_by_name(top);
-    if(i!=B_LAST && is_wonder(i)) i = B_LAST;
-    help_update_improvement(pitem, top, i);
+    help_update_improvement(pitem, top);
     break;
   case HELP_WONDER:
-    i = find_improvement_by_name(top);
-    if(i!=B_LAST && !is_wonder(i)) i = B_LAST;
-    help_update_wonder(pitem, top, i);
+    help_update_wonder(pitem, top);
     break;
   case HELP_UNIT:
-    help_update_unit_type(pitem, top, find_unit_type_by_name(top));
+    help_update_unit_type(pitem, top);
     break;
   case HELP_TECH:
-    help_update_tech(pitem, top, find_tech_by_name(top));
+    help_update_tech(pitem, top);
     break;
   case HELP_TERRAIN:
-    help_update_terrain(pitem, top, get_terrain_by_name(top));
+    help_update_terrain(pitem, top);
     break;
   case HELP_GOVERNMENT:
-    help_update_government(pitem, top, find_government_by_name(top));
+    help_update_government(pitem, top);
     break;
   case HELP_TEXT:
   default:
@@ -1162,10 +1163,10 @@ static void help_tree_node_callback(Widget w, XtPointer client_data,
   size_t tech=(size_t)client_data;
   
   if(!help_tree_destroy_children(w)) {
-    if(advances[tech].req[0]!=A_NONE)
-      create_tech_tree(help_tech_tree, w, advances[tech].req[0], 1);
-    if(advances[tech].req[1]!=A_NONE)
-      create_tech_tree(help_tech_tree, w, advances[tech].req[1], 1);
+    if(advance_required(tech, AR_ONE)!=A_NONE)
+      create_tech_tree(help_tech_tree, w, advance_required(tech, AR_ONE), 1);
+    if(advance_required(tech, AR_TWO)!=A_NONE)
+      create_tech_tree(help_tech_tree, w, advance_required(tech, AR_TWO), 1);
   }
   
 }
