@@ -18,31 +18,34 @@
 #include <assert.h>
 #include <stdlib.h>		/* qsort */
 
+/* utility */
 #include "fcintl.h"
 #include "log.h"
 #include "support.h"
 
+/* common */
 #include "combat.h"
+#include "game.h"
 #include "unitlist.h"
 
+/* client */
 #include "agents.h"
 #include "chatline_common.h"
 #include "cityrep_g.h"
-#include "civclient.h"
+#include "client_main.h"
 #include "climisc.h"
-#include "clinet.h"
 #include "cma_core.h"
 #include "control.h"
+#include "editor.h"
 #include "fcintl.h"
 #include "goto.h"
+#include "mapctrl_common.h"
 #include "mapctrl_g.h"
 #include "mapview_g.h"
 #include "options.h"
 #include "overview_common.h"
 #include "tilespec.h"
 
-#include "mapctrl_common.h"
-
 /* Selection Rectangle */
 static int rec_anchor_x, rec_anchor_y;  /* canvas coordinates for anchor */
 static struct tile *rec_canvas_center_tile;
@@ -58,7 +61,10 @@ static bool rectangle_append;
 bool tiles_hilited_cities = FALSE;
 
 /* The mapcanvas clipboard */
-struct city_production clipboard = {.value = -1};
+struct universal clipboard =
+{ .kind = VUT_NONE,
+  .value = {.building = NULL}
+};
 
 /* Goto with drag and drop. */
 bool keyboardless_goto_button_down = FALSE;
@@ -68,9 +74,6 @@ struct tile *keyboardless_goto_start_tile;
 /* Update the workers for a city on the map, when the update is received */
 struct city *city_workers_display = NULL;
 
-static bool turn_done_state;
-static bool is_turn_done_state_valid = FALSE;
-
 /*************************************************************************/
 
 static void clipboard_send_production_packet(struct city *pcity);
@@ -151,13 +154,14 @@ static void define_tiles_within_rectangle(void)
 
       /*  Tile passed all tests; process it.
        */
-      if (ptile->city && city_owner(ptile->city) == game.player_ptr) {
+      if (NULL != tile_city(ptile)
+          && tile_owner(ptile) == client.conn.playing) {
 	/* FIXME: handle rectangle_append */
         mapdeco_set_highlight(ptile, TRUE);
         tiles_hilited_cities = TRUE;
       }
       unit_list_iterate(ptile->units, punit) {
-        if (unit_owner(punit) == game.player_ptr) {
+        if (unit_owner(punit) == client.conn.playing) {
           unit_list_append(units, punit);
         }
       } unit_list_iterate_end;
@@ -175,7 +179,6 @@ static void define_tiles_within_rectangle(void)
       add_unit_focus(punit);
     } unit_list_iterate_end;
   }
-  unit_list_unlink_all(units);
   unit_list_free(units);
 
   /* Clear previous rectangle. */
@@ -292,7 +295,7 @@ void cancel_selection_rectangle(void)
 **************************************************************************/
 bool is_city_hilited(struct city *pcity)
 {
-  return pcity && mapdeco_is_highlight_set(pcity->tile);
+  return pcity && mapdeco_is_highlight_set(city_tile(pcity));
 }
 
 /**************************************************************************
@@ -315,16 +318,7 @@ void release_right_button(int canvas_x, int canvas_y)
   if (rectangle_active) {
     define_tiles_within_rectangle();
   } else {
-    /* NB: Assumes 'rectangle_append' was set because <SHIFT> was on. */
-    if (rectangle_append) {
-      struct tile *ptile = canvas_pos_to_tile(canvas_x, canvas_y);
-      if (ptile != NULL) {
-        /* <SHIFT> + RMB: Copy Production. */
-        clipboard_copy_production(ptile);
-      }
-    } else {
-      recenter_button_pressed(canvas_x, canvas_y);
-    }
+    recenter_button_pressed(canvas_x, canvas_y);
   }
   rectangle_active = FALSE;
   rbutton_down = FALSE;
@@ -335,7 +329,7 @@ void release_right_button(int canvas_x, int canvas_y)
 **************************************************************************/
 void toggle_tile_hilite(struct tile *ptile)
 {
-  struct city *pcity = ptile->city;
+  struct city *pcity = tile_city(ptile);
 
   if (mapdeco_is_highlight_set(ptile)) {
     mapdeco_set_highlight(ptile, FALSE);
@@ -343,7 +337,7 @@ void toggle_tile_hilite(struct tile *ptile)
       toggle_city_hilite(pcity, FALSE); /* cityrep.c */
     }
   }
-  else if (pcity && city_owner(pcity) == game.player_ptr) {
+  else if (NULL != pcity && city_owner(pcity) == client.conn.playing) {
     mapdeco_set_highlight(ptile, TRUE);
     tiles_hilited_cities = TRUE;
     toggle_city_hilite(pcity, TRUE);
@@ -378,14 +372,15 @@ void key_city_overlay(int canvas_x, int canvas_y)
 **************************************************************************/
 void clipboard_copy_production(struct tile *ptile)
 {
-  struct city *pcity = ptile->city;
+  char buffer[256];
+  struct city *pcity = tile_city(ptile);
 
   if (!can_client_issue_orders()) {
     return;
   }
 
   if (pcity) {
-    if (city_owner(pcity) != game.player_ptr)  {
+    if (city_owner(pcity) != client.conn.playing)  {
       return;
     }
     clipboard = pcity->production;
@@ -394,22 +389,20 @@ void clipboard_copy_production(struct tile *ptile)
     if (!punit) {
       return;
     }
-    if (!can_player_build_unit_direct(game.player_ptr, unit_type(punit)))  {
-      create_event(ptile, E_BAD_COMMAND,
-		   _("You don't know how to build %s!"),
-		   unit_name_translation(punit));
+    if (!can_player_build_unit_direct(client.conn.playing, unit_type(punit)))  {
+      create_event(ptile, E_BAD_COMMAND, ftc_client,
+                   _("You don't know how to build %s!"),
+                   unit_name_translation(punit));
       return;
     }
-    clipboard.is_unit = TRUE;
-    clipboard.value = unit_type(punit)->index;
+    clipboard.kind = VUT_UTYPE;
+    clipboard.value.utype = unit_type(punit);
   }
   upgrade_canvas_clipboard();
 
   create_event(ptile, E_CITY_PRODUCTION_CHANGED, /* ? */
-	       _("Copy %s to clipboard."),
-	       clipboard.is_unit
-	       ? utype_name_translation(utype_by_number(clipboard.value))
-	       : improvement_name_translation(clipboard.value));
+               ftc_client, _("Copy %s to clipboard."),
+               universal_name_translation(&clipboard, buffer, sizeof(buffer)));
 }
 
 /**************************************************************************
@@ -421,24 +414,25 @@ void clipboard_paste_production(struct city *pcity)
   if (!can_client_issue_orders()) {
     return;
   }
-  if (clipboard.value == -1) {
-    create_event(pcity->tile, E_BAD_COMMAND, _("Clipboard is empty."));
+  if (NULL == clipboard.value.building) {
+    create_event(city_tile(pcity), E_BAD_COMMAND, ftc_client,
+                 _("Clipboard is empty."));
     return;
   }
   if (!tiles_hilited_cities) {
-    if (pcity && city_owner(pcity) == game.player_ptr) {
+    if (NULL != pcity && city_owner(pcity) == client.conn.playing) {
       clipboard_send_production_packet(pcity);
     }
     return;
   }
   else {
-    connection_do_buffer(&aconnection);
-    city_list_iterate(game.player_ptr->cities, pcity) {
+    connection_do_buffer(&client.conn);
+    city_list_iterate(client.conn.playing->cities, pcity) {
       if (is_city_hilited(pcity)) {
         clipboard_send_production_packet(pcity);
       }
     } city_list_iterate_end;
-    connection_do_unbuffer(&aconnection);
+    connection_do_unbuffer(&client.conn);
   }
 }
 
@@ -447,14 +441,14 @@ void clipboard_paste_production(struct city *pcity)
 **************************************************************************/
 static void clipboard_send_production_packet(struct city *pcity)
 {
-  if ((clipboard.is_unit == pcity->production.is_unit
-       && clipboard.value == pcity->production.value)
-      || !city_can_build_impr_or_unit(pcity, clipboard)) {
+  if (are_universals_equal(&pcity->production, &clipboard)
+      || !can_city_build_now(pcity, clipboard)) {
     return;
   }
 
-  dsend_packet_city_change(&aconnection, pcity->id, clipboard.value,
-			   clipboard.is_unit);
+  dsend_packet_city_change(&client.conn, pcity->id,
+			   clipboard.kind,
+			   universal_number(&clipboard));
 }
 
 /**************************************************************************
@@ -466,12 +460,12 @@ void upgrade_canvas_clipboard(void)
   if (!can_client_issue_orders()) {
     return;
   }
-  if (clipboard.is_unit)  {
-    struct unit_type *u
-      = can_upgrade_unittype(game.player_ptr, utype_by_number(clipboard.value));
+  if (VUT_UTYPE == clipboard.kind)  {
+    struct unit_type *u =
+      can_upgrade_unittype(client.conn.playing, clipboard.value.utype);
 
     if (u)  {
-      clipboard.value = u->index;
+      clipboard.value.utype = u;
     }
   }
 }
@@ -510,16 +504,14 @@ void maybe_activate_keyboardless_goto(int canvas_x, int canvas_y)
 }
 
 /**************************************************************************
- Return TRUE iff the turn done button is enabled.
+ Return TRUE iff the turn done button should be enabled.
 **************************************************************************/
-bool get_turn_done_button_state()
+bool get_turn_done_button_state(void)
 {
-  if (!is_turn_done_state_valid) {
-    update_turn_done_button_state();
-  }
-  assert(is_turn_done_state_valid);
-
-  return turn_done_state;
+  return (can_client_issue_orders()
+          && !client.conn.playing->ai_data.control
+          && !client.conn.playing->phase_done
+          && !agents_busy());
 }
 
 /**************************************************************************
@@ -573,28 +565,25 @@ void wakeup_button_pressed(int canvas_x, int canvas_y)
 **************************************************************************/
 void adjust_workers_button_pressed(int canvas_x, int canvas_y)
 {
-  int city_x, city_y;
-  enum city_tile_type worker;
   struct tile *ptile = canvas_pos_to_tile(canvas_x, canvas_y);
 
-  if (can_client_issue_orders() && ptile) {
+  if (NULL != ptile && can_client_issue_orders()) {
     struct city *pcity = find_city_near_tile(ptile);
 
     if (pcity && !cma_is_city_under_agent(pcity, NULL)) {
-      if (!map_to_city_map(&city_x, &city_y, pcity, ptile)) {
-	assert(0);
-      }
+      int city_x, city_y;
+      bool success;
 
-      worker = get_worker_city(pcity, city_x, city_y);
-      if (worker == C_TILE_WORKER) {
-	dsend_packet_city_make_specialist(&aconnection, pcity->id,
+      success = city_base_to_city_map(&city_x, &city_y, pcity, ptile);
+      assert(success);
+
+      if (NULL != tile_worked(ptile) && tile_worked(ptile) == pcity) {
+	dsend_packet_city_make_specialist(&client.conn, pcity->id,
 					  city_x, city_y);
-      } else if (worker == C_TILE_EMPTY) {
-	dsend_packet_city_make_worker(&aconnection, pcity->id,
+      } else if (city_can_work_tile(pcity, ptile)) {
+	dsend_packet_city_make_worker(&client.conn, pcity->id,
 				      city_x, city_y);
       } else {
-	/* If worker == C_TILE_UNAVAILABLE then we can't use this tile.  No
-	 * packet is sent and city_workers_display is not updated. */
 	return;
       }
 
@@ -623,36 +612,17 @@ void recenter_button_pressed(int canvas_x, int canvas_y)
 /**************************************************************************
  Update the turn done button state.
 **************************************************************************/
-void update_turn_done_button_state()
+void update_turn_done_button_state(void)
 {
-  bool new_state;
-
-  if (!is_turn_done_state_valid) {
-    turn_done_state = FALSE;
-    is_turn_done_state_valid = TRUE;
-    set_turn_done_button_state(turn_done_state);
-    freelog(LOG_DEBUG, "setting turn done button state init %d",
-	    turn_done_state);
-  }
-
-  new_state = (can_client_issue_orders()
-	       && !game.player_ptr->phase_done && !agents_busy()
-	       && !turn_done_sent);
-  if (new_state == turn_done_state) {
-    return;
-  }
-
-  freelog(LOG_DEBUG, "setting turn done button state from %d to %d",
-	  turn_done_state, new_state);
-  turn_done_state = new_state;
+  bool turn_done_state = get_turn_done_button_state();
 
   set_turn_done_button_state(turn_done_state);
 
   if (turn_done_state) {
     if (waiting_for_end_turn
-	|| (game.player_ptr
-	    && game.player_ptr->ai.control
-	    && !ai_manual_turn_done)) {
+        || (NULL != client.conn.playing
+            && client.conn.playing->ai_data.control
+            && !ai_manual_turn_done)) {
       send_turn_done();
     } else {
       update_turn_done_button(TRUE);
