@@ -42,6 +42,7 @@
 #include "dialogs_g.h"
 #include "gui_main_g.h"
 #include "menu_g.h"
+#include "repodlgs_g.h"
 #include "voteinfo_bar_g.h"
 
 /* client */
@@ -392,8 +393,11 @@ bool gui_gtk2_new_messages_go_to_top = FALSE;
 bool gui_gtk2_show_message_window_buttons = TRUE;
 bool gui_gtk2_metaserver_tab_first = FALSE;
 bool gui_gtk2_allied_chat_only = FALSE;
-bool gui_gtk2_small_display_layout = FALSE;
+bool gui_gtk2_small_display_layout = TRUE;
+bool gui_gtk2_merge_notebooks = TRUE;
 bool gui_gtk2_mouse_over_map_focus = FALSE;
+int gui_gtk2_citydlg_xsize = GUI_GTK2_CITYDLG_DEFAULT_XSIZE;
+int gui_gtk2_citydlg_ysize = GUI_GTK2_CITYDLG_DEFAULT_YSIZE;
 char gui_gtk2_font_city_label[512] = "Monospace 8";
 char gui_gtk2_font_notify_label[512] = "Monospace Bold 9";
 char gui_gtk2_font_spaceship_label[512] = "Monospace 8";
@@ -834,10 +838,12 @@ static struct client_option options[] = {
                      "[hour:minute:second]."),
                   COC_INTERFACE, GUI_GTK2, FALSE, NULL),
   GEN_BOOL_OPTION(gui_gtk2_split_bottom_notebook,
-                  N_("Split bottom notebook area"),
-                  N_("Enabling this option will split the bottom "
-                     "notebook into a left and right notebook so that "
-                     "two tabs may be viewed at once."),
+                  N_("Split message/chat notebook area"),
+                  N_("Enabling this option will split the message/chat "
+                     "notebook into two separate areas so that both can "
+                     "be viewed at once. This option is ignored if "
+                     "'Merge the message notebook and the map notebook' "
+                     "is set."),
                   COC_INTERFACE, GUI_GTK2, FALSE, NULL),
   GEN_BOOL_OPTION(gui_gtk2_new_messages_go_to_top,
                   N_("New message events go to top of list"),
@@ -884,12 +890,33 @@ static struct client_option options[] = {
                      "status, and the unit information box will be "
                      "extended over the entire left side of the window. "
                      "This option requires a restart in order to take "
-                     "effect."), COC_INTERFACE, GUI_GTK2, FALSE, NULL),
+                     "effect."), COC_INTERFACE, GUI_GTK2, TRUE, NULL),
+  GEN_BOOL_OPTION(gui_gtk2_merge_notebooks,
+                  N_("Merge the message notebook and the map notebook"),
+                  N_("If this option is enabled, the message notebook "
+                     "will be merged into the world map notebook. Thus, "
+                     "the size needed for the main window will be reduced. "
+                     "This option requires a restart in order to take "
+                     "effect."), COC_INTERFACE, GUI_GTK2, TRUE, NULL),
   GEN_BOOL_OPTION(gui_gtk2_mouse_over_map_focus,
                   N_("Mouse over the map widget selects it automatically"),
                   N_("If this option is enabled, then the map will be "
                      "focused when the mouse will be floating over it."),
                   COC_INTERFACE, GUI_GTK2, FALSE, NULL),
+  GEN_INT_OPTION(gui_gtk2_citydlg_xsize,
+                 N_("Width of the city dialog"),
+                 N_("This value is only used if the width of the city "
+                    "dialog is saved."),
+                 COC_INTERFACE, GUI_GTK2, GUI_GTK2_CITYDLG_DEFAULT_XSIZE,
+                 GUI_GTK2_CITYDLG_MIN_XSIZE, GUI_GTK2_CITYDLG_MAX_XSIZE,
+                 NULL),
+  GEN_INT_OPTION(gui_gtk2_citydlg_ysize,
+                 N_("Height of the city dialog"),
+                 N_("This value is only used if the height of the city "
+                    "dialog is saved."),
+                 COC_INTERFACE, GUI_GTK2, GUI_GTK2_CITYDLG_DEFAULT_YSIZE,
+                 GUI_GTK2_CITYDLG_MIN_YSIZE, GUI_GTK2_CITYDLG_MAX_YSIZE,
+                 NULL),
   GEN_FONT_OPTION(gui_gtk2_font_city_label, "city_label",
                   N_("City Label"),
                   N_("This font is used to display the city labels on city "
@@ -2247,14 +2274,12 @@ static void mapview_redraw_callback(struct client_option *poption)
 }
 
 /****************************************************************************
-   Callback when the reqtree  show icons option is changed.
-   The tree is recalculated.
+  Callback when the reqtree show icons option is changed. The tree is
+  recalculated.
 ****************************************************************************/
 static void reqtree_show_icons_callback(struct client_option *poption)
 {
-  /* This will close research dialog, when it's open again the techtree will
-   * be recalculated */
-  popdown_all_game_dialogs();
+  science_dialog_redraw();
 }
 
 /****************************************************************************
