@@ -30,23 +30,67 @@
 
 #include "tech.h"
 
-struct advance advances[A_LAST];
 /* the advances array is now setup in:
-   server/ruleset.c (for the server)
-   client/packhand.c (for the client) */
+ * server/ruleset.c (for the server)
+ * client/packhand.c (for the client)
+ */
+struct advance advances[A_LAST];
 
 /* Precalculated costs according to techcost style 1.  These do not include
  * the sciencebox multiplier. */
 static double techcoststyle1[A_LAST];
 
-static const char *flag_names[] = {
-  "Bonus_Tech", "Bridge", "Railroad", "Fortress",
-  "Population_Pollution_Inc", 
-  "Airbase", "Farmland", "Reduce_Trireme_Loss1", "Reduce_Trireme_Loss2", 
-  "Build_Airborne"
-};
 /* Note that these strings must correspond with the enums in tech_flag_id,
    in common/tech.h */
+static const char *flag_names[] = {
+  "Bonus_Tech",
+  "Bridge",
+  "Railroad",
+  "Population_Pollution_Inc", 
+  "Farmland",
+  "Build_Airborne"
+};
+
+
+/**************************************************************************
+  Return the last item of advances/technologies.
+**************************************************************************/
+const struct advance *advance_array_last(void)
+{
+  if (game.control.num_tech_types > 0) {
+    return &advances[game.control.num_tech_types - 1];
+  }
+  return NULL;
+}
+
+/**************************************************************************
+  Return the number of advances/technologies.
+**************************************************************************/
+Tech_type_id advance_count(void)
+{
+  return game.control.num_tech_types;
+}
+
+/**************************************************************************
+  Return the advance index.
+
+  Currently same as advance_number(), paired with advance_count()
+  indicates use as an array index.
+**************************************************************************/
+Tech_type_id advance_index(const struct advance *padvance)
+{
+  assert(padvance);
+  return padvance - advances;
+}
+
+/**************************************************************************
+  Return the advance index.
+**************************************************************************/
+Tech_type_id advance_number(const struct advance *padvance)
+{
+  assert(padvance);
+  return padvance->item_number;
+}
 
 /**************************************************************************
   Return the advance for the given advance index.
@@ -62,14 +106,14 @@ struct advance *advance_by_number(const Tech_type_id atype)
 
 /**************************************************************************
   Returns state of the tech for current pplayer.
-  This can be: TECH_KNOW, TECH_UNKNOWN or TECH_REACHABLE
+  This can be: TECH_KNOWN, TECH_UNKNOWN, or TECH_PREREQS_KNOWN
   Should be called with existing techs or A_FUTURE
 
   If pplayer is NULL this checks whether any player knows the tech (used
   by the client).
 **************************************************************************/
-enum tech_state get_invention(const struct player *pplayer,
-			      Tech_type_id tech)
+enum tech_state player_invention_state(const struct player *pplayer,
+				       Tech_type_id tech)
 {
   assert(tech == A_FUTURE
          || (tech >= 0 && tech < game.control.num_tech_types));
@@ -96,20 +140,22 @@ enum tech_state get_invention(const struct player *pplayer,
 /**************************************************************************
 ...
 **************************************************************************/
-void set_invention(struct player *pplayer, Tech_type_id tech,
-		   enum tech_state value)
+enum tech_state player_invention_set(struct player *pplayer,
+				     Tech_type_id tech,
+				     enum tech_state value)
 {
   struct player_research *research = get_player_research(pplayer);
+  enum tech_state old = research->inventions[tech].state;
 
-  if (research->inventions[tech].state == value) {
-    return;
+  if (old == value) {
+    return old;
   }
-
   research->inventions[tech].state = value;
 
   if (value == TECH_KNOWN) {
     game.info.global_advances[tech] = TRUE;
   }
+  return old;
 }
 
 /**************************************************************************
@@ -134,6 +180,32 @@ bool is_tech_a_req_for_goal(const struct player *pplayer, Tech_type_id tech,
   }
 }
 
+/**************************************************************************
+  Accessor for requirements.
+**************************************************************************/
+Tech_type_id advance_required(const Tech_type_id tech,
+			      enum tech_req require)
+{
+  assert(require >= 0 && require < AR_SIZE);
+  assert(tech >= A_NONE || tech < A_LAST);
+  if (A_NEVER == advances[tech].require[require]) {
+    /* out of range */
+    return A_LAST;
+  }
+  return advance_number(advances[tech].require[require]);
+}
+
+/**************************************************************************
+  Accessor for requirements.
+**************************************************************************/
+struct advance *advance_requires(const struct advance *padvance,
+				 enum tech_req require)
+{
+  assert(require >= 0 && require < AR_SIZE);
+  assert(NULL != padvance);
+  return padvance->require[require];
+}
+
 /**************************************************************************
   Marks all techs which are requirements for goal in
   pplayer->research->inventions[goal].required_techs. Works recursive.
@@ -144,8 +216,8 @@ static void build_required_techs_helper(struct player *pplayer,
 {
   /* The is_tech_a_req_for_goal condition is true if the tech is
    * already marked */
-  if (!tech_is_available(pplayer, tech)
-      || get_invention(pplayer, tech) == TECH_KNOWN
+  if (!player_invention_reachable(pplayer, tech)
+      || player_invention_state(pplayer, tech) == TECH_KNOWN
       || is_tech_a_req_for_goal(pplayer, tech, goal)) {
     return;
   }
@@ -153,14 +225,15 @@ static void build_required_techs_helper(struct player *pplayer,
   /* Mark the tech as required for the goal */
   BV_SET(get_player_research(pplayer)->inventions[goal].required_techs, tech);
 
-  if (advances[tech].req[0] == goal || advances[tech].req[1] == goal) {
+  if (advance_required(tech, AR_ONE) == goal
+   || advance_required(tech, AR_TWO) == goal) {
     freelog(LOG_FATAL, "tech \"%s\": requires itself",
 	    advance_name_by_player(pplayer, goal));
     exit(EXIT_FAILURE);
   }
 
-  build_required_techs_helper(pplayer, advances[tech].req[0], goal);
-  build_required_techs_helper(pplayer, advances[tech].req[1], goal);
+  build_required_techs_helper(pplayer, advance_required(tech, AR_ONE), goal);
+  build_required_techs_helper(pplayer, advance_required(tech, AR_TWO), goal);
 }
 
 /**************************************************************************
@@ -174,7 +247,7 @@ static void build_required_techs(struct player *pplayer, Tech_type_id goal)
 
   BV_CLR_ALL(research->inventions[goal].required_techs);
   
-  if (get_invention(pplayer, goal) == TECH_KNOWN) {
+  if (player_invention_state(pplayer, goal) == TECH_KNOWN) {
     research->inventions[goal].num_required_techs = 0;
     research->inventions[goal].bulbs_required = 0;
     return;
@@ -188,8 +261,8 @@ static void build_required_techs(struct player *pplayer, Tech_type_id goal)
   research->inventions[goal].num_required_techs = 1;
 
   counter = 0;
-  tech_type_iterate(i) {
-    if (i == A_NONE || !is_tech_a_req_for_goal(pplayer, i, goal)) {
+  advance_index_iterate(A_FIRST, i) {
+    if (!is_tech_a_req_for_goal(pplayer, i, goal)) {
       continue;
     }
 
@@ -203,7 +276,7 @@ static void build_required_techs(struct player *pplayer, Tech_type_id goal)
     research->inventions[goal].num_required_techs++;
     research->inventions[goal].bulbs_required +=
 	base_total_bulbs_required(pplayer, i);
-  } tech_type_iterate_end;
+  } advance_index_iterate_end;
 
   /* Undo the changes made above */
   research->techs_researched -= counter;
@@ -216,14 +289,18 @@ static void build_required_techs(struct player *pplayer, Tech_type_id goal)
   pplayer may be NULL in which case a simplified result is returned
   (used by the client).
 **************************************************************************/
-bool tech_is_available(const struct player *pplayer, Tech_type_id id)
+bool player_invention_reachable(const struct player *pplayer,
+                                const Tech_type_id tech)
 {
-  if (!tech_exists(id)) {
+  Tech_type_id root;
+
+  if (!valid_advance_by_number(tech)) {
     return FALSE;
   }
 
-  if (advances[id].root_req != A_NONE
-      && get_invention(pplayer, advances[id].root_req) != TECH_KNOWN) {
+  root = advance_required(tech, AR_ROOT);
+  if (A_NONE != root
+   && TECH_KNOWN != player_invention_state(pplayer, root)) {
     /* This tech requires knowledge of another tech before being 
      * available. Prevents sharing of untransferable techs. */
     return FALSE;
@@ -232,53 +309,54 @@ bool tech_is_available(const struct player *pplayer, Tech_type_id id)
 }
 
 /**************************************************************************
-  Mark as TECH_REACHABLE each tech which is available, not known and
+  Mark as TECH_PREREQS_KNOWN each tech which is available, not known and
   which has all requirements fullfiled.
   If there is no such a tech mark A_FUTURE as researchable.
   
   Recalculate research->num_known_tech_with_flag
-  Should be called always after set_invention()
+  Should always be called after player_invention_set()
 **************************************************************************/
-void update_research(struct player *pplayer)
+void player_research_update(struct player *pplayer)
 {
   enum tech_flag_id flag;
   int researchable = 0;
 
-  tech_type_iterate(i) {
-    if (i == A_NONE) {
-      /* This is set when the game starts, but not everybody finds out
-       * right away. */
-      set_invention(pplayer, i, TECH_KNOWN);
-    } else if (!tech_is_available(pplayer, i)) {
-      set_invention(pplayer, i, TECH_UNKNOWN);
+  /* This is set when the game starts, but not everybody finds out
+   * right away. */
+  player_invention_set(pplayer, A_NONE, TECH_KNOWN);
+
+  advance_index_iterate(A_FIRST, i) {
+    if (!player_invention_reachable(pplayer, i)) {
+      player_invention_set(pplayer, i, TECH_UNKNOWN);
     } else {
-      if (get_invention(pplayer, i) == TECH_REACHABLE) {
-	set_invention(pplayer, i, TECH_UNKNOWN);
+      if (player_invention_state(pplayer, i) == TECH_PREREQS_KNOWN) {
+	player_invention_set(pplayer, i, TECH_UNKNOWN);
       }
 
-      if (get_invention(pplayer, i) == TECH_UNKNOWN
-	  && get_invention(pplayer, advances[i].req[0]) == TECH_KNOWN
-	  && get_invention(pplayer, advances[i].req[1]) == TECH_KNOWN) {
-	set_invention(pplayer, i, TECH_REACHABLE);
+      if (player_invention_state(pplayer, i) == TECH_UNKNOWN
+	  && player_invention_state(pplayer, advance_required(i, AR_ONE)) == TECH_KNOWN
+	  && player_invention_state(pplayer, advance_required(i, AR_TWO)) == TECH_KNOWN) {
+	player_invention_set(pplayer, i, TECH_PREREQS_KNOWN);
 	researchable++;
       }
     }
     build_required_techs(pplayer, i);
-  } tech_type_iterate_end;
+  } advance_index_iterate_end;
   
   /* No techs we can research? Mark A_FUTURE as researchable */
   if (researchable == 0) {
-    set_invention(pplayer, A_FUTURE, TECH_REACHABLE);
+    player_invention_set(pplayer, A_FUTURE, TECH_PREREQS_KNOWN);
   }
 
   for (flag = 0; flag < TF_LAST; flag++) {
     get_player_research(pplayer)->num_known_tech_with_flag[flag] = 0;
 
-    tech_type_iterate(i) {
-      if (get_invention(pplayer, i) == TECH_KNOWN && advance_has_flag(i, flag)) {
+    advance_index_iterate(A_NONE, i) {
+      if (player_invention_state(pplayer, i) == TECH_KNOWN
+       && advance_has_flag(i, flag)) {
 	get_player_research(pplayer)->num_known_tech_with_flag[flag]++;
       }
-    } tech_type_iterate_end;
+    } advance_index_iterate_end;
   }
 }
 
@@ -286,70 +364,91 @@ void update_research(struct player *pplayer)
   Return the next tech we should research to advance towards our goal.
   Returns A_UNSET if nothing is available or the goal is already known.
 **************************************************************************/
-Tech_type_id get_next_tech(const struct player *pplayer, Tech_type_id goal)
+Tech_type_id player_research_step(const struct player *pplayer,
+				  Tech_type_id goal)
 {
   Tech_type_id sub_goal;
 
-  if (!tech_is_available(pplayer, goal)
-      || get_invention(pplayer, goal) == TECH_KNOWN) {
+  if (!player_invention_reachable(pplayer, goal)) {
     return A_UNSET;
   }
-  if (get_invention(pplayer, goal) == TECH_REACHABLE) {
+  switch (player_invention_state(pplayer, goal)) {
+  case TECH_KNOWN:
+    return A_UNSET;
+  case TECH_PREREQS_KNOWN:
     return goal;
-  }
-  sub_goal = get_next_tech(pplayer, advances[goal].req[0]);
+  case TECH_UNKNOWN:
+  default:
+    break;
+  };
+  sub_goal = player_research_step(pplayer, advance_required(goal, AR_ONE));
   if (sub_goal != A_UNSET) {
     return sub_goal;
   } else {
-    return get_next_tech(pplayer, advances[goal].req[1]);
+    return player_research_step(pplayer, advance_required(goal, AR_TWO));
   }
 }
 
 /**************************************************************************
-Returns 1 if the tech "exists" in this game, 0 otherwise.
-A tech doesn't exist if one of:
-- id is out of range
-- the tech has been flagged as removed by setting its req values
-  to A_LAST (this function returns 0 if either req is A_LAST, rather
-  than both, to be on the safe side)
+  Returns pointer when the advance "exists" in this game, returns NULL
+  otherwise.
+
+  A tech doesn't exist if it has been flagged as removed by setting its
+  require values to A_NEVER. Note that this function returns NULL if either
+  of req values is A_NEVER, rather than both, to be on the safe side.
 **************************************************************************/
-bool tech_exists(Tech_type_id id)
+struct advance *valid_advance(struct advance *padvance)
 {
-  if (id < 0 || id >= game.control.num_tech_types) {
-    return FALSE;
-  } else {
-    return advances[id].req[0] != A_LAST && advances[id].req[1] != A_LAST;
+  if (NULL == padvance
+   || A_NEVER == padvance->require[AR_ONE]
+   || A_NEVER == padvance->require[AR_TWO]) {
+    return NULL;
   }
+
+  return padvance;
+}
+
+/**************************************************************************
+  Returns pointer when the advance "exists" in this game,
+  returns NULL otherwise.
+
+  In addition to valid_advance(), tests for id is out of range.
+**************************************************************************/
+struct advance *valid_advance_by_number(const Tech_type_id id)
+{
+  return valid_advance(advance_by_number(id));
 }
 
 /**************************************************************************
  Does a linear search of advances[].name.translated
- Returns A_LAST if none match.
+ Returns NULL when none match.
 **************************************************************************/
-Tech_type_id find_advance_by_translated_name(const char *s)
+struct advance *find_advance_by_translated_name(const char *name)
 {
-  tech_type_iterate(i) {
-    if (0 == strcmp(advance_name_translation(i), s)) {
-      return i;
+  advance_iterate(A_NONE, padvance) {
+    if (0 == strcmp(advance_name_translation(padvance), name)) {
+      return padvance;
     }
-  } tech_type_iterate_end;
-  return A_LAST;
+  } advance_iterate_end;
+
+  return NULL;
 }
 
 /**************************************************************************
  Does a linear search of advances[].name.vernacular
- Returns A_LAST if none match.
+ Returns NULL when none match.
 **************************************************************************/
-Tech_type_id find_advance_by_rule_name(const char *s)
+struct advance *find_advance_by_rule_name(const char *name)
 {
-  const char *qs = Qn_(s);
+  const char *qname = Qn_(name);
 
-  tech_type_iterate(i) {
-    if (0 == mystrcasecmp(advance_rule_name(i), qs)) {
-      return i;
+  advance_iterate(A_NONE, padvance) {
+    if (0 == mystrcasecmp(advance_rule_name(padvance), qname)) {
+      return padvance;
     }
-  } tech_type_iterate_end;
-  return A_LAST;
+  } advance_iterate_end;
+
+  return NULL;
 }
 
 /**************************************************************************
@@ -358,7 +457,7 @@ Tech_type_id find_advance_by_rule_name(const char *s)
 bool advance_has_flag(Tech_type_id tech, enum tech_flag_id flag)
 {
   assert(flag >= 0 && flag < TF_LAST);
-  return TEST_BIT(advances[tech].flags, flag);
+  return TEST_BIT(advance_by_number(tech)->flags, flag);
 }
 
 /**************************************************************************
@@ -383,13 +482,12 @@ enum tech_flag_id find_advance_flag_by_rule_name(const char *s)
  Search for a tech with a given flag starting at index
  Returns A_LAST if no tech has been found
 **************************************************************************/
-Tech_type_id find_advance_by_flag(int index, enum tech_flag_id flag)
+Tech_type_id find_advance_by_flag(Tech_type_id index, enum tech_flag_id flag)
 {
-  Tech_type_id i;
-  for(i = index;i < game.control.num_tech_types; i++)
+  advance_index_iterate(index, i)
   {
     if(advance_has_flag(i,flag)) return i;
-  }
+  } advance_index_iterate_end
   return A_LAST;
 }
 
@@ -442,7 +540,7 @@ int base_total_bulbs_required(const struct player *pplayer,
 
   if (pplayer
       && !is_future_tech(tech)
-      && get_invention(pplayer, tech) == TECH_KNOWN) {
+      && player_invention_state(pplayer, tech) == TECH_KNOWN) {
     /* A non-future tech which is already known costs nothing. */
     return 0;
   }
@@ -478,11 +576,7 @@ int base_total_bulbs_required(const struct player *pplayer,
     base_cost = 0.0;
   }
 
-  /* Research becomes more expensive this year and after. */
-  if (game.info.tech_cost_double_year != 0
-      && game.info.year >= game.info.tech_cost_double_year) {
-    base_cost *= 2.0;
-  }
+  base_cost *= get_player_bonus(pplayer, EFT_TECH_COST_FACTOR);
 
   switch (game.info.tech_leakage) {
   case 0:
@@ -495,7 +589,7 @@ int base_total_bulbs_required(const struct player *pplayer,
 
       players_iterate(other) {
 	players++;
-	if (get_invention(other, tech) == TECH_KNOWN
+	if (player_invention_state(other, tech) == TECH_KNOWN
 	    && pplayer && player_has_embassy(pplayer, other)) {
 	  players_with_tech_and_embassy++;
 	}
@@ -512,7 +606,7 @@ int base_total_bulbs_required(const struct player *pplayer,
 
       players_iterate(other) {
 	players++;
-	if (get_invention(other, tech) == TECH_KNOWN) {
+	if (player_invention_state(other, tech) == TECH_KNOWN) {
 	  players_with_tech++;
 	}
       } players_iterate_end;
@@ -531,7 +625,7 @@ int base_total_bulbs_required(const struct player *pplayer,
 	  continue;
 	}
 	players++;
-	if (get_invention(other, tech) == TECH_KNOWN) {
+	if (player_invention_state(other, tech) == TECH_KNOWN) {
 	  players_with_tech++;
 	}
       } players_iterate_end;
@@ -546,12 +640,12 @@ int base_total_bulbs_required(const struct player *pplayer,
   }
 
   /* Assign a science penalty to the AI at easier skill levels.  This code
-   * can also be adpoted to create an extra-hard AI skill level where the AI
+   * can also be adopted to create an extra-hard AI skill level where the AI
    * gets science benefits */
 
-  if (pplayer && pplayer->ai.control) {
-    assert(pplayer->ai.science_cost > 0);
-    base_cost *= (double)pplayer->ai.science_cost / 100.0;
+  if (pplayer && pplayer->ai_data.control) {
+    assert(pplayer->ai_data.science_cost > 0);
+    base_cost *= (double)pplayer->ai_data.science_cost / 100.0;
   }
 
   base_cost *= (double)game.info.sciencebox / 100.0;
@@ -599,15 +693,15 @@ int total_bulbs_required_for_goal(const struct player *pplayer,
 **************************************************************************/
 static int precalc_tech_data_helper(Tech_type_id tech, bool *counted)
 {
-  if (tech == A_NONE || !tech_exists(tech) || counted[tech]) {
+  if (tech == A_NONE || !valid_advance_by_number(tech) || counted[tech]) {
     return 0;
   }
 
   counted[tech] = TRUE;
 
   return 1 + 
-      precalc_tech_data_helper(advances[tech].req[0], counted)+ 
-      precalc_tech_data_helper(advances[tech].req[1], counted);
+      precalc_tech_data_helper(advance_required(tech, AR_ONE), counted)+ 
+      precalc_tech_data_helper(advance_required(tech, AR_TWO), counted);
 }
 
 /**************************************************************************
@@ -617,18 +711,18 @@ void precalc_tech_data()
 {
   bool counted[A_LAST];
 
-  tech_type_iterate(tech) {
+  advance_index_iterate(A_NONE, tech) {
     memset(counted, 0, sizeof(counted));
     advances[tech].num_reqs = precalc_tech_data_helper(tech, counted);
-  } tech_type_iterate_end;
+  } advance_index_iterate_end;
 
-  tech_type_iterate(tech) {
+  advance_index_iterate(A_NONE, tech) {
     double reqs = advances[tech].num_reqs + 1;
     const double base = game.info.base_tech_cost / 2;
     const double cost = base * reqs * sqrt(reqs);
 
     techcoststyle1[tech] = MAX(cost, game.info.base_tech_cost);
-  } tech_type_iterate_end;
+  } advance_index_iterate_end;
 }
 
 /**************************************************************************
@@ -671,17 +765,20 @@ const char *advance_name_by_player(const struct player *pplayer, Tech_type_id te
         char buffer[1024];
   
         my_snprintf(buffer, sizeof(buffer), "%s %d",
-                    advance_rule_name(tech),
+                    advance_rule_name(&advances[tech]),
                     research->future_tech + 1);
         future.p[research->future_tech] = mystrdup(buffer);
       }
       return future.p[research->future_tech];
     } else {
-      return advance_rule_name(tech);
+      return advance_rule_name(&advances[tech]);
     }
+  case A_UNKNOWN:
+  case A_UNSET:
+    return advance_rule_name(&advances[tech]);
   default:
     /* Includes A_NONE */
-    return advance_rule_name(tech);
+    return advance_rule_name(advance_by_number(tech));
   };
 }
 
@@ -718,11 +815,14 @@ const char *advance_name_for_player(const struct player *pplayer, Tech_type_id t
       }
       return future.p[research->future_tech];
     } else {
-      return advance_name_translation(tech);
+      return advance_name_translation(&advances[tech]);
     }
+  case A_UNKNOWN:
+  case A_UNSET:
+    return advance_name_translation(&advances[tech]);
   default:
     /* Includes A_NONE */
-    return advance_name_translation(tech);
+    return advance_name_translation(advance_by_number(tech));
   };
 }
 
@@ -742,25 +842,24 @@ const char *advance_name_researching(const struct player *pplayer)
   Return the (translated) name of the given advance/technology.
   You don't have to free the return pointer.
 **************************************************************************/
-const char *advance_name_translation(Tech_type_id tech)
+const char *advance_name_translation(struct advance *padvance)
 {
-/*  assert(tech_exists(tech)); now called for A_UNSET, A_FUTURE */
-  if (NULL == advances[tech].name.translated) {
+  if (NULL == padvance->name.translated) {
     /* delayed (unified) translation */
-    advances[tech].name.translated = ('\0' == advances[tech].name.vernacular[0])
-				     ? advances[tech].name.vernacular
-				     : Q_(advances[tech].name.vernacular);
+    padvance->name.translated = ('\0' == padvance->name.vernacular[0])
+				? padvance->name.vernacular
+				: Q_(padvance->name.vernacular);
   }
-  return advances[tech].name.translated;
+  return padvance->name.translated;
 }
 
 /****************************************************************************
   Return the (untranslated) rule name of the advance/technology.
   You don't have to free the return pointer.
 ****************************************************************************/
-const char *advance_rule_name(Tech_type_id tech)
+const char *advance_rule_name(const struct advance *padvance)
 {
-  return Qn_(advances[tech].name.vernacular); 
+  return Qn_(padvance->name.vernacular); 
 }
 
 /**************************************************************************
@@ -782,7 +881,7 @@ void techs_init(void)
   int i;
 
   for (i = 0; i < ARRAY_SIZE(advances); i++) {
-    advances[i].index = i;
+    advances[i].item_number = i;
   }
 
   /* Initialize dummy tech A_NONE */
@@ -825,11 +924,9 @@ static void tech_free(Tech_type_id tech)
 ***************************************************************/
 void techs_free(void)
 {
-  Tech_type_id i;
-
-  for (i = A_FIRST; i < game.control.num_tech_types; i++) {
+  advance_index_iterate(A_FIRST, i) {
     tech_free(i);
-  }
+  } advance_index_iterate_end;
 }
 
 /***************************************************************
@@ -839,5 +936,6 @@ void player_research_init(struct player_research* research)
 {
   memset(research, 0, sizeof(*research));
   research->tech_goal = A_UNSET;
-  research->changed_from = -1;
+  research->researching = A_UNSET;
+  research->researching_saved = A_UNKNOWN;
 }
