@@ -36,19 +36,30 @@ enum tile_special_type infrastructure_specials[] = {
   S_IRRIGATION,
   S_FARMLAND,
   S_MINE,
-  S_FORTRESS,
-  S_AIRBASE,
+  S_OLD_FORTRESS,
+  S_OLD_AIRBASE,
   S_LAST
 };
 
+static const char *terrain_class_names[] = {
+  N_("Land"),
+  N_("Oceanic")
+};
+
+static const char *terrain_alteration_names[] = {
+  N_("CanIrrigate"),
+  N_("CanMine"),
+  N_("CanRoad")
+};
+
 /* T_UNKNOWN isn't allowed here. */
-#define SANITY_CHECK_TERRAIN(pterrain)					    \
-  assert((pterrain)->index >= 0						    \
-	 && (pterrain)->index < game.control.terrain_count		    \
-	 && &civ_terrains[(pterrain)->index] == (pterrain))
+#define SANITY_CHECK_TERRAIN(pterrain)					\
+  assert((pterrain)->item_number >= 0					\
+	 && (pterrain)->item_number < terrain_count()			\
+	 && &civ_terrains[terrain_index(pterrain)] == (pterrain))
 
 /****************************************************************************
-  Inialize terrain structures.
+  Initialize terrain and resource structures.
 ****************************************************************************/
 void terrains_init(void)
 {
@@ -56,19 +67,110 @@ void terrains_init(void)
 
   for (i = 0; i < ARRAY_SIZE(civ_terrains); i++) {
     /* Can't use terrain_by_number here because it does a bounds check. */
-    civ_terrains[i].index = i;
+    civ_terrains[i].item_number = i;
   }
   for (i = 0; i < ARRAY_SIZE(civ_resources); i++) {
-    civ_resources[i].index = i;
+    civ_resources[i].item_number = i;
   }
 }
 
+/****************************************************************************
+  Free memory which is associated with terrain types.
+****************************************************************************/
+void terrains_free(void)
+{
+  terrain_type_iterate(pterrain) {
+    if (pterrain->helptext != NULL) {
+      free(pterrain->helptext);
+      pterrain->helptext = NULL;
+    }
+    if (pterrain->resources != NULL) {
+      /* Server allocates this on ruleset loading, client when
+       * ruleset packet is received. */
+      free(pterrain->resources);
+      pterrain->resources = NULL;
+    }
+  } terrain_type_iterate_end;
+}
+
+/**************************************************************************
+  Return the first item of terrains.
+**************************************************************************/
+struct terrain *terrain_array_first(void)
+{
+  if (game.control.terrain_count > 0) {
+    return civ_terrains;
+  }
+  return NULL;
+}
+
+/**************************************************************************
+  Return the last item of terrains.
+**************************************************************************/
+const struct terrain *terrain_array_last(void)
+{
+  if (game.control.terrain_count > 0) {
+    return &civ_terrains[game.control.terrain_count - 1];
+  }
+  return NULL;
+}
+
+/**************************************************************************
+  Return the number of terrains.
+**************************************************************************/
+Terrain_type_id terrain_count(void)
+{
+  return game.control.terrain_count;
+}
+
+/**************************************************************************
+  Return the terrain identifier.
+**************************************************************************/
+char terrain_identifier(const struct terrain *pterrain)
+{
+  assert(pterrain);
+  return pterrain->identifier;
+}
+
+/**************************************************************************
+  Return the terrain index.
+
+  Currently same as terrain_number(), paired with terrain_count()
+  indicates use as an array index.
+**************************************************************************/
+Terrain_type_id terrain_index(const struct terrain *pterrain)
+{
+  assert(pterrain);
+  return pterrain - civ_terrains;
+}
+
+/**************************************************************************
+  Return the terrain index.
+**************************************************************************/
+Terrain_type_id terrain_number(const struct terrain *pterrain)
+{
+  assert(pterrain);
+  return pterrain->item_number;
+}
+
+/****************************************************************************
+  Return the terrain for the given terrain index.
+****************************************************************************/
+struct terrain *terrain_by_number(const Terrain_type_id type)
+{
+  if (type < 0 || type >= game.control.terrain_count) {
+    /* This isn't an error; some T_UNKNOWN callers depend on it. */
+    return NULL;
+  }
+  return &civ_terrains[type];
+}
+
 /****************************************************************************
   Return the terrain type matching the identifier, or T_UNKNOWN if none matches.
 ****************************************************************************/
-struct terrain *terrain_by_identifier(const char identifier)
+struct terrain *find_terrain_by_identifier(const char identifier)
 {
-  if (UNKNOWN_TERRAIN_IDENTIFIER == identifier) {
+  if (TERRAIN_UNKNOWN_IDENTIFIER == identifier) {
     return T_UNKNOWN;
   }
   terrain_type_iterate(pterrain) {
@@ -80,18 +182,6 @@ struct terrain *terrain_by_identifier(const char identifier)
   return T_UNKNOWN;
 }
 
-/****************************************************************************
-  Return the terrain for the given terrain index.
-****************************************************************************/
-struct terrain *terrain_by_number(Terrain_type_id type)
-{
-  if (type < 0 || type >= game.control.terrain_count) {
-    /* This isn't an error; some T_UNKNOWN callers depend on it. */
-    return NULL;
-  }
-  return &civ_terrains[type];
-}
-
 /****************************************************************************
   Return the terrain type matching the name, or T_UNKNOWN if none matches.
 ****************************************************************************/
@@ -122,13 +212,53 @@ struct terrain *find_terrain_by_translated_name(const char *name)
   return T_UNKNOWN;
 }
 
+/****************************************************************************
+  Return terrain having the flag. If several terrains have the flag,
+  random one is returned.
+****************************************************************************/
+struct terrain *rand_terrain_by_flag(enum terrain_flag_id flag)
+{
+  int num = 0;
+  struct terrain *terr = NULL;
+
+  terrain_type_iterate(pterr) {
+    if (terrain_has_flag(pterr, flag)) {
+      num++;
+      if (myrand(num) == 1) {
+        terr = pterr;
+      }
+    }
+  } terrain_type_iterate_end;
+
+  return terr;
+}
+
+/****************************************************************************
+  Fill terrain with flag to buffer. Returns number of terrains found.
+  Return value can be greater than size of buffer.
+****************************************************************************/
+int terrains_by_flag(enum terrain_flag_id flag, struct terrain **buffer, int bufsize)
+{
+  int num = 0;
+
+  terrain_type_iterate(pterr) {
+    if (terrain_has_flag(pterr, flag)) {
+      if (num < bufsize) {
+        buffer[num] = pterr;
+      }
+      num++;
+    }
+  } terrain_type_iterate_end;
+
+  return num;
+}
+
 /****************************************************************************
   Return the (translated) name of the terrain.
   You don't have to free the return pointer.
 ****************************************************************************/
 const char *terrain_name_translation(struct terrain *pterrain)
 {
-  SANITY_CHECK_TERRAIN(pterrain);
   if (NULL == pterrain->name.translated) {
     /* delayed (unified) translation */
     pterrain->name.translated = ('\0' == pterrain->name.vernacular[0])
@@ -162,8 +292,8 @@ enum terrain_flag_id find_terrain_flag_by_rule_name(const char *s)
     "Starter",
     "CanHaveRiver",
     "UnsafeCoast",
-    "Unsafe",
-    "Oceanic"
+    "Oceanic",
+    "Freshwater"
   };
 
   assert(ARRAY_SIZE(flag_names) == TER_COUNT);
@@ -177,41 +307,11 @@ enum terrain_flag_id find_terrain_flag_by_rule_name(const char *s)
   return TER_LAST;
 }
 
-/****************************************************************************
-  Return a random terrain that has the specified flag.  Returns T_UNKNOWN if
-  there is no matching terrain.
-  FIXME: currently called only by mapgen.c, move there and check error.
-****************************************************************************/
-struct terrain *pick_terrain_by_flag(enum terrain_flag_id flag)
-{
-  bool has_flag[T_COUNT];
-  int count = 0;
-
-  terrain_type_iterate(pterrain) {
-    if ((has_flag[pterrain->index] = terrain_has_flag(pterrain, flag))) {
-      count++;
-    }
-  } terrain_type_iterate_end;
-
-  count = myrand(count);
-  terrain_type_iterate(pterrain) {
-    if (has_flag[pterrain->index]) {
-      if (count == 0) {
-	return pterrain;
-      }
-      count--;
-    }
-  } terrain_type_iterate_end;
-
-  die("Reached end of pick_terrain_by_flag!");
-  return T_UNKNOWN;
-}
-
 /****************************************************************************
   Check for resource in terrain resources list.
 ****************************************************************************/
 bool terrain_has_resource(const struct terrain *pterrain,
-			  const struct resource *presource)
+                          const struct resource *presource)
 {
   struct resource **r = pterrain->resources;
 
@@ -224,29 +324,61 @@ bool terrain_has_resource(const struct terrain *pterrain,
   return FALSE;
 }
 
-/****************************************************************************
-  Free memory which is associated with terrain types.
-****************************************************************************/
-void terrains_free(void)
+/**************************************************************************
+  Return the first item of resources.
+**************************************************************************/
+struct resource *resource_array_first(void)
 {
-  terrain_type_iterate(pterrain) {
-    if (pterrain->helptext != NULL) {
-      free(pterrain->helptext);
-      pterrain->helptext = NULL;
-    }
-    if (pterrain->resources != NULL) {
-      /* Server allocates this on ruleset loading, client when
-       * ruleset packet is received. */
-      free(pterrain->resources);
-      pterrain->resources = NULL;
-    }
-  } terrain_type_iterate_end;
+  if (game.control.resource_count > 0) {
+    return civ_resources;
+  }
+  return NULL;
+}
+
+/**************************************************************************
+  Return the last item of resources.
+**************************************************************************/
+const struct resource *resource_array_last(void)
+{
+  if (game.control.resource_count > 0) {
+    return &civ_resources[game.control.resource_count - 1];
+  }
+  return NULL;
+}
+
+/**************************************************************************
+  Return the resource count.
+**************************************************************************/
+Resource_type_id resource_count(void)
+{
+  return game.control.resource_count;
+}
+
+/**************************************************************************
+  Return the resource index.
+
+  Currently same as resource_number(), paired with resource_count()
+  indicates use as an array index.
+**************************************************************************/
+Resource_type_id resource_index(const struct resource *presource)
+{
+  assert(presource);
+  return presource - civ_resources;
+}
+
+/**************************************************************************
+  Return the resource index.
+**************************************************************************/
+Resource_type_id resource_number(const struct resource *presource)
+{
+  assert(presource);
+  return presource->item_number;
 }
 
 /****************************************************************************
   Return the resource for the given resource index.
 ****************************************************************************/
-struct resource *resource_by_number(Resource_type_id type)
+struct resource *resource_by_number(const Resource_type_id type)
 {
   if (type < 0 || type >= game.control.resource_count) {
     /* This isn't an error; some callers depend on it. */
@@ -256,14 +388,28 @@ struct resource *resource_by_number(Resource_type_id type)
 }
 
 /****************************************************************************
-  Return the resource type matching the name, or T_UNKNOWN if none matches.
+  Return the resource type matching the identifier, or NULL when none matches.
+****************************************************************************/
+struct resource *find_resource_by_identifier(const char identifier)
+{
+  resource_type_iterate(presource) {
+    if (presource->identifier == identifier) {
+      return presource;
+    }
+  } resource_type_iterate_end;
+
+  return NULL;
+}
+
+/****************************************************************************
+  Return the resource type matching the name, or NULL when none matches.
 ****************************************************************************/
 struct resource *find_resource_by_rule_name(const char *name)
 {
   const char *qname = Qn_(name);
 
   resource_type_iterate(presource) {
-    if (0 == strcmp(resource_rule_name(presource), qname)) {
+    if (0 == mystrcasecmp(resource_rule_name(presource), qname)) {
       return presource;
     }
   } resource_type_iterate_end;
@@ -332,7 +478,7 @@ bool is_terrain_near_tile(const struct tile *ptile,
   }
 
   adjc_iterate(ptile, adjc_tile) {
-    if (adjc_tile->terrain == pterrain) {
+    if (tile_terrain(adjc_tile) == pterrain) {
       return TRUE;
     }
   } adjc_iterate_end;
@@ -350,7 +496,7 @@ int count_terrain_near_tile(const struct tile *ptile,
   int count = 0, total = 0;
 
   variable_adjc_iterate(ptile, adjc_tile, cardinal_only) {
-    if (pterrain && tile_get_terrain(adjc_tile) == pterrain) {
+    if (pterrain && tile_terrain(adjc_tile) == pterrain) {
       count++;
     }
     total++;
@@ -372,7 +518,8 @@ int count_terrain_property_near_tile(const struct tile *ptile,
   int count = 0, total = 0;
 
   variable_adjc_iterate(ptile, adjc_tile, cardinal_only) {
-    if (adjc_tile->terrain->property[prop] > 0) {
+    struct terrain *pterrain = tile_terrain(adjc_tile);
+    if (pterrain->property[prop] > 0) {
       count++;
     }
     total++;
@@ -395,10 +542,10 @@ static const char *tile_special_type_names[] =
   N_("Mine"),
   N_("Pollution"),
   N_("Hut"),
-  N_("Fortress"),
+  N_("Fortress"), /* Obsolete, placeholder for backward compatibility */
   N_("River"),
   N_("Farmland"),
-  N_("Airbase"),
+  N_("Airbase"),  /* Obsolete, placeholder for backward compatibility */
   N_("Fallout")
 };
 
@@ -474,6 +621,64 @@ bool contains_special(bv_special set,
   return BV_ISSET(set, to_test_for);
 }
 
+/****************************************************************************
+ Returns TRUE iff any specials are set on the tile.
+****************************************************************************/
+bool contains_any_specials(bv_special set)
+{
+  return BV_ISSET_ANY(set);
+}
+
+/****************************************************************************
+  Returns TRUE iff the special can be supported by the terrain type.
+****************************************************************************/
+bool is_native_terrain_to_special(enum tile_special_type special,
+                                  const struct terrain *pterrain)
+{
+  /* FIXME: The special definition should be moved into the ruleset. */
+  switch (special) {
+  case S_ROAD:
+    return (terrain_control.may_road
+            && 0 != pterrain->road_time);
+  case S_IRRIGATION:
+    return (terrain_control.may_irrigate
+            && pterrain == pterrain->irrigation_result);
+  case S_RAILROAD:
+    return (terrain_control.may_road
+            && 0 != pterrain->road_time);
+  case S_MINE:
+    return (terrain_control.may_mine
+            && pterrain == pterrain->mining_result);
+  case S_POLLUTION:
+    return !terrain_has_flag(pterrain, TER_NO_POLLUTION);
+  case S_HUT:
+    return TRUE;
+  case S_RIVER:
+    return terrain_has_flag(pterrain, TER_CAN_HAVE_RIVER);
+  case S_FARMLAND:
+    return (terrain_control.may_irrigate
+            && pterrain == pterrain->irrigation_result);
+  case S_FALLOUT:
+    return !terrain_has_flag(pterrain, TER_NO_POLLUTION);
+  case S_OLD_FORTRESS:
+  case S_OLD_AIRBASE:
+  case S_LAST:
+    break;
+  }
+
+  return FALSE;
+}
+
+/****************************************************************************
+  Returns TRUE iff the special can be supported by the terrain type of the
+  tile.
+****************************************************************************/
+bool is_native_tile_to_special(enum tile_special_type special,
+                               const struct tile *ptile)
+{
+  return is_native_terrain_to_special(special, tile_terrain(ptile));
+}
+
 /****************************************************************************
   Returns TRUE iff any tile adjacent to (map_x,map_y) has the given special.
 ****************************************************************************/
@@ -518,12 +723,9 @@ bool is_terrain_flag_near_tile(const struct tile *ptile,
 			       enum terrain_flag_id flag)
 {
   adjc_iterate(ptile, adjc_tile) {
-    struct terrain* pterrain = tile_get_terrain(adjc_tile);
-    if (pterrain == NULL) {
-      continue;
-    }
-    
-    if (terrain_has_flag(pterrain, flag)) {
+    struct terrain* pterrain = tile_terrain(adjc_tile);
+    if (T_UNKNOWN != pterrain
+	&& terrain_has_flag(pterrain, flag)) {
       return TRUE;
     }
   } adjc_iterate_end;
@@ -541,8 +743,9 @@ int count_terrain_flag_near_tile(const struct tile *ptile,
   int count = 0, total = 0;
 
   variable_adjc_iterate(ptile, adjc_tile, cardinal_only) {
-    if (adjc_tile->terrain != T_UNKNOWN
-	&& terrain_has_flag(adjc_tile->terrain, flag)) {
+    struct terrain *pterrain = tile_terrain(adjc_tile);
+    if (T_UNKNOWN != pterrain
+	&& terrain_has_flag(pterrain, flag)) {
       count++;
     }
     total++;
@@ -560,11 +763,11 @@ int count_terrain_flag_near_tile(const struct tile *ptile,
     eg: "Road/Farmland"
   This only includes "infrastructure", i.e., man-made specials.
 ****************************************************************************/
-const char *get_infrastructure_text(bv_special spe)
+const char *get_infrastructure_text(bv_special spe, bv_bases bases)
 {
   static char s[256];
   char *p;
-  
+
   s[0] = '\0';
 
   /* Since railroad requires road, Road/Railroad is redundant */
@@ -585,13 +788,11 @@ const char *get_infrastructure_text(bv_special spe)
     cat_snprintf(s, sizeof(s), "%s/", _("Mine"));
   }
 
-  if (contains_special(spe, S_FORTRESS)) {
-    cat_snprintf(s, sizeof(s), "%s/", _("Fortress"));
-  }
-
-  if (contains_special(spe, S_AIRBASE)) {
-    cat_snprintf(s, sizeof(s), "%s/", _("Airbase"));
-  }
+  base_type_iterate(pbase) {
+    if (BV_ISSET(bases, base_index(pbase))) {
+      cat_snprintf(s, sizeof(s), "%s/", base_name_translation(pbase));
+    }
+  } base_type_iterate_end;
 
   p = s + strlen(s) - 1;
   if (*p == '/') {
@@ -617,10 +818,12 @@ enum tile_special_type get_infrastructure_prereq(enum tile_special_type spe)
 
 /****************************************************************************
   Returns the highest-priority (best) infrastructure (man-made special) to
-  be pillaged from the terrain set.  May return S_NO_SPECIAL if nothing
+  be pillaged from the terrain set.  May return S_LAST if nothing
   better is available.
+  Bases are encoded as numbers beyond S_LAST.
 ****************************************************************************/
-enum tile_special_type get_preferred_pillage(bv_special pset)
+int get_preferred_pillage(bv_special pset,
+                          bv_bases bases)
 {
   if (contains_special(pset, S_FARMLAND)) {
     return S_FARMLAND;
@@ -631,12 +834,11 @@ enum tile_special_type get_preferred_pillage(bv_special pset)
   if (contains_special(pset, S_MINE)) {
     return S_MINE;
   }
-  if (contains_special(pset, S_FORTRESS)) {
-    return S_FORTRESS;
-  }
-  if (contains_special(pset, S_AIRBASE)) {
-    return S_AIRBASE;
-  }
+  base_type_iterate(pbase) {
+    if (BV_ISSET(bases, base_index(pbase))) {
+      return S_LAST + base_index(pbase) + 1;
+    }
+  } base_type_iterate_end;
   if (contains_special(pset, S_RAILROAD)) {
     return S_RAILROAD;
   }
@@ -645,3 +847,163 @@ enum tile_special_type get_preferred_pillage(bv_special pset)
   }
   return S_LAST;
 }
+
+/****************************************************************************
+  Does terrain type belong to terrain class?
+****************************************************************************/
+bool terrain_belongs_to_class(const struct terrain *pterrain,
+                              enum terrain_class class)
+{
+  switch(class) {
+   case TC_LAND:
+     return !is_ocean(pterrain);
+   case TC_OCEAN:
+     return is_ocean(pterrain);
+   case TC_LAST:
+     return FALSE;
+  }
+
+  assert(FALSE);
+  return FALSE;
+}
+
+/****************************************************************************
+  Is there terrain of the given class near tile?
+****************************************************************************/
+bool is_terrain_class_near_tile(const struct tile *ptile, enum terrain_class class)
+{
+  switch(class) {
+   case TC_LAND:
+     adjc_iterate(ptile, adjc_tile) {
+       struct terrain* pterrain = tile_terrain(adjc_tile);
+       if (T_UNKNOWN != pterrain
+           && !terrain_has_flag(pterrain, TER_OCEANIC)) {
+         return TRUE;
+       }
+     } adjc_iterate_end;
+     return FALSE;
+   case TC_OCEAN:
+     return is_ocean_near_tile(ptile);
+   case TC_LAST:
+     return FALSE;
+  }
+
+  assert(FALSE);
+  return FALSE;
+}
+
+
+/****************************************************************************
+  Return the terrain class value matching name, or TC_LAST if none matches.
+****************************************************************************/
+enum terrain_class find_terrain_class_by_rule_name(const char *name)
+{
+  int i;
+
+  for (i = 0; i < TC_LAST; i++) {
+    if (0 == strcmp(terrain_class_names[i], name)) {
+      return i;
+    }
+  }
+
+  return TC_LAST;
+}
+
+/****************************************************************************
+  Return the (untranslated) rule name of the given terrain class.
+  You don't have to free the return pointer.
+****************************************************************************/
+const char *terrain_class_rule_name(enum terrain_class tclass)
+{
+  if (tclass < 0 || tclass >= TC_LAST) {
+    return NULL;
+  }
+
+  return terrain_class_names[tclass];
+}
+
+/****************************************************************************
+  Return the (translated) name of the given terrain class.
+  You don't have to free the return pointer.
+****************************************************************************/
+const char *terrain_class_name_translation(enum terrain_class tclass)
+{
+  if (tclass < 0 || tclass >= TC_LAST) {
+    return NULL;
+  }
+
+  return _(terrain_class_names[tclass]);
+}
+
+/****************************************************************************
+  Can terrain support given infrastructure?
+****************************************************************************/
+bool terrain_can_support_alteration(const struct terrain *pterrain,
+                                    enum terrain_alteration alter)
+{
+  switch (alter) {
+   case TA_CAN_IRRIGATE:
+     return (terrain_control.may_irrigate
+          && (pterrain == pterrain->irrigation_result));
+   case TA_CAN_MINE:
+     return (terrain_control.may_mine
+          && (pterrain == pterrain->mining_result));
+   case TA_CAN_ROAD:
+     return (terrain_control.may_road && (pterrain->road_time > 0));
+   case TA_LAST:
+   default:
+     break;
+  }
+
+  assert(FALSE);
+  return FALSE;
+}
+
+/****************************************************************************
+  Return the terrain alteration possibility value matching name, or TA_LAST
+  if none matches.
+****************************************************************************/
+enum terrain_alteration find_terrain_alteration_by_rule_name(const char *name)
+{
+  int i;
+
+  for (i = 0; i < TA_LAST; i++) {
+    if (0 == strcmp(terrain_alteration_names[i], name)) {
+      return i;
+    }
+  }
+
+  return TA_LAST;
+}
+
+/****************************************************************************
+  Return the (untranslated) rule name of the given terrain alteration.
+  You don't have to free the return pointer.
+****************************************************************************/
+const char *terrain_alteration_rule_name(enum terrain_alteration talter)
+{
+  if (talter < 0 || talter >= TA_LAST) {
+    return NULL;
+  }
+
+  return terrain_alteration_names[talter];
+}
+
+/****************************************************************************
+  Return the (translated) name of the infrastructure (e.g., "Irrigation").
+  You don't have to free the return pointer.
+****************************************************************************/
+const char *terrain_alteration_name_translation(enum terrain_alteration talter)
+{
+  switch(talter) {
+   case TA_CAN_IRRIGATE:
+     return special_name_translation(S_IRRIGATION);
+   case TA_CAN_MINE:
+     return special_name_translation(S_MINE);
+   case TA_CAN_ROAD:
+     return special_name_translation(S_ROAD);
+   case TA_LAST:
+   default:
+     return NULL;
+  }
+}
