@@ -21,6 +21,7 @@ enum start_mode {
   MT_VARIABLE
 };
 
-bool create_start_positions(enum start_mode mode);
+bool create_start_positions(enum start_mode mode,
+			    struct unit_type *initial_unit);
 
 #endif
