@@ -66,6 +66,7 @@
 #include "dataio.h"
 #include "events.h"
 #include "fcintl.h"
+#include "game.h"
 #include "log.h"
 #include "mem.h"
 #include "netintf.h"
@@ -82,6 +83,7 @@
 #include "plrhand.h"
 #include "srv_main.h"
 #include "stdinhand.h"
+#include "voting.h"
 
 #include "sernet.h"
 
@@ -91,7 +93,8 @@ static struct connection connections[MAX_NUM_CONNECTIONS];
 TEndpointInfo serv_info;
 EndpointRef serv_ep;
 #else
-static int sock;
+static int *listen_socks;
+static int listen_count;
 static int socklan;
 #endif
 
@@ -116,6 +119,9 @@ static int socklan;
 #define SPECLIST_TAG timer
 #define SPECLIST_TYPE struct timer
 #include "speclist.h"
+#define timer_list_iterate(ARG_list, NAME_item) \
+  TYPED_LIST_ITERATE(struct timer, (ARG_list), NAME_item)
+#define timer_list_iterate_end LIST_ITERATE_END
 
 #define PROCESSING_TIME_STATISTICS 0
 
@@ -196,22 +202,29 @@ static void handle_readline_input_callback(char *line)
 *****************************************************************************/
 void close_connection(struct connection *pconn)
 {
-  while (timer_list_size(pconn->server.ping_timers) > 0) {
-    struct timer *timer = timer_list_get(pconn->server.ping_timers, 0);
+  if (!pconn) {
+    return;
+  }
 
-    timer_list_unlink(pconn->server.ping_timers, timer);
-    free_timer(timer);
+  cancel_connection_votes(pconn);
+
+  if (pconn->server.ping_timers != NULL) {
+    timer_list_iterate(pconn->server.ping_timers, timer) {
+      free_timer(timer);
+    } timer_list_iterate_end;
+    timer_list_free(pconn->server.ping_timers);
+    pconn->server.ping_timers = NULL;
   }
-  assert(timer_list_size(pconn->server.ping_timers) == 0);
-  timer_list_free(pconn->server.ping_timers);
 
   /* safe to do these even if not in lists: */
   conn_list_unlink(game.all_connections, pconn);
   conn_list_unlink(game.est_connections, pconn);
 
-  pconn->player = NULL;
+  pconn->playing = NULL;
   pconn->access_level = ALLOW_NONE;
   connection_common_close(pconn);
+
+  send_updated_vote_totals(NULL);
 }
 
 /*****************************************************************************
@@ -226,7 +239,6 @@ void close_connections_and_socket(void)
     if(connections[i].used) {
       close_connection(&connections[i]);
     }
-    conn_list_unlink_all(connections[i].self);
     conn_list_free(connections[i].self);
   }
 
@@ -234,8 +246,10 @@ void close_connections_and_socket(void)
   conn_list_free(game.all_connections);
   conn_list_free(game.est_connections);
 
-  my_closesocket(sock);
-  my_closesocket(socklan);
+  for (i = 0; i < listen_count; i++) {
+    fc_closesocket(listen_socks[i]);
+  }
+  fc_closesocket(socklan);
 
 #ifdef HAVE_LIBREADLINE
   if (history_file) {
@@ -247,7 +261,7 @@ void close_connections_and_socket(void)
   send_server_info_to_metaserver(META_GOODBYE);
   server_close_meta();
 
-  my_shutdown_network();
+  fc_shutdown_network();
 }
 
 /*****************************************************************************
@@ -324,7 +338,7 @@ void flush_packets(void)
       return;
     }
 
-    if(my_select(max_desc+1, NULL, &writefs, &exceptfs, &tv)<=0) {
+    if(fc_select(max_desc+1, NULL, &writefs, &exceptfs, &tv)<=0) {
       return;
     }
 
@@ -427,8 +441,9 @@ functions.  That is, other functions should not need to do so.  --dwp
 *****************************************************************************/
 enum server_events server_sniff_all_input(void)
 {
-  int i;
+  int i, s;
   int max_desc;
+  bool excepting;
   fd_set readfs, writefs, exceptfs;
   struct timeval tv;
 #ifdef SOCKET_ZERO_ISNT_STDIN
@@ -488,9 +503,9 @@ enum server_events server_sniff_all_input(void)
       if (connections && conn_list_size(game.est_connections) == 0) {
 	if (last_noplayers != 0) {
 	  if (time(NULL) > last_noplayers + srvarg.quitidle) {
-	    save_game_auto("Lost all connections");
-	    set_meta_message_string("restarting for lack of players");
-	    freelog(LOG_NORMAL, get_meta_message_string());
+	    save_game_auto("Lost all connections", "quitidle");
+            freelog(LOG_NORMAL, _("Restarting for lack of players."));
+            set_meta_message_string("restarting for lack of players");
 	    (void) send_server_info_to_metaserver(META_INFO);
 
             set_server_state(S_S_OVER);
@@ -509,14 +524,13 @@ enum server_events server_sniff_all_input(void)
             connections = FALSE;
 	  }
 	} else {
-          char buf[256];
 	  last_noplayers = time(NULL);
-	  
-	  my_snprintf(buf, sizeof(buf),
-		      "restarting in %d seconds for lack of players",
-		      srvarg.quitidle);
-          set_meta_message_string((const char *)buf);
-	  freelog(LOG_NORMAL, get_meta_message_string());
+
+	  freelog(LOG_NORMAL,
+		  _("restarting in %d seconds for lack of players"),
+		  srvarg.quitidle);
+
+          set_meta_message_string(N_("restarting soon for lack of players"));
 	  (void) send_server_info_to_metaserver(META_INFO);
 	}
       } else {
@@ -525,7 +539,7 @@ enum server_events server_sniff_all_input(void)
     }
 
     /* Pinging around for statistics */
-    if (time(NULL) > (game.last_ping + game.info.pingtime)) {
+    if (time(NULL) > (game.server.last_ping + game.info.pingtime)) {
       /* send data about the previous run */
       send_ping_times_to_all();
 
@@ -548,7 +562,7 @@ enum server_events server_sniff_all_input(void)
 	  ping_connection(pconn);
 	}
       } conn_list_iterate_end;
-      game.last_ping = time(NULL);
+      game.server.last_ping = time(NULL);
     }
 
     /* if we've waited long enough after a failure, respond to the client */
@@ -559,7 +573,7 @@ enum server_events server_sniff_all_input(void)
     } conn_list_iterate_end
 
     /* Don't wait if timeout == -1 (i.e. on auto games) */
-    if (S_S_INITIAL != server_state() && game.info.timeout == -1) {
+    if (S_S_RUNNING == server_state() && game.info.timeout == -1) {
       (void) send_server_info_to_metaserver(META_REFRESH);
       return S_E_END_OF_TURN_TIMEOUT;
     }
@@ -581,17 +595,20 @@ enum server_events server_sniff_all_input(void)
 #endif /* SOCKET_ZERO_ISNT_STDIN */
     }
 
+    max_desc = 0;
+    for (i = 0; i < listen_count; i++) {
+      FD_SET(listen_socks[i], &readfs);
+      FD_SET(listen_socks[i], &exceptfs);
+      max_desc = MAX(max_desc, listen_socks[i]);
+    }
+
     if (with_ggz) {
 #ifdef GGZ_SERVER
       int ggz_sock = get_ggz_socket();
 
       FD_SET(ggz_sock, &readfs);
-      max_desc = MAX(sock, ggz_sock);
+      max_desc = MAX(max_desc, ggz_sock);
 #endif /* GGZ_SERVER */
-    } else {
-      FD_SET(sock, &readfs);
-      FD_SET(sock, &exceptfs);
-      max_desc = sock;
     }
 
     for (i = 0; i < MAX_NUM_CONNECTIONS; i++) {
@@ -606,13 +623,13 @@ enum server_events server_sniff_all_input(void)
     }
     con_prompt_off();		/* output doesn't generate a new prompt */
 
-    if (my_select(max_desc + 1, &readfs, &writefs, &exceptfs, &tv) == 0) {
+    if (fc_select(max_desc + 1, &readfs, &writefs, &exceptfs, &tv) == 0) {
       /* timeout */
       (void) send_server_info_to_metaserver(META_REFRESH);
       if (game.info.timeout > 0
 	  && S_S_RUNNING == server_state()
-	  && game.phase_timer
-	  && (read_timer_seconds(game.phase_timer)
+	  && game.server.phase_timer
+	  && (read_timer_seconds(game.server.phase_timer)
 	      > game.info.seconds_to_phasedone)) {
 	con_prompt_off();
 	return S_E_END_OF_TURN_TIMEOUT;
@@ -650,14 +667,28 @@ enum server_events server_sniff_all_input(void)
     }
 
     if (!with_ggz) { /* No listening socket when using GGZ. */
-      if (FD_ISSET(sock, &exceptfs)) {	     /* handle Ctrl-Z suspend/resume */
+      excepting = FALSE;
+      for (i = 0; i < listen_count; i++) {
+        if (FD_ISSET(listen_socks[i], &exceptfs)) {
+          excepting = TRUE;
+          break;
+        }
+      }
+      if (excepting) {                  /* handle Ctrl-Z suspend/resume */
 	continue;
       }
-      if(FD_ISSET(sock, &readfs)) {	     /* new players connects */
-	freelog(LOG_VERBOSE, "got new connection");
-	if(server_accept_connection(sock)==-1) {
-	  freelog(LOG_ERROR, "failed accepting connection");
-	}
+      for (i = 0; i < listen_count; i++) {
+        s = listen_socks[i];
+        if (FD_ISSET(s, &readfs)) {     /* new players connects */
+          freelog(LOG_VERBOSE, "got new connection");
+          if (-1 == server_accept_connection(s)) {
+            /* There will be a LOG_ERROR message from
+             * server_accept_connection() if something
+             * goes wrong, so no need to make another
+             * error-level message here. */
+            freelog(LOG_VERBOSE, "failed accepting connection");
+          }
+        }
       }
     }
     for (i = 0; i < MAX_NUM_CONNECTIONS; i++) {
@@ -711,7 +742,8 @@ enum server_events server_sniff_all_input(void)
       if (didget >= 1) {
         buffer[didget-1] = '\0'; /* overwrite newline character */
         didget--;
-        freelog(LOG_DEBUG, "Got line: \"%s\" (%ld, %ld)", buffer, didget, len);
+        freelog(LOG_DEBUG, "Got line: \"%s\" (%ld, %ld)", buffer,
+                (long int) didget, (long int) len);
       }
 #else  /* HAVE_GETLINE */
       buffer = malloc(BUF_SIZE + 1);
@@ -774,8 +806,9 @@ enum server_events server_sniff_all_input(void)
 
   if (game.info.timeout > 0
       && S_S_RUNNING == server_state()
-      && game.phase_timer
-      && read_timer_seconds(game.phase_timer) > game.info.seconds_to_phasedone) {
+      && game.server.phase_timer
+      && (read_timer_seconds(game.server.phase_timer)
+          > game.info.seconds_to_phasedone)) {
     return S_E_END_OF_TURN_TIMEOUT;
   }
   return S_E_OTHERWISE;
@@ -820,24 +853,70 @@ static int server_accept_connection(int sockfd)
   socklen_t fromlen;
 
   int new_sock;
-  union my_sockaddr fromend;
+  union fc_sockaddr fromend;
+  bool nameinfo = FALSE;
+#ifdef IPV6_SUPPORT
+  char host[NI_MAXHOST], service[NI_MAXSERV];
+  char dst[INET6_ADDRSTRLEN];
+#else  /* IPv6 support */
   struct hostent *from;
+  const char *host = NULL;
+  const char *dst;
+#endif /* IPv6 support */
 
   fromlen = sizeof(fromend);
 
-  if ((new_sock = accept(sockfd, &fromend.sockaddr, &fromlen)) == -1) {
-    freelog(LOG_ERROR, "accept failed: %s", mystrerror());
+  if ((new_sock = accept(sockfd, &fromend.saddr, &fromlen)) == -1) {
+    freelog(LOG_ERROR, "accept failed: %s", fc_strerror(fc_get_errno()));
     return -1;
   }
 
-  from =
-      gethostbyaddr((char *) &fromend.sockaddr_in.sin_addr,
-		    sizeof(fromend.sockaddr_in.sin_addr), AF_INET);
+#ifdef IPV6_SUPPORT
+  if (fromend.saddr.sa_family == AF_INET6) {
+    inet_ntop(AF_INET6, &fromend.saddr_in6.sin6_addr,
+              dst, sizeof(dst));
+  } else {
+    inet_ntop(AF_INET, &fromend.saddr_in4.sin_addr, dst, sizeof(dst));
+  }
+#else  /* IPv6 support */
+  dst = inet_ntoa(fromend.saddr_in4.sin_addr);
+#endif /* IPv6 support */
+
+  if (0 != game.server.maxconnectionsperhost) {
+    int count = 0;
+
+    conn_list_iterate(game.all_connections, pconn) {
+      if (0 != strcmp(dst, pconn->server.ipaddr)) {
+        continue;
+      }
+      if (++count >= game.server.maxconnectionsperhost) {
+        freelog(LOG_VERBOSE, "Rejecting new connection from %s: maximum "
+                "number of connections for this address exceeded (%d).",
+                dst, game.server.maxconnectionsperhost);
+
+        /* Disconnect the accepted socket. */
+        fc_closesocket(new_sock);
+
+        return -1;
+      }
+    } conn_list_iterate_end;
+  }
+
+#ifdef IPV6_SUPPORT
+  nameinfo = (0 == getnameinfo(&fromend.saddr, fromlen, host, NI_MAXHOST,
+                               service, NI_MAXSERV, NI_NUMERICSERV)
+              && '\0' != host[0]);
+#else  /* IPv6 support */
+  from = gethostbyaddr((char *) &fromend.saddr_in4.sin_addr,
+                       sizeof(fromend.saddr_in4.sin_addr), AF_INET);
+  if (NULL != from && '\0' != from->h_name[0]) {
+    host = from->h_name;
+    nameinfo = TRUE;
+  }
+#endif /* IPv6 support */
 
   return server_make_connection(new_sock,
-				(from ? from->h_name
-				 : inet_ntoa(fromend.sockaddr_in.sin_addr)),
-				inet_ntoa(fromend.sockaddr_in.sin_addr));
+                                (nameinfo ? host : dst), dst);
 }
 
 /********************************************************************
@@ -850,7 +929,7 @@ int server_make_connection(int new_sock, const char *client_addr, const char *cl
 {
   int i;
 
-  my_nonblock(new_sock);
+  fc_nonblock(new_sock);
 
   for(i=0; i<MAX_NUM_CONNECTIONS; i++) {
     struct connection *pconn = &connections[i];
@@ -858,20 +937,18 @@ int server_make_connection(int new_sock, const char *client_addr, const char *cl
       connection_common_init(pconn);
       pconn->sock = new_sock;
       pconn->observer = FALSE;
-      pconn->player = NULL;
+      pconn->playing = NULL;
       pconn->capability[0] = '\0';
       pconn->access_level = access_level_for_next_connection();
       pconn->delayed_disconnect = FALSE;
       pconn->notify_of_writable_data = NULL;
-      pconn->is_server = TRUE;
       pconn->server.currently_processed_request_id = 0;
       pconn->server.last_request_id_seen = 0;
       pconn->server.auth_tries = 0;
       pconn->server.auth_settime = 0;
       pconn->server.status = AS_NOT_ESTABLISHED;
-      pconn->server.ping_timers =
-	  fc_malloc(sizeof(*pconn->server.ping_timers));
       pconn->server.ping_timers = timer_list_new();
+      pconn->server.granted_access_level = pconn->access_level;
       pconn->ping_time = -1.0;
       pconn->incoming_packet_notify = NULL;
       pconn->outgoing_packet_notify = NULL;
@@ -890,6 +967,7 @@ int server_make_connection(int new_sock, const char *client_addr, const char *cl
   }
 
   freelog(LOG_ERROR, "maximum number of connections reached");
+  fc_closesocket(new_sock);
   return -1;
 }
 
@@ -900,77 +978,200 @@ int server_make_connection(int new_sock, const char *client_addr, const char *cl
 int server_open_socket(void)
 {
   /* setup socket address */
-  union my_sockaddr src;
-  union my_sockaddr addr;
-  struct ip_mreq mreq;
-  const char *group;
-  int opt;
+  union fc_sockaddr names[2];
+  union fc_sockaddr addr;
+  struct ip_mreq mreq4;
+  const char *cause, *group;
+  int i, j, name_count, on, s;
+  int lan_family;
 
-  /* Create socket for client connections. */
-  if((sock = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
-    die("socket failed: %s", mystrerror());
-  }
+#ifdef IPV6_SUPPORT
+  struct ipv6_mreq mreq6;
+#endif
 
-  opt=1; 
-  if(setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, 
-		(char *)&opt, sizeof(opt)) == -1) {
-    freelog(LOG_ERROR, "SO_REUSEADDR failed: %s", mystrerror());
-  }
-
-  if (!net_lookup_service(srvarg.bind_addr, srvarg.port, &src)) {
-    freelog(LOG_ERROR, _("Server: bad address: [%s:%d]."),
+  /* Lookup addresses to bind. */
+  if (!net_lookup_service(srvarg.bind_addr, srvarg.port, &names[0], FALSE)) {
+    freelog(LOG_FATAL, _("Server: bad address: <%s:%d>."),
 	    srvarg.bind_addr, srvarg.port);
     exit(EXIT_FAILURE);
   }
+  name_count = 1;
+#ifdef IPV6_SUPPORT
+  if (names[0].saddr.sa_family == AF_INET6) {
+    /* net_lookup_service() prefers IPv6 address.
+     * Check if there is also IPv4 address.
+     * TODO: This would be easier using getaddrinfo() */
+    if (net_lookup_service(srvarg.bind_addr, srvarg.port,
+                           &names[1], TRUE /* force IPv4 */)) {
+      name_count = 2;
+    }
+  }
+#endif
 
-  if(bind(sock, &src.sockaddr, sizeof (src)) == -1) {
-    freelog(LOG_FATAL, "bind failed: %s", mystrerror());
+  cause = NULL;
+  on = 1;
+
+  /* Loop to create sockets, bind, listen. */
+  listen_socks = fc_calloc(name_count, sizeof(listen_socks[0]));
+  listen_count = 0;
+  for (i = 0; i < name_count; i++) {
+
+    /* Create socket for client connections. */
+    s = socket(names[i].saddr.sa_family, SOCK_STREAM, 0);
+    if (s == -1) {
+      /* Probably EAFNOSUPPORT or EPROTONOSUPPORT.
+       * Kernel might have disabled AF_INET6. */
+      cause = "socket";
+      continue;
+    }
+
+    if (setsockopt(s, SOL_SOCKET, SO_REUSEADDR, 
+                   (char *)&on, sizeof(on)) == -1) {
+      freelog(LOG_ERROR, "setsockopt SO_REUSEADDR failed: %s",
+              fc_strerror(fc_get_errno()));
+      sockaddr_debug(&names[i]);
+    }
+
+    /* AF_INET6 sockets should use IPv6 only,
+     * without stealing IPv4 from AF_INET sockets. */
+#ifdef IPV6_SUPPORT
+    if (names[i].saddr.sa_family == AF_INET6) {
+#ifdef IPV6_V6ONLY
+      if (setsockopt(s, IPPROTO_IPV6, IPV6_V6ONLY,
+                     (char *)&on, sizeof(on)) == -1) {
+        freelog(LOG_ERROR, "setsockopt IPV6_V6ONLY failed: %s",
+                fc_strerror(fc_get_errno()));
+        sockaddr_debug(&names[i]);
+      }
+#endif
+    }
+#endif
+
+    if (bind(s, &names[i].saddr, sockaddr_size(&names[i])) == -1) {
+      cause = "bind";
+
+      if (fc_get_errno() == EADDRNOTAVAIL) {
+        /* Close only this socket. This address is not available.
+         * This can happen with the IPv6 wildcard address if this
+         * machine has no IPv6 interfaces. */
+        fc_closesocket(s);
+        continue;
+      } else {
+        /* Close all sockets. Another program might have bound to
+         * one of our addresses, and might hijack some of our
+         * connections. */
+        fc_closesocket(s);
+        for (j = 0; j < listen_count; j++) {
+          fc_closesocket(listen_socks[j]);
+        }
+        listen_count = 0;
+        break;
+      }
+    }
+
+    if (listen(s, MAX_NUM_CONNECTIONS) == -1) {
+      cause = "listen";
+      fc_closesocket(s);
+      continue;
+    }
+
+    listen_socks[listen_count] = s;
+    listen_count++;
+  }
+
+  if (listen_count == 0) {
+    freelog(LOG_FATAL, "%s failed: %s", cause, fc_strerror(fc_get_errno()));
+    for (i = 0; i < name_count; i++) {
+      sockaddr_debug(&names[i]);
+    }
     exit(EXIT_FAILURE);
   }
 
-  if(listen(sock, MAX_NUM_CONNECTIONS) == -1) {
-    freelog(LOG_FATAL, "listen failed: %s", mystrerror());
-    exit(EXIT_FAILURE);
+  if (srvarg.announce == ANNOUNCE_NONE) {
+    return 0;
+  }
+
+#ifdef IPV6_SUPPORT
+  if (srvarg.announce == ANNOUNCE_IPV6) {
+    lan_family = AF_INET6;
+  } else
+#endif /* IPV6 used */
+  {
+    lan_family = AF_INET;
   }
 
   /* Create socket for server LAN announcements */
-  if ((socklan = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
-     freelog(LOG_ERROR, "socket failed: %s", mystrerror());
+  if ((socklan = socket(lan_family, SOCK_DGRAM, 0)) < 0) {
+    freelog(LOG_ERROR, "socket failed: %s", fc_strerror(fc_get_errno()));
   }
 
   if (setsockopt(socklan, SOL_SOCKET, SO_REUSEADDR,
-                 (char *)&opt, sizeof(opt)) == -1) {
-    freelog(LOG_ERROR, "SO_REUSEADDR failed: %s", mystrerror());
+                 (char *)&on, sizeof(on)) == -1) {
+    freelog(LOG_ERROR, "SO_REUSEADDR failed: %s", fc_strerror(fc_get_errno()));
   }
 
-  my_nonblock(socklan);
+  fc_nonblock(socklan);
 
-  group = get_multicast_group();
+  group = get_multicast_group(srvarg.announce == ANNOUNCE_IPV6);
 
   memset(&addr, 0, sizeof(addr));
-  addr.sockaddr_in.sin_family = AF_INET;
-  addr.sockaddr_in.sin_addr.s_addr = htonl(INADDR_ANY);
-  addr.sockaddr_in.sin_port = htons(SERVER_LAN_PORT);
 
-  if (bind(socklan, &addr.sockaddr, sizeof(addr)) < 0) {
-    freelog(LOG_ERROR, "bind failed: %s", mystrerror());
+  addr.saddr.sa_family = lan_family;
+
+#ifdef IPV6_SUPPORT
+  if (addr.saddr.sa_family == AF_INET6) {
+    addr.saddr_in6.sin6_family = AF_INET6;
+    addr.saddr_in6.sin6_port = htons(SERVER_LAN_PORT);
+    addr.saddr_in6.sin6_addr = in6addr_any;
+  } else
+#endif /* IPv6 support */
+  {
+    addr.saddr_in4.sin_family = AF_INET;
+    addr.saddr_in4.sin_port = htons(SERVER_LAN_PORT);
+    addr.saddr_in4.sin_addr.s_addr = htonl(INADDR_ANY);
   }
 
-  mreq.imr_multiaddr.s_addr = inet_addr(group);
-  mreq.imr_interface.s_addr = htonl(INADDR_ANY);
+  if (bind(socklan, &addr.saddr, sockaddr_size(&addr)) < 0) {
+    freelog(LOG_ERROR, "Lan bind failed: %s", fc_strerror(fc_get_errno()));
+  }
 
-  if (setsockopt(socklan, IPPROTO_IP, IP_ADD_MEMBERSHIP,
-                 (const char*)&mreq, sizeof(mreq)) < 0) {
-    freelog(LOG_ERROR, "setsockopt failed: %s", mystrerror());
+#ifndef IPV6_SUPPORT
+  {
+#ifdef HAVE_INET_ATON
+    inet_aton(group, &mreq4.imr_multiaddr);
+#else  /* HEVE_INET_ATON */
+    mreq4.imr_multiaddr.s_addr = inet_addr(group);
+#endif /* HAVE_INET_ATON */
+#else  /* IPv6 support */
+  if (addr.saddr.sa_family == AF_INET6) {
+    inet_pton(AF_INET6, group, &mreq6.ipv6mr_multiaddr.s6_addr);
+    mreq6.ipv6mr_interface = 0; /* TODO: Interface selection */
+    if (setsockopt(socklan, IPPROTO_IPV6, FC_IPV6_ADD_MEMBERSHIP,
+                   (const char*)&mreq6, sizeof(mreq6)) < 0) {
+      freelog(LOG_ERROR, "FC_IPV6_ADD_MEMBERSHIP (%s) failed: %s",
+              group, fc_strerror(fc_get_errno()));
+    }
+  } else {
+    inet_pton(AF_INET, group, &mreq4.imr_multiaddr.s_addr);
+#endif /* IPv6 support */
+    mreq4.imr_interface.s_addr = htonl(INADDR_ANY);
+
+    if (setsockopt(socklan, IPPROTO_IP, IP_ADD_MEMBERSHIP,
+                   (const char*)&mreq4, sizeof(mreq4)) < 0) {
+      freelog(LOG_ERROR, "IP_ADD_MEMBERSHIP (%s) failed: %s",
+              group, fc_strerror(fc_get_errno()));
+    }
   }
 
   close_socket_set_callback(close_socket_callback);
+
   return 0;
 }
 
 
 /********************************************************************
-...
+  Initialize connection related stuff. Attention: Logging is not
+  available within this functions!
 ********************************************************************/
 void init_connections(void)
 {
@@ -1014,6 +1215,9 @@ static void start_processing_request(struct connection *pconn,
 **************************************************************************/
 static void finish_processing_request(struct connection *pconn)
 {
+  if (!pconn || !pconn->used) {
+    return;
+  }
   assert(pconn->server.currently_processed_request_id);
   freelog(LOG_DEBUG, "finish processing packet %d from connection %d",
 	  pconn->server.currently_processed_request_id, pconn->id);
@@ -1042,7 +1246,7 @@ void handle_conn_pong(struct connection *pconn)
   struct timer *timer;
 
   if (timer_list_size(pconn->server.ping_timers) == 0) {
-    freelog(LOG_NORMAL, "got unexpected pong from %s",
+    freelog(LOG_ERROR, "got unexpected pong from %s",
 	    conn_description(pconn));
     return;
   }
@@ -1102,9 +1306,9 @@ static void get_lanserver_announcement(void)
   tv.tv_sec = 0;
   tv.tv_usec = 0;
 
-  while (my_select(socklan + 1, &readfs, NULL, &exceptfs, &tv) == -1) {
+  while (fc_select(socklan + 1, &readfs, NULL, &exceptfs, &tv) == -1) {
     if (errno != EINTR) {
-      freelog(LOG_ERROR, "select failed: %s", mystrerror());
+      freelog(LOG_ERROR, "select failed: %s", fc_strerror(fc_get_errno()));
       return;
     }
     /* EINTR can happen sometimes, especially when compiling with -pg.
@@ -1143,7 +1347,7 @@ static void send_lanserver_response(void)
   char players[256];
   char status[256];
   struct data_out dout;
-  union my_sockaddr addr;
+  union fc_sockaddr addr;
   int socksend, setting = 1;
   const char *group;
   size_t size;
@@ -1153,16 +1357,16 @@ static void send_lanserver_response(void)
 
   /* Create a socket to broadcast to client. */
   if ((socksend = socket(AF_INET,SOCK_DGRAM, 0)) < 0) {
-    freelog(LOG_ERROR, "socket failed: %s", mystrerror());
+    freelog(LOG_ERROR, "socket failed: %s", fc_strerror(fc_get_errno()));
     return;
   }
 
   /* Set the UDP Multicast group IP address of the packet. */
-  group = get_multicast_group();
+  group = get_multicast_group(srvarg.announce == ANNOUNCE_IPV6);
   memset(&addr, 0, sizeof(addr));
-  addr.sockaddr_in.sin_family = AF_INET;
-  addr.sockaddr_in.sin_addr.s_addr = inet_addr(group);
-  addr.sockaddr_in.sin_port = htons(SERVER_LAN_PORT + 1);
+  addr.saddr_in4.sin_family = AF_INET;
+  addr.saddr_in4.sin_addr.s_addr = inet_addr(group);
+  addr.saddr_in4.sin_port = htons(SERVER_LAN_PORT + 1);
 
 /* this setsockopt call fails on Windows 98, so we stick with the default
  * value of 1 on Windows, which should be fine in most cases */
@@ -1171,14 +1375,14 @@ static void send_lanserver_response(void)
   ttl = SERVER_LAN_TTL;
   if (setsockopt(socksend, IPPROTO_IP, IP_MULTICAST_TTL, 
                  (const char*)&ttl, sizeof(ttl))) {
-    freelog(LOG_ERROR, "setsockopt failed: %s", mystrerror());
+    freelog(LOG_ERROR, "setsockopt failed: %s", fc_strerror(fc_get_errno()));
     return;
   }
 #endif
 
   if (setsockopt(socksend, SOL_SOCKET, SO_BROADCAST, 
                  (const char*)&setting, sizeof(setting))) {
-    freelog(LOG_ERROR, "setsockopt failed: %s", mystrerror());
+    freelog(LOG_ERROR, "setsockopt failed: %s", fc_strerror(fc_get_errno()));
     return;
   }
 
@@ -1192,20 +1396,24 @@ static void send_lanserver_response(void)
 
   switch (server_state()) {
   case S_S_INITIAL:
-    my_snprintf(status, sizeof(status), "Pregame");
+    /* TRANS: Game state for local server */
+    my_snprintf(status, sizeof(status), _("Pregame"));
     break;
   case S_S_RUNNING:
-    my_snprintf(status, sizeof(status), "Running");
+    /* TRANS: Game state for local server */
+    my_snprintf(status, sizeof(status), _("Running"));
     break;
   case S_S_OVER:
-    my_snprintf(status, sizeof(status), "Game over");
+    /* TRANS: Game state for local server */
+    my_snprintf(status, sizeof(status), _("Game over"));
     break;
   case S_S_GENERATING_WAITING:
-    my_snprintf(status, sizeof(status), "Waiting");
+    /* TRANS: Game state for local server */
+    my_snprintf(status, sizeof(status), _("Waiting"));
   }
 
    my_snprintf(players, sizeof(players), "%d",
-               get_num_human_and_ai_players());
+               normal_player_count());
    my_snprintf(port, sizeof(port), "%d",
               srvarg.port );
 
@@ -1220,11 +1428,12 @@ static void send_lanserver_response(void)
   size = dio_output_used(&dout);
 
   /* Sending packet to client with the information gathered above. */
-  if (sendto(socksend, buffer,  size, 0, &addr.sockaddr,
-      sizeof(addr)) < 0) {
-    freelog(LOG_ERROR, "sendto failed: %s", mystrerror());
+  if (sendto(socksend, buffer,  size, 0, &addr.saddr,
+      sockaddr_size(&addr)) < 0) {
+    freelog(LOG_ERROR, "landserver response sendto failed: %s",
+	    fc_strerror(fc_get_errno()));
     return;
   }
 
-  my_closesocket(socksend);
+  fc_closesocket(socksend);
 }
