@@ -358,8 +358,7 @@ static bool check_include(struct inputfile *inf)
     len = strlen(include_prefix);
   }
   assert_sanity(inf);
-  if (inf->at_eof || inf->in_string || inf->cur_line.n <= len
-      || inf->cur_line_pos > 0) {
+  if (inf->in_string || inf->cur_line.n <= len || inf->cur_line_pos > 0) {
     return FALSE;
   }
   if (strncmp(inf->cur_line.str, include_prefix, len)!=0) {
@@ -469,28 +468,12 @@ static bool read_a_line(struct inputfile *inf)
     if (!ret) {
       /* fgets failed */
       inf->at_eof = TRUE;
-      if (pos != 0) {
-	inf_warn(inf, "missing newline at EOF, or failed read");
-	/* treat as simple EOF, ignoring last line: */
-	pos = 0;
-      }
-      line->str[0] = '\0';
-      line->n = 0;
       if (inf->in_string) {
-	/* Note: Don't allow multi-line strings to cross "include"
-	   boundaries */
-	inf_log(inf, LOG_ERROR, "Multi-line string went to end-of-file");
+        /* Note: Don't allow multi-line strings to cross "include"
+         * boundaries */
+        inf_log(inf, LOG_ERROR, "Multi-line string went to end-of-file");
         return FALSE;
       }
-      if (inf->included_from) {
-	/* Pop the include, and get next line from file above instead. */
-	struct inputfile *inc = inf->included_from;
-	inf_close_partial(inf);
-	*inf = *inc;		/* so the user pointer in still valid
-				   (and inf pointers in calling functions) */
-	free(inc);
-	return read_a_line(inf);
-      }
       break;
     }
     
@@ -507,16 +490,33 @@ static bool read_a_line(struct inputfile *inf)
     }
     astr_minsize(line, line->n*2);
   }
-  inf->line_num++;
-  inf->cur_line_pos = 0;
 
-  astr_minsize(&inf->copy_line, inf->cur_line.n + ((inf->cur_line.n == 0) ? 1 : 0));
-  strcpy(inf->copy_line.str, inf->cur_line.str);
+  if (!inf->at_eof) {
+    inf->line_num++;
+    inf->cur_line_pos = 0;
 
-  if (check_include(inf)) {
-    return read_a_line(inf);
+    astr_minsize(&inf->copy_line,
+                 inf->cur_line.n + ((inf->cur_line.n == 0) ? 1 : 0));
+    strcpy(inf->copy_line.str, inf->cur_line.str);
+
+    if (check_include(inf)) {
+      return read_a_line(inf);
+    }
+    return TRUE;
+  } else {
+    line->str[0] = '\0';
+    line->n = 0;
+    if (inf->included_from) {
+      /* Pop the include, and get next line from file above instead. */
+      struct inputfile *inc = inf->included_from;
+      inf_close_partial(inf);
+      *inf = *inc;    /* so the user pointer in still valid
+                       * (and inf pointers in calling functions) */
+      free(inc);
+      return read_a_line(inf);
+    }
+    return FALSE;
   }
-  return (!inf->at_eof);
 }
 
 /********************************************************************** 
@@ -577,9 +577,7 @@ static void inf_warn(struct inputfile *inf, const char *message)
 /********************************************************************** 
   ...
 ***********************************************************************/
-static const char *get_token(struct inputfile *inf,
-			     enum inf_token_type type,
-			     bool required)
+const char *inf_token(struct inputfile *inf, enum inf_token_type type)
 {
   const char *c;
   const char *name;
@@ -592,7 +590,7 @@ static const char *get_token(struct inputfile *inf,
   func = tok_tab[type].func;
   
   if (!func) {
-    freelog(LOG_NORMAL, "token type %d (%s) not supported yet", type, name);
+    freelog(LOG_ERROR, "token type %d (%s) not supported yet", type, name);
     c = NULL;
   } else {
     if (!have_line(inf))
@@ -603,27 +601,11 @@ static const char *get_token(struct inputfile *inf,
       c = func(inf);
     }
   }
-  if (c) {
-    if (INF_DEBUG_FOUND) {
-      freelog(LOG_DEBUG, "inputfile: found %s '%s'", name, inf->token.str);
-    }
-  } else if (required) {
-    freelog(LOG_FATAL, "Did not find token %s in %s line %d", 
-            name, inf->filename, inf->line_num);
-    exit(EXIT_FAILURE);
+  if (c && INF_DEBUG_FOUND) {
+    freelog(LOG_DEBUG, "inputfile: found %s '%s'", name, inf->token.str);
   }
   return c;
 }
-  
-const char *inf_token(struct inputfile *inf, enum inf_token_type type)
-{
-  return get_token(inf, type, FALSE);
-}
-
-const char *inf_token_required(struct inputfile *inf, enum inf_token_type type)
-{
-  return get_token(inf, type, TRUE);
-}
 
 /********************************************************************** 
   Read as many tokens of specified type as possible, discarding
