@@ -50,6 +50,7 @@
 #include <windows.h>	/* GetTempPath */
 #endif
 
+#include "fcintl.h"
 #include "log.h"
 #include "shared.h"		/* TRUE, FALSE */
 #include "support.h"
@@ -75,17 +76,21 @@ static void set_socket_errno(void)
       errno = WSAGetLastError();
       return;
     default:
-      freelog(LOG_ERROR, "Missing errno mapping for Winsock error #%d. "
-                         "Please report this message at %s.",
-                         WSAGetLastError(), BUG_URL);
+      freelog(LOG_ERROR,
+              "Missing errno mapping for Winsock error #%d.",
+              WSAGetLastError());
+      freelog(LOG_ERROR,
+              /* TRANS: No full stop after the URL, could cause confusion. */
+              _("Please report this message at %s"),
+              BUG_URL);
   }
 }
-#endif
+#endif /* HAVE_WINSOCK */
 
 /***************************************************************
   Connect a socket to an address
 ***************************************************************/
-int my_connect(int sockfd, const struct sockaddr *serv_addr, socklen_t addrlen)
+int fc_connect(int sockfd, const struct sockaddr *serv_addr, socklen_t addrlen)
 {
   int result;
   
@@ -103,7 +108,7 @@ int my_connect(int sockfd, const struct sockaddr *serv_addr, socklen_t addrlen)
 /***************************************************************
   Wait for a number of sockets to change status
 **************************************************************/
-int my_select(int n, fd_set *readfds, fd_set *writefds, fd_set *exceptfds,
+int fc_select(int n, fd_set *readfds, fd_set *writefds, fd_set *exceptfds,
               struct timeval *timeout)
 {
   int result;
@@ -122,7 +127,7 @@ int my_select(int n, fd_set *readfds, fd_set *writefds, fd_set *exceptfds,
 /***************************************************************
   Read from a socket.
 ***************************************************************/
-int my_readsocket(int sock, void *buf, size_t size)
+int fc_readsocket(int sock, void *buf, size_t size)
 {
   int result;
   
@@ -141,7 +146,7 @@ int my_readsocket(int sock, void *buf, size_t size)
 /***************************************************************
   Write to a socket.
 ***************************************************************/
-int my_writesocket(int sock, const void *buf, size_t size)
+int fc_writesocket(int sock, const void *buf, size_t size)
 {
   int result;
         
@@ -151,8 +156,12 @@ int my_writesocket(int sock, const void *buf, size_t size)
     set_socket_errno();
   }
 #else
+#  ifdef MSG_NOSIGNAL
+  result = send(sock, buf, size, MSG_NOSIGNAL);
+#  else
   result = write(sock, buf, size);
-#endif
+#  endif /* MSG_NOSIGNAL */
+#endif /* HAVE_WINSOCK */
 
   return result;
 }
@@ -160,7 +169,7 @@ int my_writesocket(int sock, const void *buf, size_t size)
 /***************************************************************
   Close a socket.
 ***************************************************************/
-void my_closesocket(int sock)
+void fc_closesocket(int sock)
 {
 #ifdef HAVE_WINSOCK
   closesocket(sock);
@@ -172,13 +181,13 @@ void my_closesocket(int sock)
 /***************************************************************
   Initialize network stuff.
 ***************************************************************/
-void my_init_network(void)
+void fc_init_network(void)
 {
 #ifdef HAVE_WINSOCK
   WSADATA wsa;
 
   if (WSAStartup(MAKEWORD(1, 1), &wsa) != 0) {
-    freelog(LOG_ERROR, "no usable WINSOCK.DLL: %s", mystrerror());
+    freelog(LOG_ERROR, "no usable WINSOCK.DLL: %s", fc_strerror(fc_get_errno()));
   }
 #endif
 
@@ -191,7 +200,7 @@ void my_init_network(void)
 /***************************************************************
   Shutdown network stuff.
 ***************************************************************/
-void my_shutdown_network(void)
+void fc_shutdown_network(void)
 {
 #ifdef HAVE_WINSOCK
   WSACleanup();
@@ -201,7 +210,7 @@ void my_shutdown_network(void)
 /***************************************************************
   Set socket to non-blocking.
 ***************************************************************/
-void my_nonblock(int sockfd)
+void fc_nonblock(int sockfd)
 {
 #ifdef NONBLOCKING_SOCKETS
 #ifdef HAVE_WINSOCK
@@ -212,20 +221,20 @@ void my_nonblock(int sockfd)
   int f_set;
 
   if ((f_set=fcntl(sockfd, F_GETFL)) == -1) {
-    freelog(LOG_ERROR, "fcntl F_GETFL failed: %s", mystrerror());
+    freelog(LOG_ERROR, "fcntl F_GETFL failed: %s", fc_strerror(fc_get_errno()));
   }
 
   f_set |= O_NONBLOCK;
 
   if (fcntl(sockfd, F_SETFL, f_set) == -1) {
-    freelog(LOG_ERROR, "fcntl F_SETFL failed: %s", mystrerror());
+    freelog(LOG_ERROR, "fcntl F_SETFL failed: %s", fc_strerror(fc_get_errno()));
   }
 #else
 #ifdef HAVE_IOCTL
   long value=1;
 
   if (ioctl(sockfd, FIONBIO, (char*)&value) == -1) {
-    freelog(LOG_ERROR, "ioctl failed: %s", mystrerror());
+    freelog(LOG_ERROR, "ioctl failed: %s", fc_strerror(fc_get_errno()));
   }
 #endif
 #endif
@@ -235,37 +244,145 @@ void my_nonblock(int sockfd)
 #endif
 }
 
+/***************************************************************************
+  Write information about socaddr to debug log.
+***************************************************************************/
+void sockaddr_debug(union fc_sockaddr *addr)
+{
+#ifdef IPV6_SUPPORT
+  char buf[INET6_ADDRSTRLEN] = "Unknown";
+
+  if (addr->saddr.sa_family == AF_INET6) { 
+    inet_ntop(AF_INET6, &addr->saddr_in6.sin6_addr, buf, INET6_ADDRSTRLEN);
+    freelog(LOG_DEBUG, "Host: %s, Port: %d (IPv6)",
+            buf, ntohs(addr->saddr_in6.sin6_port));
+  } else {
+    inet_ntop(AF_INET, &addr->saddr_in4.sin_addr, buf, INET_ADDRSTRLEN);
+    freelog(LOG_DEBUG, "Host: %s, Port: %d (IPv4)",
+            buf, ntohs(addr->saddr_in4.sin_port));
+  }
+#else  /* IPv6 support */
+  char *buf;
+
+  buf = inet_ntoa(addr->saddr_in4.sin_addr);
+
+  freelog(LOG_DEBUG, "Host: %s, Port: %d",
+          buf, ntohs(addr->saddr_in4.sin_port));
+#endif /* IPv6 support */
+}
+
+/***************************************************************************
+  Gets size of address to fc_sockaddr. IPv6/IPv4 must be selected before
+  calling this.
+***************************************************************************/
+int sockaddr_size(union fc_sockaddr *addr)
+{
+#ifdef IPV6_SUPPORT
+  if (addr->saddr.sa_family == AF_INET6) {
+    return sizeof(addr->saddr_in6);
+  } else
+#endif /* IPV6_SUPPORT */
+  {
+    return sizeof(addr->saddr_in4);
+  }
+}
+
+/***************************************************************************
+  Returns wether address is IPv6 address.
+***************************************************************************/
+bool sockaddr_ipv6(union fc_sockaddr *addr)
+{
+#ifdef IPV6_SUPPORT
+  if (addr->saddr.sa_family == AF_INET6) {
+    return TRUE;
+  } else
+#endif /* IPv6 support */
+  {
+    return FALSE;
+  }
+}
+
 /***************************************************************************
   Look up the service at hostname:port and fill in *sa.
 ***************************************************************************/
-bool net_lookup_service(const char *name, int port, union my_sockaddr *addr)
+bool net_lookup_service(const char *name, int port, union fc_sockaddr *addr,
+			bool force_ipv4)
 {
   struct hostent *hp;
-  struct sockaddr_in *sock = &addr->sockaddr_in;
+  struct sockaddr_in *sock4;
+#ifdef IPV6_SUPPORT
+  struct sockaddr_in6 *sock6;
+#endif /* IPv6 support */
 
-  sock->sin_family = AF_INET;
-  sock->sin_port = htons(port);
+  sock4 = &addr->saddr_in4;
 
-  if (!name) {
-    sock->sin_addr.s_addr = htonl(INADDR_ANY);
-    return TRUE;
+#ifdef IPV6_SUPPORT
+  sock6 = &addr->saddr_in6;
+
+    if (!force_ipv4) {
+    addr->saddr.sa_family = AF_INET6;
+    sock6->sin6_port = htons(port);
+
+    if (!name) {
+      sock6->sin6_addr = in6addr_any;
+      return TRUE;
+    }
+
+    if (inet_pton(AF_INET6, name, &sock6->sin6_addr)) {
+      return TRUE;
+    }
+    /* TODO: Replace gethostbyname2() with getaddrinfo() */
+    hp = gethostbyname2(name, AF_INET6);
+  } else
+#endif /* IPv6 support */
+  {
+    addr->saddr.sa_family = AF_INET;
+    sock4->sin_port = htons(port);
+
+    if (!name) {
+      sock4->sin_addr.s_addr = htonl(INADDR_ANY);
+      return TRUE;
+    }
   }
 
-#ifdef HAVE_INET_ATON
-  if (inet_aton(name, &sock->sin_addr) != 0) {
+#ifdef IPV6_SUPPORT
+  if (force_ipv4 || !hp || hp->h_addrtype != AF_INET6) {
+    /* Try to fallback to IPv4 resolution */
+    if (!force_ipv4) {
+      freelog(LOG_DEBUG, "Falling back to IPv4");
+    }
+    hp = gethostbyname2(name, AF_INET);
+    if (!hp || hp->h_addrtype != AF_INET) {
+      return FALSE;
+    }
+    addr->saddr.sa_family = AF_INET;
+    sock4->sin_port = htons(port);
+  }
+#else  /* IPV6 support */
+#if defined(HAVE_INET_ATON)
+  if (inet_aton(name, &sock4->sin_addr) != 0) {
     return TRUE;
   }
-#else
-  if ((sock->sin_addr.s_addr = inet_addr(name)) != INADDR_NONE) {
+#else  /* HAVE_INET_ATON */
+  if ((sock4->sin_addr.s_addr = inet_addr(name)) != INADDR_NONE) {
     return TRUE;
   }
-#endif
+#endif /* HAVE_INET_ATON */
   hp = gethostbyname(name);
   if (!hp || hp->h_addrtype != AF_INET) {
     return FALSE;
   }
+#endif /* IPv6 support */
+
+#ifdef IPV6_SUPPORT
+  if (addr->saddr.sa_family == AF_INET6) {
+    memcpy(&sock6->sin6_addr, hp->h_addr, hp->h_length);
+  } else
+#endif /* IPv6 support */
+  {
+    memcpy(&sock4->sin_addr, hp->h_addr, hp->h_length);
+  }
 
-  memcpy(&sock->sin_addr, hp->h_addr, hp->h_length);
   return TRUE;
 }
 
@@ -273,7 +390,7 @@ bool net_lookup_service(const char *name, int port, union my_sockaddr *addr)
   Writes buf to socket and returns the response in an fz_FILE.
   Use only on blocking sockets.
 *************************************************************************/
-fz_FILE *my_querysocket(int sock, void *buf, size_t size)
+fz_FILE *fc_querysocket(int sock, void *buf, size_t size)
 {
   FILE *fp;
 
@@ -284,7 +401,7 @@ fz_FILE *my_querysocket(int sock, void *buf, size_t size)
   }
   fflush(fp);
 
-  /* we don't use my_closesocket on sock here since when fp is closed
+  /* we don't use fc_closesocket on sock here since when fp is closed
    * sock will also be closed. fdopen doesn't dup the socket descriptor. */
 #else
   {
@@ -300,7 +417,7 @@ fz_FILE *my_querysocket(int sock, void *buf, size_t size)
       GetTempPath(sizeof(filename), filename);
       sz_strlcat(filename, "fctmp");
 
-      fp = fopen(filename, "w+b");
+      fp = fc_fopen(filename, "w+b");
     }
 #else
 
@@ -312,16 +429,16 @@ fz_FILE *my_querysocket(int sock, void *buf, size_t size)
       return NULL;
     }
 
-    my_writesocket(sock, buf, size);
+    fc_writesocket(sock, buf, size);
 
-    while ((n = my_readsocket(sock, tmp, sizeof(tmp))) > 0) {
+    while ((n = fc_readsocket(sock, tmp, sizeof(tmp))) > 0) {
       if (fwrite(tmp, 1, n, fp) != n) {
 	die("socket %d: write error", sock);
       }
     }
     fflush(fp);
 
-    my_closesocket(sock);
+    fc_closesocket(sock);
 
     rewind(fp);
   }
@@ -334,9 +451,11 @@ fz_FILE *my_querysocket(int sock, void *buf, size_t size)
   Returns a valid httpd server and port, plus the path to the resource
   at the url location.
 *************************************************************************/
-const char *my_lookup_httpd(char *server, int *port, const char *url)
+const char *fc_lookup_httpd(char *server, int *port, const char *url)
 {
   const char *purl, *str, *ppath, *pport;
+  const char *str2;
+  int chars_between = 0;
 
   if ((purl = getenv("http_proxy")) && purl[0] != '\0') {
     if (strncmp(purl, "http://", strlen("http://")) != 0) {
@@ -353,17 +472,29 @@ const char *my_lookup_httpd(char *server, int *port, const char *url)
 
   str += strlen("http://");
 
-  pport = strchr(str, ':');
-  ppath = strchr(str, '/');
+  if (*str == '[') {
+    /* Literal IPv6 address (RFC 2732) */
+    str++;
+    str2 = strchr(str, ']') + 1;
+    if (!str2) {
+      str2 = str + strlen(str);
+    }
+    chars_between = 1;
+  } else {
+    str2 = str;
+  }
+
+  pport = strchr(str2, ':');
+  ppath = strchr(str2, '/');
 
   /* snarf server. */
   server[0] = '\0';
 
   if (pport) {
-    strncat(server, str, MIN(MAX_LEN_ADDR, pport-str));
+    strncat(server, str, MIN(MAX_LEN_ADDR, pport-str-chars_between));
   } else {
     if (ppath) {
-      strncat(server, str, MIN(MAX_LEN_ADDR, ppath-str));
+      strncat(server, str, MIN(MAX_LEN_ADDR, ppath-str-chars_between));
     } else {
       strncat(server, str, MAX_LEN_ADDR);
     }
@@ -400,7 +531,7 @@ static bool is_url_safe(unsigned ch)
   URL-encode a string as per RFC 2396.
   Should work for all ASCII based charsets: including UTF-8.
 ***************************************************************/
-const char *my_url_encode(const char *txt)
+const char *fc_url_encode(const char *txt)
 {
   static char buf[2048];
   unsigned ch;
@@ -436,8 +567,8 @@ int find_next_free_port(int starting_port)
   int port, s = socket(AF_INET, SOCK_STREAM, 0);
 
   for (port = starting_port;; port++) {
-    union my_sockaddr tmp;
-    struct sockaddr_in *sock = &tmp.sockaddr_in;
+    union fc_sockaddr tmp;
+    struct sockaddr_in *sock = &tmp.saddr_in4;
 
     memset(&tmp, 0, sizeof(tmp));
 
@@ -445,12 +576,12 @@ int find_next_free_port(int starting_port)
     sock->sin_port = htons(port);
     sock->sin_addr.s_addr = htonl(INADDR_ANY);
 
-    if (bind(s, &tmp.sockaddr, sizeof(tmp.sockaddr)) == 0) {
+    if (bind(s, &tmp.saddr, sockaddr_size(&tmp)) == 0) {
       break;
     }
   }
 
-  my_closesocket(s);
+  fc_closesocket(s);
   
   return port;
 }
